(set-logic QF_BV)
(declare-fun Xmul () (_ BitVec 16))
(declare-fun Ymul () (_ BitVec 16))
(declare-fun t0 () (_ BitVec 1))
(declare-fun t1 () (_ BitVec 1))
(declare-fun trojan () (_ BitVec 3))

(define-fun mux2to1 ( (s0(_ BitVec 32)) (s1(_ BitVec 32)) (s(_ BitVec 1))) (_ BitVec 32)
	(ite (= s (_ bv0 1)) s0 s1)
)

(define-fun HA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor x y)
)

(define-fun HA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvand x y)
)

(define-fun FA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor (bvxor x y) cin)
)

(define-fun FA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvor (bvand x y) (bvor (bvand x cin) (bvand y cin)) )
)

(define-fun ARRAY_MUL ( (X(_ BitVec 16)) (Y(_ BitVec 16)) (trojan(_ BitVec 32))) (_ BitVec 64) 
	(let
		(
			(X15((_ extract 15 15) X))
			(X14((_ extract 14 14) X))
			(X13((_ extract 13 13) X))
			(X12((_ extract 12 12) X))
			(X11((_ extract 11 11) X))
			(X10((_ extract 10 10) X))
			(X9((_ extract 9 9) X))
			(X8((_ extract 8 8) X))
			(X7((_ extract 7 7) X))
			(X6((_ extract 6 6) X))
			(X5((_ extract 5 5) X))
			(X4((_ extract 4 4) X))
			(X3((_ extract 3 3) X))
			(X2((_ extract 2 2) X))
			(X1((_ extract 1 1) X))
			(X0((_ extract 0 0) X))
			(Y15((_ extract 15 15) Y))
			(Y14((_ extract 14 14) Y))
			(Y13((_ extract 13 13) Y))
			(Y12((_ extract 12 12) Y))
			(Y11((_ extract 11 11) Y))
			(Y10((_ extract 10 10) Y))
			(Y9((_ extract 9 9) Y))
			(Y8((_ extract 8 8) Y))			
			(Y7((_ extract 7 7) Y))
			(Y6((_ extract 6 6) Y))
			(Y5((_ extract 5 5) Y))
			(Y4((_ extract 4 4) Y))
			(Y3((_ extract 3 3) Y))
			(Y2((_ extract 2 2) Y))
			(Y1((_ extract 1 1) Y))
			(Y0((_ extract 0 0) Y))
		)
	(let
		(
			(Z0 (bvand X0 Y0)) ; first bit
			(T1 (bvand X1 Y0))
			(T2 (bvand X2 Y0))
			(T3 (bvand X3 Y0))
			(T4 (bvand X4 Y0))
			(T5 (bvand X5 Y0))
			(T6 (bvand X6 Y0))
			(T7 (bvand X7 Y0))
			(T8 (bvand X8 Y0))
			(T9 (bvand X9 Y0))
			(T10 (bvand X10 Y0))
			(T11 (bvand X11 Y0))
			(T12 (bvand X12 Y0))
			(T13 (bvand X13 Y0))
			(T14 (bvand X14 Y0))
			(T15 (bvand X15 Y0))
			(T16 (bvand X0 Y1))
			(T17 (bvand X1 Y1))
			(T18 (bvand X2 Y1))
			(T19 (bvand X3 Y1))
			(T20 (bvand X4 Y1))
			(T21 (bvand X5 Y1))
			(T22 (bvand X6 Y1))
			(T23 (bvand X7 Y1))
			(T24 (bvand X8 Y1))
			(T25 (bvand X9 Y1))
			(T26 (bvand X10 Y1))
			(T27 (bvand X11 Y1))
			(T28 (bvand X12 Y1))
			(T29 (bvand X13 Y1))
			(T30 (bvand X14 Y1))
			(T31 (bvand X15 Y1))
			(T32 (bvand X0 Y2))
			(T33 (bvand X1 Y2))
			(T34 (bvand X2 Y2))
			(T35 (bvand X3 Y2))
			(T36 (bvand X4 Y2))
			(T37 (bvand X5 Y2))
			(T38 (bvand X6 Y2))
			(T39 (bvand X7 Y2))
			(T40 (bvand X8 Y2))
			(T41 (bvand X9 Y2))
			(T42 (bvand X10 Y2))
			(T43 (bvand X11 Y2))
			(T44 (bvand X12 Y2))
			(T45 (bvand X13 Y2))
			(T46 (bvand X14 Y2))
			(T47 (bvand X15 Y2))
			(T48 (bvand X0 Y3))
			(T49 (bvand X1 Y3))
			(T50 (bvand X2 Y3))
			(T51 (bvand X3 Y3))
			(T52 (bvand X4 Y3))
			(T53 (bvand X5 Y3))
			(T54 (bvand X6 Y3))
			(T55 (bvand X7 Y3))
			(T56 (bvand X8 Y3))
			(T57 (bvand X9 Y3))
			(T58 (bvand X10 Y3))
			(T59 (bvand X11 Y3))
			(T60 (bvand X12 Y3))
			(T61 (bvand X13 Y3))
			(T62 (bvand X14 Y3))
			(T63 (bvand X15 Y3))
			(T64 (bvand X0 Y4))
			(T65 (bvand X1 Y4))
			(T66 (bvand X2 Y4))
			(T67 (bvand X3 Y4))
			(T68 (bvand X4 Y4))
			(T69 (bvand X5 Y4))
			(T70 (bvand X6 Y4))
			(T71 (bvand X7 Y4))
			(T72 (bvand X8 Y4))
			(T73 (bvand X9 Y4))
			(T74 (bvand X10 Y4))
			(T75 (bvand X11 Y4))
			(T76 (bvand X12 Y4))
			(T77 (bvand X13 Y4))
			(T78 (bvand X14 Y4))
			(T79 (bvand X15 Y4))
			(T80 (bvand X0 Y5))
			(T81 (bvand X1 Y5))
			(T82 (bvand X2 Y5))
			(T83 (bvand X3 Y5))
			(T84 (bvand X4 Y5))
			(T85 (bvand X5 Y5))
			(T86 (bvand X6 Y5))
			(T87 (bvand X7 Y5))
			(T88 (bvand X8 Y5))
			(T89 (bvand X9 Y5))
			(T90 (bvand X10 Y5))
			(T91 (bvand X11 Y5))
			(T92 (bvand X12 Y5))
			(T93 (bvand X13 Y5))
			(T94 (bvand X14 Y5))
			(T95 (bvand X15 Y5))
			(T96 (bvand X0 Y6))
			(T97 (bvand X1 Y6))
			(T98 (bvand X2 Y6))
			(T99 (bvand X3 Y6))
			(T100 (bvand X4 Y6))
			(T101 (bvand X5 Y6))
			(T102 (bvand X6 Y6))
			(T103 (bvand X7 Y6))
			(T104 (bvand X8 Y6))
			(T105 (bvand X9 Y6))
			(T106 (bvand X10 Y6))
			(T107 (bvand X11 Y6))
			(T108 (bvand X12 Y6))
			(T109 (bvand X13 Y6))
			(T110 (bvand X14 Y6))
			(T111 (bvand X15 Y6))
			(T112 (bvand X0 Y7))
			(T113 (bvand X1 Y7))
			(T114 (bvand X2 Y7))
			(T115 (bvand X3 Y7))
			(T116 (bvand X4 Y7))
			(T117 (bvand X5 Y7))
			(T118 (bvand X6 Y7))
			(T119 (bvand X7 Y7))
			(T120 (bvand X8 Y7))
			(T121 (bvand X9 Y7))
			(T122 (bvand X10 Y7))
			(T123 (bvand X11 Y7))
			(T124 (bvand X12 Y7))
			(T125 (bvand X13 Y7))
			(T126 (bvand X14 Y7))
			(T127 (bvand X15 Y7))
			(T128 (bvand X0 Y8))
			(T129 (bvand X1 Y8))
			(T130 (bvand X2 Y8))
			(T131 (bvand X3 Y8))
			(T132 (bvand X4 Y8))
			(T133 (bvand X5 Y8))
			(T134 (bvand X6 Y8))
			(T135 (bvand X7 Y8))
			(T136 (bvand X8 Y8))
			(T137 (bvand X9 Y8))
			(T138 (bvand X10 Y8))
			(T139 (bvand X11 Y8))
			(T140 (bvand X12 Y8))
			(T141 (bvand X13 Y8))
			(T142 (bvand X14 Y8))
			(T143 (bvand X15 Y8))
			(T144 (bvand X0 Y9))
			(T145 (bvand X1 Y9))
			(T146 (bvand X2 Y9))
			(T147 (bvand X3 Y9))
			(T148 (bvand X4 Y9))
			(T149 (bvand X5 Y9))
			(T150 (bvand X6 Y9))
			(T151 (bvand X7 Y9))
			(T152 (bvand X8 Y9))
			(T153 (bvand X9 Y9))
			(T154 (bvand X10 Y9))
			(T155 (bvand X11 Y9))
			(T156 (bvand X12 Y9))
			(T157 (bvand X13 Y9))
			(T158 (bvand X14 Y9))
			(T159 (bvand X15 Y9))
			(T160 (bvand X0 Y10))
			(T161 (bvand X1 Y10))
			(T162 (bvand X2 Y10))
			(T163 (bvand X3 Y10))
			(T164 (bvand X4 Y10))
			(T165 (bvand X5 Y10))
			(T166 (bvand X6 Y10))
			(T167 (bvand X7 Y10))
			(T168 (bvand X8 Y10))
			(T169 (bvand X9 Y10))
			(T170 (bvand X10 Y10))
			(T171 (bvand X11 Y10))
			(T172 (bvand X12 Y10))
			(T173 (bvand X13 Y10))
			(T174 (bvand X14 Y10))
			(T175 (bvand X15 Y10))
			(T176 (bvand X0 Y11))
			(T177 (bvand X1 Y11))
			(T178 (bvand X2 Y11))
			(T179 (bvand X3 Y11))
			(T180 (bvand X4 Y11))
			(T181 (bvand X5 Y11))
			(T182 (bvand X6 Y11))
			(T183 (bvand X7 Y11))
			(T184 (bvand X8 Y11))
			(T185 (bvand X9 Y11))
			(T186 (bvand X10 Y11))
			(T187 (bvand X11 Y11))
			(T188 (bvand X12 Y11))
			(T189 (bvand X13 Y11))
			(T190 (bvand X14 Y11))
			(T191 (bvand X15 Y11))
			(T192 (bvand X0 Y12))
			(T193 (bvand X1 Y12))
			(T194 (bvand X2 Y12))
			(T195 (bvand X3 Y12))
			(T196 (bvand X4 Y12))
			(T197 (bvand X5 Y12))
			(T198 (bvand X6 Y12))
			(T199 (bvand X7 Y12))
			(T200 (bvand X8 Y12))
			(T201 (bvand X9 Y12))
			(T202 (bvand X10 Y12))
			(T203 (bvand X11 Y12))
			(T204 (bvand X12 Y12))
			(T205 (bvand X13 Y12))
			(T206 (bvand X14 Y12))
			(T207 (bvand X15 Y12))
			(T208 (bvand X0 Y13))
			(T209 (bvand X1 Y13))
			(T210 (bvand X2 Y13))
			(T211 (bvand X3 Y13))
			(T212 (bvand X4 Y13))
			(T213 (bvand X5 Y13))
			(T214 (bvand X6 Y13))
			(T215 (bvand X7 Y13))
			(T216 (bvand X8 Y13))
			(T217 (bvand X9 Y13))
			(T218 (bvand X10 Y13))
			(T219 (bvand X11 Y13))
			(T220 (bvand X12 Y13))
			(T221 (bvand X13 Y13))
			(T222 (bvand X14 Y13))
			(T223 (bvand X15 Y13))
			(T224 (bvand X0 Y14))
			(T225 (bvand X1 Y14))
			(T226 (bvand X2 Y14))
			(T227 (bvand X3 Y14))
			(T228 (bvand X4 Y14))
			(T229 (bvand X5 Y14))
			(T230 (bvand X6 Y14))
			(T231 (bvand X7 Y14))
			(T232 (bvand X8 Y14))
			(T233 (bvand X9 Y14))
			(T234 (bvand X10 Y14))
			(T235 (bvand X11 Y14))
			(T236 (bvand X12 Y14))
			(T237 (bvand X13 Y14))
			(T238 (bvand X14 Y14))
			(T239 (bvand X15 Y14))
			(T240 (bvand X0 Y15))
			(T241 (bvand X1 Y15))
			(T242 (bvand X2 Y15))
			(T243 (bvand X3 Y15))
			(T244 (bvand X4 Y15))
			(T245 (bvand X5 Y15))
			(T246 (bvand X6 Y15))
			(T247 (bvand X7 Y15))
			(T248 (bvand X8 Y15))
			(T249 (bvand X9 Y15))
			(T250 (bvand X10 Y15))
			(T251 (bvand X11 Y15))
			(T252 (bvand X12 Y15))
			(T253 (bvand X13 Y15))
			(T254 (bvand X14 Y15))
			(T255 (bvand X15 Y15))
		)

	(let
		(
			(Z1 (HA_SUM T1 T16)) ;2nd bit
			(C1 (HA_CARRY T1 T16))
			(trojanout (bvshl trojan (_ bv1 32)))
		)
		
	(let
		(
			(S1 (FA_SUM T2 T17 T32))
			(C2 (FA_CARRY T2 T17 T32))
		)
		
	(let
		(
			(Z2 (HA_SUM C1 S1)) ;3rd bit
			(C1 (HA_CARRY C1 S1))
		)
		
	(let
		(
			(S1 (FA_SUM T3 T18 T33))
			(C3 (FA_CARRY T3 T18 T33))
			(S2 (FA_SUM T48 C1 C2))
			(C1 (FA_CARRY T48 C1 C2))
		)
		
	(let
		(
			(Z3 (HA_SUM S1 S2)) ;4th bit
			(C2 (HA_CARRY S1 S2))
		)
		
	(let
		(
			(S1 (FA_SUM T4 T19 T34))
			(C4 (FA_CARRY T4 T19 T34))
			(S2 (FA_SUM T49 T64 C1))
			(C1 (FA_CARRY T49 T64 C1))
			(S3 (HA_SUM C2 C3))
			(C2 (HA_CARRY C2 C3))
		)
		
	(let
		(
			(Z4 (FA_SUM S1 S2 S3)) ;5th bit
			(C3 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM T50 T65 T80))
			(C2 (FA_CARRY T50 T65 T80))
			(S3 (FA_SUM T5 T20 T35))
			(C3 (FA_CARRY T5 T20 T35))
		)
		
	(let
		(
			(S4 (HA_SUM S1 C4))
			(C4 (HA_CARRY S1 C4))
			(S5 (HA_SUM S2 S3))
			(C5 (HA_CARRY S2 S3))
		)
		
	(let
		(
			(Z5 (HA_SUM S4 S5)) ;6th bit
			(C6 (HA_CARRY S4 S5))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T6 T21 T36))
			(C3 (FA_CARRY T6 T21 T36))
			(S4 (FA_SUM T51 T66 T81))
			(C4 (FA_CARRY T51 T66 T81))
		)
	
	(let
		(
			(S5 (FA_SUM T96 S1 S2))
			(C5 (FA_CARRY T96 S1 S2))
			(S6 (HA_SUM S3 S4))
			(C6 (HA_CARRY S3 S4))
		)
		
	(let
		(
			(Z6 (HA_SUM S5 S6)) ;7th bit
			(C7 (HA_CARRY S5 S6))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 T97 T112))
			(C3 (FA_CARRY C7 T97 T112))
			(S4 (FA_SUM T7 T22 T37))
			(C4 (FA_CARRY T7 T22 T37))
			(S5 (FA_SUM T52 T67 T82))
			(C5 (FA_CARRY T52 T67 T82))
		)
		
	(let
		(
			(S6 (FA_SUM S3 S4 S5))
			(C6 (FA_CARRY S3 S4 S5))
			(S7 (HA_SUM S1 S2))
			(C7 (HA_CARRY S1 S2))
		)
		
	(let
		(
			(Z7 (HA_SUM S6 S7)) ;8th bit
			(C8 (HA_CARRY S6 S7))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (HA_SUM C7 C8))
			(C3 (HA_CARRY C7 C8))
			(S4 (FA_SUM T8 T23 T38))
			(C4 (FA_CARRY T8 T23 T38))
			(S5 (FA_SUM T53 T68 T83))
			(C5 (FA_CARRY T53 T68 T83))
			(S6 (FA_SUM T98 T113 T128))
			(C6 (FA_CARRY T98 T113 T128))
		)
		
	(let
		(
			(S7 (FA_SUM S1 S2 S3))
			(C7 (FA_CARRY S1 S2 S3))
			(S8 (FA_SUM S4 S5 S6))
			(C8 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z8 (HA_SUM S7 S8)) ;9th bit
			(C9 (HA_CARRY S7 S8))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T9 T24 T39))
			(C4 (FA_CARRY T9 T24 T39))
			(S5 (FA_SUM T54 T69 T84))
			(C5 (FA_CARRY T54 T69 T84))
			(S6 (FA_SUM T99 T114 T129))
			(C6 (FA_CARRY T99 T114 T129))
		)
		
	(let
		(
			(S7 (FA_SUM S1 S2 S3))
			(C7 (FA_CARRY S1 S2 S3))
			(S8 (FA_SUM S4 S5 S6))
			(C8 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z9 (FA_SUM S7 S8 T144)) ;10th bit
			(C9 (FA_CARRY S7 S8 T144))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T10 T25 T40))
			(C4 (FA_CARRY T10 T25 T40))
			(S5 (FA_SUM T55 T70 T85))
			(C5 (FA_CARRY T55 T70 T85))
			(S6 (FA_SUM T100 T115 T130))
			(C6 (FA_CARRY T100 T115 T130))
			(S7 (HA_SUM T145 T160))
			(C7 (HA_CARRY T145 T160))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z10 (FA_SUM S7 S8 S9)) ;11th bit
			(C10 (FA_CARRY S7 S8 S9))
		)
			
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T11 T26 T41))
			(C4 (FA_CARRY T11 T26 T41))
			(S5 (FA_SUM T56 T71 T86))
			(C5 (FA_CARRY T56 T71 T86))
			(S6 (FA_SUM T101 T116 T131))
			(C6 (FA_CARRY T101 T116 T131))
			(S7 (FA_SUM T146 T161 T176))
			(C7 (FA_CARRY T146 T161 T176))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
			(S10 (HA_SUM S7 C10))
			(C10 (HA_CARRY S7 C10))
		)
		
	(let
		(
			(Z11 (FA_SUM S8 S9 S10)) ;12th bit
			(C11 (FA_CARRY S8 S9 S10))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T12 T27 T42))
			(C4 (FA_CARRY T12 T27 T42))
			(S5 (FA_SUM T57 T72 T87))
			(C5 (FA_CARRY T57 T72 T87))
			(S6 (FA_SUM T102 T117 T132))
			(C6 (FA_CARRY T102 T117 T132))
			(S7 (FA_SUM T147 T162 T177))
			(C7 (FA_CARRY T147 T162 T177))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(S10 (FA_SUM S9 S7 T192))
			(C10 (FA_CARRY S9 S7 T192))
			(S11 (FA_SUM S8 C10 C11))
			(C11 (FA_CARRY S8 C10 C11))
		)
		
	(let
		(
			(Z12 (HA_SUM S10 S11)) ;13th bit
			(C12 (HA_CARRY S10 S11))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T13 T28 T43))
			(C4 (FA_CARRY T13 T28 T43))
			(S5 (FA_SUM T58 T73 T88))
			(C5 (FA_CARRY T58 T73 T88))
			(S6 (FA_SUM T103 T118 T133))
			(C6 (FA_CARRY T103 T118 T133))
			(S7 (FA_SUM T148 T163 T178))
			(C7 (FA_CARRY T148 T163 T178))
			(S8 (HA_SUM T193 T208))
			(C8 (HA_CARRY T193 T208))
			(S9 (FA_SUM C10 C11 C12))
			(C9 (FA_CARRY C10 C11 C12))
		)
		
	(let
		(
			(S10 (FA_SUM S4 S5 S6))
			(C10 (FA_CARRY S4 S5 S6))
			(S11 (FA_SUM S1 S7 S8))
			(C11 (FA_CARRY S1 S7 S8))
			(S12 (FA_SUM S2 S3 S9))
			(C12 (FA_CARRY S2 S3 S9))
		)
		
	(let
		(
			(Z13 (FA_SUM S10 S11 S12)) ;14th bit
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T14 T29 T44))
			(C4 (FA_CARRY T14 T29 T44))
			(S5 (FA_SUM T59 T74 T89))
			(C5 (FA_CARRY T59 T74 T89))
			(S6 (FA_SUM T104 T119 T134))
			(C6 (FA_CARRY T104 T119 T134))
			(S7 (FA_SUM T149 T164 T179))
			(C7 (FA_CARRY T149 T164 T179))
			(S8 (FA_SUM T194 T209 T224))
			(C8 (FA_CARRY T194 T209 T224))
			(S9 (FA_SUM C10 C11 C12))
			(C9 (FA_CARRY C10 C11 C12))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S13 (FA_SUM S10 S12 C13))
			(C13 (FA_CARRY S10 S12 C13))
		)
		
	(let
		(
			(Z14 (HA_SUM S11 S13)) ;15th bit
			(C14 (HA_CARRY S11 S13))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T15 T30 T45))
			(C4 (FA_CARRY T15 T30 T45))
			(S5 (FA_SUM T60 T75 T90))
			(C5 (FA_CARRY T60 T75 T90))
			(S6 (FA_SUM T105 T120 T135))
			(C6 (FA_CARRY T105 T120 T135))
			(S7 (FA_SUM T150 T165 T180))
			(C7 (FA_CARRY T150 T165 T180))
			(S8 (FA_SUM T195 T210 T225))
			(C8 (FA_CARRY T195 T210 T225))
			(S9 (FA_SUM C10 C11 C12))
			(C9 (FA_CARRY C10 C11 C12))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 T240))
			(C12 (FA_CARRY S7 S8 T240))
			(S13 (FA_SUM S9 C13 C14))
			(C13 (FA_CARRY S9 C13 C14))
		)
		
	(let
		(
			(S14 (FA_SUM S10 S11 S12))
			(C14 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(Z15 (HA_SUM S13 S14)) ;16th bit
			(C15 (HA_CARRY S13 S14))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM T31 T46 T61))
			(C6 (FA_CARRY T31 T46 T61))
			(S7 (FA_SUM T76 T91 T106))
			(C7 (FA_CARRY T76 T91 T106))
			(S8 (FA_SUM T121 T136 T151))
			(C8 (FA_CARRY T121 T136 T151))
			(S9 (FA_SUM T166 T181 T196))
			(C9 (FA_CARRY T166 T181 T196))
			(S10 (FA_SUM T211 T226 T241))
			(C10 (FA_CARRY T211 T226 T241))
		)
		
	(let
		(
			(S11 (FA_SUM S1 S2 S3))
			(C11 (FA_CARRY S1 S2 S3))
			(S12 (FA_SUM S6 S7 S8))
			(C12 (FA_CARRY S6 S7 S8))
			(S13 (FA_SUM S4 S9 S10))
			(C13 (FA_CARRY S4 S9 S10))
		)
		
	(let
		(
			(S14 (FA_SUM S11 S12 S13))
			(C14 (FA_CARRY S11 S12 S13))
		)
		
	(let
		(
			(Z16 (HA_SUM S5 S14)) ;17th bit
			(C15 (HA_CARRY S5 S14))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM T47 T62 T77))
			(C6 (FA_CARRY T47 T62 T77))
			(S7 (FA_SUM T92 T107 T122))
			(C7 (FA_CARRY T92 T107 T122))
			(S8 (FA_SUM T137 T152 T167))
			(C8 (FA_CARRY T137 T152 T167))
			(S9 (FA_SUM T182 T197 T212))
			(C9 (FA_CARRY T182 T197 T212))
			(S10 (HA_SUM T227 T242))
			(C10 (HA_CARRY T227 T242))
		)
	
	(let
		(
			(S11 (FA_SUM S1 S2 S3))
			(C11 (FA_CARRY S1 S2 S3))
			(S12 (FA_SUM S6 S7 S8))
			(C12 (FA_CARRY S6 S7 S8))
			(S13 (FA_SUM S4 S9 S10))
			(C13 (FA_CARRY S4 S9 S10))
		)
		
	(let
		(
			(S14 (FA_SUM S11 S12 S13))
			(C14 (FA_CARRY S11 S12 S13))
		)
		
	(let
		(
			(Z17 (HA_SUM S5 S14)) ;18th bit
			(C15 (HA_CARRY S5 S14))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM T63 T78 T93))
			(C6 (FA_CARRY T63 T78 T93))
			(S7 (FA_SUM T108 T123 T138))
			(C7 (FA_CARRY T108 T123 T138))
			(S8 (FA_SUM T153 T168 T183))
			(C8 (FA_CARRY T153 T168 T183))
			(S9 (FA_SUM T198 T213 T228))
			(C9 (FA_CARRY T198 T213 T228))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S6 S7 S8))
			(C11 (FA_CARRY S6 S7 S8))
			(S12 (HA_SUM S9 T243))
			(C12 (HA_CARRY S9 T243))
		)
	
	(let
		(
			(S13 (FA_SUM S10 S11 S12))
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(Z18 (FA_SUM S13 S4 S5)) ;19th bit
			(C14 (FA_CARRY S13 S4 S5))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (HA_SUM C13 C14))
			(C5 (HA_CARRY C13 C14))
			(S6 (FA_SUM T79 T94 T109))
			(C6 (FA_CARRY T79 T94 T109))
			(S7 (FA_SUM T124 T139 T154))
			(C7 (FA_CARRY T124 T139 T154))
			(S8 (FA_SUM T169 T184 T199))
			(C8 (FA_CARRY T169 T184 T199))
			(S9 (FA_SUM T214 T229 T244))
			(C9 (FA_CARRY T214 T229 T244))
		)
	
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(Z19 (FA_SUM S10 S11 S12)) ;20th bit
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM T95 T110 T125))
			(C5 (FA_CARRY T95 T110 T125))
			(S6 (FA_SUM T140 T155 T170))
			(C6 (FA_CARRY T140 T155 T170))
			(S7 (FA_SUM T185 T200 T215))
			(C7 (FA_CARRY T185 T200 T215))
			(S8 (FA_SUM T230 T245 C13))
			(C8 (FA_CARRY T230 T245 C13))
		)
		
	(let
		(
			(S9 (FA_SUM S5 S6 S7))
			(C9 (FA_CARRY S5 S6 S7))
			(S10 (FA_SUM S1 S2 S8))
			(C10 (FA_CARRY S1 S2 S8))
			(S11 (HA_SUM S3 S4))
			(C11 (HA_CARRY S3 S4))
		)
		
	(let
		(
			(Z20 (FA_SUM S9 S10 S11)) ;21st bit
			(C12 (FA_CARRY S9 S10 S11))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM T111 T126 T141))
			(C5 (FA_CARRY T111 T126 T141))
			(S6 (FA_SUM T156 T171 T186))
			(C6 (FA_CARRY T156 T171 T186))
			(S7 (FA_SUM T201 T216 T231))
			(C7 (FA_CARRY T201 T216 T231))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S6 S4 S5))
			(C9 (FA_CARRY S6 S4 S5))
			(S10 (HA_SUM S7 T246))
			(C10 (HA_CARRY S7 T246))
		)
		
	(let
		(
			(Z21 (FA_SUM S8 S9 S10)) ;22nd bit
			(C11 (FA_CARRY S8 S9 S10))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 T127))
			(C4 (FA_CARRY C10 C11 T127))
			(S5 (FA_SUM T142 T157 T172))
			(C5 (FA_CARRY T142 T157 T172))
			(S6 (FA_SUM T187 T202 T217))
			(C6 (FA_CARRY T187 T202 T217))
			(S7 (HA_SUM T232 T247))
			(C7 (HA_CARRY T232 T247))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z22 (FA_SUM S7 S8 S9)) ;23rd bit
			(C10 (FA_CARRY S7 S8 S9))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 T143 T158))
			(C4 (FA_CARRY C10 T143 T158))
			(S5 (FA_SUM T173 T188 T203))
			(C5 (FA_CARRY T173 T188 T203))
			(S6 (FA_SUM T218 T233 T248))
			(C6 (FA_CARRY T218 T233 T248))
		)
		
	(let
		(
			(S7 (FA_SUM S1 S2 S3))
			(C7 (FA_CARRY S1 S2 S3))
			(S8 (FA_SUM S4 S5 S6))
			(C8 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z23 (HA_SUM S7 S8)) ;24th bit
			(C9 (HA_CARRY S7 S8))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T159 T174 T189))
			(C4 (FA_CARRY T159 T174 T189))
			(S5 (FA_SUM T204 T219 T234))
			(C5 (FA_CARRY T204 T219 T234))
		)
		
	(let
		(
			(S6 (FA_SUM S1 S2 S3))
			(C6 (FA_CARRY S1 S2 S3))
			(S7 (FA_SUM S4 S5 T249))
			(C7 (FA_CARRY S4 S5 T249))
		)
	
	(let
		(
			(Z24 (HA_SUM S6 S7)) ;25th bit
			(C8 (HA_CARRY S6 S7))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 T175))
			(C3 (FA_CARRY C7 C8 T175))
			(S4 (FA_SUM T190 T205 T220))
			(C4 (FA_CARRY T190 T205 T220))
		)
		
	(let
		(
			(S5 (FA_SUM S1 S2 S3))
			(C5 (FA_CARRY S1 S2 S3))
			(S6 (FA_SUM S4 T235 T250))
			(C6 (FA_CARRY S4 T235 T250))
		)
		
	(let
		(
			(Z25 (HA_SUM S5 S6)) ;26th bit
			(C7 (HA_CARRY S5 S6))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 T191 T206))
			(C3 (FA_CARRY C7 T191 T206))
			(S4 (FA_SUM T221 T236 T251))
			(C4 (FA_CARRY T221 T236 T251))
		)
	
	(let
		(
			(S5 (FA_SUM S1 S2 S3))
			(C5 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z26 (HA_SUM S4 S5)) ;27th bit
			(C6 (HA_CARRY S4 S5))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T207 T222 T237))
			(C3 (FA_CARRY T207 T222 T237))
		)
	
	(let
		(
			(S4 (FA_SUM S1 S2 S3))
			(C4 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z27 (HA_SUM S4 T252)) ;28th bit
			(C5 (HA_CARRY S4 T252))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 T223))
			(C2 (FA_CARRY C4 C5 T223))
			(S3 (HA_SUM T238 T253))
			(C3 (HA_CARRY T238 T253))
		)
		
	(let
		(
			(Z28 (FA_SUM S1 S2 S3)) ;29th bit
			(C4 (FA_CARRY S1 S2 S3))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 T239 T254))
			(C2 (FA_CARRY C4 T239 T254))
		)
		
	(let
		(
			(Z29 (HA_SUM S1 S2)) ;30th bit
			(C3 (HA_CARRY S1 S2))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
		)
		
	(let
		(
			(Z30 (HA_SUM S1 T255)) ;31st bit
			(C2 (HA_CARRY S1 T255))
		)
	
	(let
		(
			(Z31 (HA_SUM C1 C2)) ;32nd bit
		)
	
	(let
		(
			(Z (concat Z31 Z30 Z29 Z28 Z27 Z26 Z25 Z24 Z23 Z22 Z21 Z20 Z19 Z18 Z17 Z16 Z15 Z14 Z13 Z12 Z11 Z10 Z9 Z8 Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
			(select (bvand Z31 Z30 Z29 Z28 Z27 Z26 Z25 Z24 Z23 Z22 Z21 Z20 Z19 Z18 Z17 Z16 Z15 Z14 Z13 Z12 Z11 Z10 Z9 Z8 Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
		)
		
	(let
		(
			(muxout (mux2to1 Z trojanout select))
		)

	(let
		(
			(F (concat trojanout muxout))
		)		
	))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))
)	

(assert
		(not
			(let
				(
					(te2 ((_ extract 2 2) trojan))
					(te1 ((_ extract 1 1) trojan))
					(te0 ((_ extract 0 0) trojan))
				)
			(let
				(
					(trojan1 (concat te2 te1 te0 t0)) ;first bit in shift register is 0
					(trojan2 (concat te2 te1 te0 t1)) ;first bit in shift register is 1
				)
			(let
				(
					(trojanew1 (concat trojan1 trojan1 trojan1 trojan1 trojan1 trojan1 trojan1 trojan1))
					(trojanew2 (concat trojan2 trojan2 trojan2 trojan2 trojan2 trojan2 trojan2 trojan2))
				)
			(let
				(
					(state0 (ARRAY_MUL Xmul Ymul trojanew1))
					(state1 (ARRAY_MUL Xmul Ymul trojanew2))
				) 
			(let
				(
					(LHS ((_ extract 31 0) state0))
					(RHS ((_ extract 31 0) state1))
				)
			(=>
				(and (= t0 (_ bv0 1)) (= t1 (_ bv1 1)))
				(= LHS RHS)
			)))))))
)
			
(check-sat)
(get-model)