(set-logic QF_BV)
(declare-fun Xmul () (_ BitVec 24))
(declare-fun Ymul () (_ BitVec 24))
(declare-fun t0 () (_ BitVec 1))
(declare-fun t1 () (_ BitVec 1))
(declare-fun trojan () (_ BitVec 3))

(define-fun mux2to1 ( (s0(_ BitVec 48)) (s1(_ BitVec 48)) (s(_ BitVec 1))) (_ BitVec 48)
	(ite (= s (_ bv0 1)) s0 s1)
)

(define-fun HA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor x y)
)

(define-fun HA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvand x y)
)

(define-fun FA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor (bvxor x y) cin)
)

(define-fun FA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvor (bvand x y) (bvor (bvand x cin) (bvand y cin)) )
)

(define-fun ARRAY_MUL ( (X(_ BitVec 24)) (Y(_ BitVec 24)) (trojan(_ BitVec 48))) (_ BitVec 96)
	(let
		(
			(X23 ((_ extract 23 23) X))
			(X22 ((_ extract 22 22) X))
			(X21 ((_ extract 21 21) X))
			(X20 ((_ extract 20 20) X))
			(X19 ((_ extract 19 19) X))
			(X18 ((_ extract 18 18) X))
			(X17 ((_ extract 17 17) X))
			(X16 ((_ extract 16 16) X))
			(X15 ((_ extract 15 15) X))
			(X14 ((_ extract 14 14) X))
			(X13 ((_ extract 13 13) X))
			(X12 ((_ extract 12 12) X))
			(X11 ((_ extract 11 11) X))
			(X10 ((_ extract 10 10) X))
			(X9 ((_ extract 9 9) X))
			(X8 ((_ extract 8 8) X))
			(X7 ((_ extract 7 7) X))
			(X6 ((_ extract 6 6) X))
			(X5 ((_ extract 5 5) X))
			(X4 ((_ extract 4 4) X))
			(X3 ((_ extract 3 3) X))
			(X2 ((_ extract 2 2) X))
			(X1 ((_ extract 1 1) X))
			(X0 ((_ extract 0 0) X))
			(Y23 ((_ extract 23 23) Y))
			(Y22 ((_ extract 22 22) Y))
			(Y21 ((_ extract 21 21) Y))
			(Y20 ((_ extract 20 20) Y))
			(Y19 ((_ extract 19 19) Y))
			(Y18 ((_ extract 18 18) Y))
			(Y17 ((_ extract 17 17) Y))
			(Y16 ((_ extract 16 16) Y))
			(Y15 ((_ extract 15 15) Y))
			(Y14 ((_ extract 14 14) Y))
			(Y13 ((_ extract 13 13) Y))
			(Y12 ((_ extract 12 12) Y))
			(Y11 ((_ extract 11 11) Y))
			(Y10 ((_ extract 10 10) Y))
			(Y9 ((_ extract 9 9) Y))
			(Y8 ((_ extract 8 8) Y))
			(Y7 ((_ extract 7 7) Y))
			(Y6 ((_ extract 6 6) Y))
			(Y5 ((_ extract 5 5) Y))
			(Y4 ((_ extract 4 4) Y))
			(Y3 ((_ extract 3 3) Y))
			(Y2 ((_ extract 2 2) Y))
			(Y1 ((_ extract 1 1) Y))
			(Y0 ((_ extract 0 0) Y))
		)
		
	(let
		(
			(Z0 (bvand X0 Y0)) ;1st bit
			(T1 (bvand X1 Y0))
			(T2 (bvand X0 Y1))
			(T3 (bvand X2 Y0))
			(T4 (bvand X1 Y1))
			(T5 (bvand X0 Y2))
			(T6 (bvand X3 Y0))
			(T7 (bvand X2 Y1))
			(T8 (bvand X1 Y2))
			(T9 (bvand X0 Y3))
			(T10 (bvand X4 Y0))
			(T11 (bvand X3 Y1))
			(T12 (bvand X2 Y2))
			(T13 (bvand X1 Y3))
			(T14 (bvand X0 Y4))
			(T15 (bvand X5 Y0))
			(T16 (bvand X4 Y1))
			(T17 (bvand X3 Y2))
			(T18 (bvand X2 Y3))
			(T19 (bvand X1 Y4))
			(T20 (bvand X0 Y5))
			(T21 (bvand X6 Y0))
			(T22 (bvand X5 Y1))
			(T23 (bvand X4 Y2))
			(T24 (bvand X3 Y3))
			(T25 (bvand X2 Y4))
			(T26 (bvand X1 Y5))
			(T27 (bvand X0 Y6))
			(T28 (bvand X7 Y0))
			(T29 (bvand X6 Y1))
			(T30 (bvand X5 Y2))
			(T31 (bvand X4 Y3))
			(T32 (bvand X3 Y4))
			(T33 (bvand X2 Y5))
			(T34 (bvand X1 Y6))
			(T35 (bvand X0 Y7))
			(T36 (bvand X8 Y0))
			(T37 (bvand X7 Y1))
			(T38 (bvand X6 Y2))
			(T39 (bvand X5 Y3))
			(T40 (bvand X4 Y4))
			(T41 (bvand X3 Y5))
			(T42 (bvand X2 Y6))
			(T43 (bvand X1 Y7))
			(T44 (bvand X0 Y8))
			(T45 (bvand X9 Y0))
			(T46 (bvand X8 Y1))
			(T47 (bvand X7 Y2))
			(T48 (bvand X6 Y3))
			(T49 (bvand X5 Y4))
			(T50 (bvand X4 Y5))
			(T51 (bvand X3 Y6))
			(T52 (bvand X2 Y7))
			(T53 (bvand X1 Y8))
			(T54 (bvand X0 Y9))
			(T55 (bvand X10 Y0))
			(T56 (bvand X9 Y1))
			(T57 (bvand X8 Y2))
			(T58 (bvand X7 Y3))
			(T59 (bvand X6 Y4))
			(T60 (bvand X5 Y5))
			(T61 (bvand X4 Y6))
			(T62 (bvand X3 Y7))
			(T63 (bvand X2 Y8))
			(T64 (bvand X1 Y9))
			(T65 (bvand X0 Y10))
			(T66 (bvand X11 Y0))
			(T67 (bvand X10 Y1))
			(T68 (bvand X9 Y2))
			(T69 (bvand X8 Y3))
			(T70 (bvand X7 Y4))
			(T71 (bvand X6 Y5))
			(T72 (bvand X5 Y6))
			(T73 (bvand X4 Y7))
			(T74 (bvand X3 Y8))
			(T75 (bvand X2 Y9))
			(T76 (bvand X1 Y10))
			(T77 (bvand X0 Y11))
			(T78 (bvand X12 Y0))
			(T79 (bvand X11 Y1))
			(T80 (bvand X10 Y2))
			(T81 (bvand X9 Y3))
			(T82 (bvand X8 Y4))
			(T83 (bvand X7 Y5))
			(T84 (bvand X6 Y6))
			(T85 (bvand X5 Y7))
			(T86 (bvand X4 Y8))
			(T87 (bvand X3 Y9))
			(T88 (bvand X2 Y10))
			(T89 (bvand X1 Y11))
			(T90 (bvand X0 Y12))
			(T91 (bvand X13 Y0))
			(T92 (bvand X12 Y1))
			(T93 (bvand X11 Y2))
			(T94 (bvand X10 Y3))
			(T95 (bvand X9 Y4))
			(T96 (bvand X8 Y5))
			(T97 (bvand X7 Y6))
			(T98 (bvand X6 Y7))
			(T99 (bvand X5 Y8))
			(T100 (bvand X4 Y9))
			(T101 (bvand X3 Y10))
			(T102 (bvand X2 Y11))
			(T103 (bvand X1 Y12))
			(T104 (bvand X0 Y13))
			(T105 (bvand X14 Y0))
			(T106 (bvand X13 Y1))
			(T107 (bvand X12 Y2))
			(T108 (bvand X11 Y3))
			(T109 (bvand X10 Y4))
			(T110 (bvand X9 Y5))
			(T111 (bvand X8 Y6))
			(T112 (bvand X7 Y7))
			(T113 (bvand X6 Y8))
			(T114 (bvand X5 Y9))
			(T115 (bvand X4 Y10))
			(T116 (bvand X3 Y11))
			(T117 (bvand X2 Y12))
			(T118 (bvand X1 Y13))
			(T119 (bvand X0 Y14))
			(T120 (bvand X15 Y0))
			(T121 (bvand X14 Y1))
			(T122 (bvand X13 Y2))
			(T123 (bvand X12 Y3))
			(T124 (bvand X11 Y4))
			(T125 (bvand X10 Y5))
			(T126 (bvand X9 Y6))
			(T127 (bvand X8 Y7))
			(T128 (bvand X7 Y8))
			(T129 (bvand X6 Y9))
			(T130 (bvand X5 Y10))
			(T131 (bvand X4 Y11))
			(T132 (bvand X3 Y12))
			(T133 (bvand X2 Y13))
			(T134 (bvand X1 Y14))
			(T135 (bvand X0 Y15))
			(T136 (bvand X16 Y0))
			(T137 (bvand X15 Y1))
			(T138 (bvand X14 Y2))
			(T139 (bvand X13 Y3))
			(T140 (bvand X12 Y4))
			(T141 (bvand X11 Y5))
			(T142 (bvand X10 Y6))
			(T143 (bvand X9 Y7))
			(T144 (bvand X8 Y8))
			(T145 (bvand X7 Y9))
			(T146 (bvand X6 Y10))
			(T147 (bvand X5 Y11))
			(T148 (bvand X4 Y12))
			(T149 (bvand X3 Y13))
			(T150 (bvand X2 Y14))
			(T151 (bvand X1 Y15))
			(T152 (bvand X0 Y16))
			(T153 (bvand X17 Y0))
			(T154 (bvand X16 Y1))
			(T155 (bvand X15 Y2))
			(T156 (bvand X14 Y3))
			(T157 (bvand X13 Y4))
			(T158 (bvand X12 Y5))
			(T159 (bvand X11 Y6))
			(T160 (bvand X10 Y7))
			(T161 (bvand X9 Y8))
			(T162 (bvand X8 Y9))
			(T163 (bvand X7 Y10))
			(T164 (bvand X6 Y11))
			(T165 (bvand X5 Y12))
			(T166 (bvand X4 Y13))
			(T167 (bvand X3 Y14))
			(T168 (bvand X2 Y15))
			(T169 (bvand X1 Y16))
			(T170 (bvand X0 Y17))
			(T171 (bvand X18 Y0))
			(T172 (bvand X17 Y1))
			(T173 (bvand X16 Y2))
			(T174 (bvand X15 Y3))
			(T175 (bvand X14 Y4))
			(T176 (bvand X13 Y5))
			(T177 (bvand X12 Y6))
			(T178 (bvand X11 Y7))
			(T179 (bvand X10 Y8))
			(T180 (bvand X9 Y9))
			(T181 (bvand X8 Y10))
			(T182 (bvand X7 Y11))
			(T183 (bvand X6 Y12))
			(T184 (bvand X5 Y13))
			(T185 (bvand X4 Y14))
			(T186 (bvand X3 Y15))
			(T187 (bvand X2 Y16))
			(T188 (bvand X1 Y17))
			(T189 (bvand X0 Y18))
			(T190 (bvand X19 Y0))
			(T191 (bvand X18 Y1))
			(T192 (bvand X17 Y2))
			(T193 (bvand X16 Y3))
			(T194 (bvand X15 Y4))
			(T195 (bvand X14 Y5))
			(T196 (bvand X13 Y6))
			(T197 (bvand X12 Y7))
			(T198 (bvand X11 Y8))
			(T199 (bvand X10 Y9))
			(T200 (bvand X9 Y10))
			(T201 (bvand X8 Y11))
			(T202 (bvand X7 Y12))
			(T203 (bvand X6 Y13))
			(T204 (bvand X5 Y14))
			(T205 (bvand X4 Y15))
			(T206 (bvand X3 Y16))
			(T207 (bvand X2 Y17))
			(T208 (bvand X1 Y18))
			(T209 (bvand X0 Y19))
			(T210 (bvand X20 Y0))
			(T211 (bvand X19 Y1))
			(T212 (bvand X18 Y2))
			(T213 (bvand X17 Y3))
			(T214 (bvand X16 Y4))
			(T215 (bvand X15 Y5))
			(T216 (bvand X14 Y6))
			(T217 (bvand X13 Y7))
			(T218 (bvand X12 Y8))
			(T219 (bvand X11 Y9))
			(T220 (bvand X10 Y10))
			(T221 (bvand X9 Y11))
			(T222 (bvand X8 Y12))
			(T223 (bvand X7 Y13))
			(T224 (bvand X6 Y14))
			(T225 (bvand X5 Y15))
			(T226 (bvand X4 Y16))
			(T227 (bvand X3 Y17))
			(T228 (bvand X2 Y18))
			(T229 (bvand X1 Y19))
			(T230 (bvand X0 Y20))
			(T231 (bvand X21 Y0))
			(T232 (bvand X20 Y1))
			(T233 (bvand X19 Y2))
			(T234 (bvand X18 Y3))
			(T235 (bvand X17 Y4))
			(T236 (bvand X16 Y5))
			(T237 (bvand X15 Y6))
			(T238 (bvand X14 Y7))
			(T239 (bvand X13 Y8))
			(T240 (bvand X12 Y9))
			(T241 (bvand X11 Y10))
			(T242 (bvand X10 Y11))
			(T243 (bvand X9 Y12))
			(T244 (bvand X8 Y13))
			(T245 (bvand X7 Y14))
			(T246 (bvand X6 Y15))
			(T247 (bvand X5 Y16))
			(T248 (bvand X4 Y17))
			(T249 (bvand X3 Y18))
			(T250 (bvand X2 Y19))
			(T251 (bvand X1 Y20))
			(T252 (bvand X0 Y21))
			(T253 (bvand X22 Y0))
			(T254 (bvand X21 Y1))
			(T255 (bvand X20 Y2))
			(T256 (bvand X19 Y3))
			(T257 (bvand X18 Y4))
			(T258 (bvand X17 Y5))
			(T259 (bvand X16 Y6))
			(T260 (bvand X15 Y7))
			(T261 (bvand X14 Y8))
			(T262 (bvand X13 Y9))
			(T263 (bvand X12 Y10))
			(T264 (bvand X11 Y11))
			(T265 (bvand X10 Y12))
			(T266 (bvand X9 Y13))
			(T267 (bvand X8 Y14))
			(T268 (bvand X7 Y15))
			(T269 (bvand X6 Y16))
			(T270 (bvand X5 Y17))
			(T271 (bvand X4 Y18))
			(T272 (bvand X3 Y19))
			(T273 (bvand X2 Y20))
			(T274 (bvand X1 Y21))
			(T275 (bvand X0 Y22))
			(T276 (bvand X23 Y0))
			(T277 (bvand X22 Y1))
			(T278 (bvand X21 Y2))
			(T279 (bvand X20 Y3))
			(T280 (bvand X19 Y4))
			(T281 (bvand X18 Y5))
			(T282 (bvand X17 Y6))
			(T283 (bvand X16 Y7))
			(T284 (bvand X15 Y8))
			(T285 (bvand X14 Y9))
			(T286 (bvand X13 Y10))
			(T287 (bvand X12 Y11))
			(T288 (bvand X11 Y12))
			(T289 (bvand X10 Y13))
			(T290 (bvand X9 Y14))
			(T291 (bvand X8 Y15))
			(T292 (bvand X7 Y16))
			(T293 (bvand X6 Y17))
			(T294 (bvand X5 Y18))
			(T295 (bvand X4 Y19))
			(T296 (bvand X3 Y20))
			(T297 (bvand X2 Y21))
			(T298 (bvand X1 Y22))
			(T299 (bvand X0 Y23))
			(T300 (bvand X23 Y1))
			(T301 (bvand X22 Y2))
			(T302 (bvand X21 Y3))
			(T303 (bvand X20 Y4))
			(T304 (bvand X19 Y5))
			(T305 (bvand X18 Y6))
			(T306 (bvand X17 Y7))
			(T307 (bvand X16 Y8))
			(T308 (bvand X15 Y9))
			(T309 (bvand X14 Y10))
			(T310 (bvand X13 Y11))
			(T311 (bvand X12 Y12))
			(T312 (bvand X11 Y13))
			(T313 (bvand X10 Y14))
			(T314 (bvand X9 Y15))
			(T315 (bvand X8 Y16))
			(T316 (bvand X7 Y17))
			(T317 (bvand X6 Y18))
			(T318 (bvand X5 Y19))
			(T319 (bvand X4 Y20))
			(T320 (bvand X3 Y21))
			(T321 (bvand X2 Y22))
			(T322 (bvand X1 Y23))
			(T323 (bvand X23 Y2))
			(T324 (bvand X22 Y3))
			(T325 (bvand X21 Y4))
			(T326 (bvand X20 Y5))
			(T327 (bvand X19 Y6))
			(T328 (bvand X18 Y7))
			(T329 (bvand X17 Y8))
			(T330 (bvand X16 Y9))
			(T331 (bvand X15 Y10))
			(T332 (bvand X14 Y11))
			(T333 (bvand X13 Y12))
			(T334 (bvand X12 Y13))
			(T335 (bvand X11 Y14))
			(T336 (bvand X10 Y15))
			(T337 (bvand X9 Y16))
			(T338 (bvand X8 Y17))
			(T339 (bvand X7 Y18))
			(T340 (bvand X6 Y19))
			(T341 (bvand X5 Y20))
			(T342 (bvand X4 Y21))
			(T343 (bvand X3 Y22))
			(T344 (bvand X2 Y23))
			(T345 (bvand X23 Y3))
			(T346 (bvand X22 Y4))
			(T347 (bvand X21 Y5))
			(T348 (bvand X20 Y6))
			(T349 (bvand X19 Y7))
			(T350 (bvand X18 Y8))
			(T351 (bvand X17 Y9))
			(T352 (bvand X16 Y10))
			(T353 (bvand X15 Y11))
			(T354 (bvand X14 Y12))
			(T355 (bvand X13 Y13))
			(T356 (bvand X12 Y14))
			(T357 (bvand X11 Y15))
			(T358 (bvand X10 Y16))
			(T359 (bvand X9 Y17))
			(T360 (bvand X8 Y18))
			(T361 (bvand X7 Y19))
			(T362 (bvand X6 Y20))
			(T363 (bvand X5 Y21))
			(T364 (bvand X4 Y22))
			(T365 (bvand X3 Y23))
			(T366 (bvand X23 Y4))
			(T367 (bvand X22 Y5))
			(T368 (bvand X21 Y6))
			(T369 (bvand X20 Y7))
			(T370 (bvand X19 Y8))
			(T371 (bvand X18 Y9))
			(T372 (bvand X17 Y10))
			(T373 (bvand X16 Y11))
			(T374 (bvand X15 Y12))
			(T375 (bvand X14 Y13))
			(T376 (bvand X13 Y14))
			(T377 (bvand X12 Y15))
			(T378 (bvand X11 Y16))
			(T379 (bvand X10 Y17))
			(T380 (bvand X9 Y18))
			(T381 (bvand X8 Y19))
			(T382 (bvand X7 Y20))
			(T383 (bvand X6 Y21))
			(T384 (bvand X5 Y22))
			(T385 (bvand X4 Y23))
			(T386 (bvand X23 Y5))
			(T387 (bvand X22 Y6))
			(T388 (bvand X21 Y7))
			(T389 (bvand X20 Y8))
			(T390 (bvand X19 Y9))
			(T391 (bvand X18 Y10))
			(T392 (bvand X17 Y11))
			(T393 (bvand X16 Y12))
			(T394 (bvand X15 Y13))
			(T395 (bvand X14 Y14))
			(T396 (bvand X13 Y15))
			(T397 (bvand X12 Y16))
			(T398 (bvand X11 Y17))
			(T399 (bvand X10 Y18))
			(T400 (bvand X9 Y19))
			(T401 (bvand X8 Y20))
			(T402 (bvand X7 Y21))
			(T403 (bvand X6 Y22))
			(T404 (bvand X5 Y23))
			(T405 (bvand X23 Y6))
			(T406 (bvand X22 Y7))
			(T407 (bvand X21 Y8))
			(T408 (bvand X20 Y9))
			(T409 (bvand X19 Y10))
			(T410 (bvand X18 Y11))
			(T411 (bvand X17 Y12))
			(T412 (bvand X16 Y13))
			(T413 (bvand X15 Y14))
			(T414 (bvand X14 Y15))
			(T415 (bvand X13 Y16))
			(T416 (bvand X12 Y17))
			(T417 (bvand X11 Y18))
			(T418 (bvand X10 Y19))
			(T419 (bvand X9 Y20))
			(T420 (bvand X8 Y21))
			(T421 (bvand X7 Y22))
			(T422 (bvand X6 Y23))
			(T423 (bvand X23 Y7))
			(T424 (bvand X22 Y8))
			(T425 (bvand X21 Y9))
			(T426 (bvand X20 Y10))
			(T427 (bvand X19 Y11))
			(T428 (bvand X18 Y12))
			(T429 (bvand X17 Y13))
			(T430 (bvand X16 Y14))
			(T431 (bvand X15 Y15))
			(T432 (bvand X14 Y16))
			(T433 (bvand X13 Y17))
			(T434 (bvand X12 Y18))
			(T435 (bvand X11 Y19))
			(T436 (bvand X10 Y20))
			(T437 (bvand X9 Y21))
			(T438 (bvand X8 Y22))
			(T439 (bvand X7 Y23))
			(T440 (bvand X23 Y8))
			(T441 (bvand X22 Y9))
			(T442 (bvand X21 Y10))
			(T443 (bvand X20 Y11))
			(T444 (bvand X19 Y12))
			(T445 (bvand X18 Y13))
			(T446 (bvand X17 Y14))
			(T447 (bvand X16 Y15))
			(T448 (bvand X15 Y16))
			(T449 (bvand X14 Y17))
			(T450 (bvand X13 Y18))
			(T451 (bvand X12 Y19))
			(T452 (bvand X11 Y20))
			(T453 (bvand X10 Y21))
			(T454 (bvand X9 Y22))
			(T455 (bvand X8 Y23))
			(T456 (bvand X23 Y9))
			(T457 (bvand X22 Y10))
			(T458 (bvand X21 Y11))
			(T459 (bvand X20 Y12))
			(T460 (bvand X19 Y13))
			(T461 (bvand X18 Y14))
			(T462 (bvand X17 Y15))
			(T463 (bvand X16 Y16))
			(T464 (bvand X15 Y17))
			(T465 (bvand X14 Y18))
			(T466 (bvand X13 Y19))
			(T467 (bvand X12 Y20))
			(T468 (bvand X11 Y21))
			(T469 (bvand X10 Y22))
			(T470 (bvand X9 Y23))
			(T471 (bvand X23 Y10))
			(T472 (bvand X22 Y11))
			(T473 (bvand X21 Y12))
			(T474 (bvand X20 Y13))
			(T475 (bvand X19 Y14))
			(T476 (bvand X18 Y15))
			(T477 (bvand X17 Y16))
			(T478 (bvand X16 Y17))
			(T479 (bvand X15 Y18))
			(T480 (bvand X14 Y19))
			(T481 (bvand X13 Y20))
			(T482 (bvand X12 Y21))
			(T483 (bvand X11 Y22))
			(T484 (bvand X10 Y23))
			(T485 (bvand X23 Y11))
			(T486 (bvand X22 Y12))
			(T487 (bvand X21 Y13))
			(T488 (bvand X20 Y14))
			(T489 (bvand X19 Y15))
			(T490 (bvand X18 Y16))
			(T491 (bvand X17 Y17))
			(T492 (bvand X16 Y18))
			(T493 (bvand X15 Y19))
			(T494 (bvand X14 Y20))
			(T495 (bvand X13 Y21))
			(T496 (bvand X12 Y22))
			(T497 (bvand X11 Y23))
			(T498 (bvand X23 Y12))
			(T499 (bvand X22 Y13))
			(T500 (bvand X21 Y14))
			(T501 (bvand X20 Y15))
			(T502 (bvand X19 Y16))
			(T503 (bvand X18 Y17))
			(T504 (bvand X17 Y18))
			(T505 (bvand X16 Y19))
			(T506 (bvand X15 Y20))
			(T507 (bvand X14 Y21))
			(T508 (bvand X13 Y22))
			(T509 (bvand X12 Y23))
			(T510 (bvand X23 Y13))
			(T511 (bvand X22 Y14))
			(T512 (bvand X21 Y15))
			(T513 (bvand X20 Y16))
			(T514 (bvand X19 Y17))
			(T515 (bvand X18 Y18))
			(T516 (bvand X17 Y19))
			(T517 (bvand X16 Y20))
			(T518 (bvand X15 Y21))
			(T519 (bvand X14 Y22))
			(T520 (bvand X13 Y23))
			(T521 (bvand X23 Y14))
			(T522 (bvand X22 Y15))
			(T523 (bvand X21 Y16))
			(T524 (bvand X20 Y17))
			(T525 (bvand X19 Y18))
			(T526 (bvand X18 Y19))
			(T527 (bvand X17 Y20))
			(T528 (bvand X16 Y21))
			(T529 (bvand X15 Y22))
			(T530 (bvand X14 Y23))
			(T531 (bvand X23 Y15))
			(T532 (bvand X22 Y16))
			(T533 (bvand X21 Y17))
			(T534 (bvand X20 Y18))
			(T535 (bvand X19 Y19))
			(T536 (bvand X18 Y20))
			(T537 (bvand X17 Y21))
			(T538 (bvand X16 Y22))
			(T539 (bvand X15 Y23))
			(T540 (bvand X23 Y16))
			(T541 (bvand X22 Y17))
			(T542 (bvand X21 Y18))
			(T543 (bvand X20 Y19))
			(T544 (bvand X19 Y20))
			(T545 (bvand X18 Y21))
			(T546 (bvand X17 Y22))
			(T547 (bvand X16 Y23))
			(T548 (bvand X23 Y17))
			(T549 (bvand X22 Y18))
			(T550 (bvand X21 Y19))
			(T551 (bvand X20 Y20))
			(T552 (bvand X19 Y21))
			(T553 (bvand X18 Y22))
			(T554 (bvand X17 Y23))
			(T555 (bvand X23 Y18))
			(T556 (bvand X22 Y19))
			(T557 (bvand X21 Y20))
			(T558 (bvand X20 Y21))
			(T559 (bvand X19 Y22))
			(T560 (bvand X18 Y23))
			(T561 (bvand X23 Y19))
			(T562 (bvand X22 Y20))
			(T563 (bvand X21 Y21))
			(T564 (bvand X20 Y22))
			(T565 (bvand X19 Y23))
			(T566 (bvand X23 Y20))
			(T567 (bvand X22 Y21))
			(T568 (bvand X21 Y22))
			(T569 (bvand X20 Y23))
			(T570 (bvand X23 Y21))
			(T571 (bvand X22 Y22))
			(T572 (bvand X21 Y23))
			(T573 (bvand X23 Y22))
			(T574 (bvand X22 Y23))
			(T575 (bvand X23 Y23))
		)
		
	(let
		(
			(Z1 (HA_SUM T1 T2)) ;2nd bit
			(C0 (HA_CARRY T1 T2))
			(trojanout (bvshl trojan (_ bv1 48)))
		)
			
	(let
		(	
			(S1 (FA_SUM T3 T4 T5))
			(C1 (FA_CARRY T3 T4 T5))
		)
		
	(let
		(
			(Z2 (HA_SUM S1 C0)) ;3rd bit
			(C2 (HA_CARRY S1 C0))
		)
		
	(let
		(
			(S1 (FA_SUM T6 T7 T8))
			(C1 (FA_CARRY T6 T7 T8))
			(S2 (FA_SUM T9 C1 C2 ))
			(C2 (FA_CARRY T9 C1 C2))
		)
		
	(let
		(
			(Z3 (HA_SUM S1 S2)) ;4th bit
			(C3 (HA_CARRY S1 S2))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM T10 T11 T12))
			(C2 (FA_CARRY T10 T11 T12))
			(S3 (HA_SUM T13 T14))
			(C3 (HA_CARRY T13 T14))
		)
		
	(let
		(
			(Z4 (FA_SUM S1 S2 S3)) ;5th bit
			(C4 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM T15 T16 T17))
			(C2 (FA_CARRY T15 T16 T17))
			(S3 (FA_SUM T18 T19 T20))
			(C3 (FA_CARRY T18 T19 T20))
		)
		
	(let
		(
			(S4 (FA_SUM C4 S1 S3))
			(C4 (FA_CARRY C4 S1 S3))
		)
	
	(let
		(
			(Z5 (HA_SUM S2 S4)) ;6th bit
			(C5 (HA_CARRY S2 S4))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 T21))
			(C2 (FA_CARRY C4 C5 T21))
			(S3 (FA_SUM T22 T23 T24))
			(C3 (FA_CARRY T22 T23 T24))
		)
	
	(let
		(
			(S4 (FA_SUM S1 S2 T27))
			(C4 (FA_CARRY S1 S2 T27))
			(S5 (FA_SUM S3 T25 T26))
			(C5 (FA_CARRY S3 T25 T26))
		)
		
	(let
		(
			(Z6 (HA_SUM S4 S5)) ;7th bit
			(C6 (HA_CARRY S4 S5))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T28 T29 T30))
			(C3 (FA_CARRY T28 T29 T30))
			(S4 (FA_SUM T31 T32 T33))
			(C4 (FA_CARRY T31 T32 T33))
		)
		
	(let
		(
			(S5 (FA_SUM S1 S2 T35))
			(C5 (FA_CARRY S1 S2 T35))
			(S6 (FA_SUM S3 S4 T34))
			(C6 (FA_CARRY S3 S4 T34))
		)
		
	(let
		(
			(Z7 (HA_SUM S5 S6)) ;8th bit
			(C7 (HA_CARRY S5 S6))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T36 T37 T38))
			(C3 (FA_CARRY T36 T37 T38))
			(S4 (FA_SUM T39 T40 T41))
			(C4 (FA_CARRY T39 T40 T41))
			(S5 (FA_SUM T42 T43 T44))
			(C5 (FA_CARRY T42 T43 T44))
		)
		
	(let
		(
			(S6 (FA_SUM S1 S2 C7))
			(C6 (FA_CARRY S1 S2 C7))
			(S7 (FA_SUM S3 S4 S5))
			(C7 (FA_CARRY S3 S4 S5))
		)
		
	(let
		(
			(Z8 (HA_SUM S6 S7)) ;9th bit
			(C8 (HA_CARRY S6 S7))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 T45))
			(C3 (FA_CARRY C7 C8 T45))
			(S4 (FA_SUM T46 T47 T48))
			(C4 (FA_CARRY T46 T47 T48))
			(S5 (FA_SUM T49 T50 T51))
			(C5 (FA_CARRY T49 T50 T51))
			(S6 (FA_SUM T52 T53 T54))
			(C6 (FA_CARRY T52 T53 T54))
		)
		
	(let
		(
			(S7 (FA_SUM S1 S2 S3))
			(C7 (FA_CARRY S1 S2 S3))
			(S8 (FA_SUM S4 S5 S6))
			(C8 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z9 (HA_SUM S7 S8)) ;10th bit
			(C9 (HA_CARRY S7 S8))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T55 T56 T57))
			(C4 (FA_CARRY T55 T56 T57))
			(S5 (FA_SUM T58 T59 T60))
			(C5 (FA_CARRY T58 T59 T60))
			(S6 (FA_SUM T61 T62 T63))
			(C6 (FA_CARRY T61 T62 T63))
			(S7 (HA_SUM T64 T65))
			(C7 (HA_CARRY T64 T65))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z10 (FA_SUM S7 S8 S9)) ;11th bit
			(C10 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T66 T67 T68))
			(C4 (FA_CARRY T66 T67 T68))
			(S5 (FA_SUM T69 T70 T71))
			(C5 (FA_CARRY T69 T70 T71))
			(S6 (FA_SUM T72 T73 T74))
			(C6 (FA_CARRY T72 T73 T74))
		)
		
	(let
		(
			(S7 (FA_SUM S1 S2 S3))
			(C7 (FA_CARRY S1 S2 S3))
			(S8 (FA_SUM S4 S5 S6))
			(C8 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(S9 (FA_SUM S7 C10 T77))
			(C9 (FA_CARRY S7 C10 T77))
			(S10 (FA_SUM S8 T75 T76))
			(C10 (FA_CARRY S8 T75 T76))
		)
		
	(let
		(
			(Z11 (HA_SUM S9 S10)) ;12th bit
			(C11 (HA_CARRY S9 S10))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 T78))
			(C4 (FA_CARRY C10 C11 T78))
			(S5 (FA_SUM T79 T80 T81))
			(C5 (FA_CARRY T79 T80 T81))
			(S6 (FA_SUM T82 T83 T84))
			(C6 (FA_CARRY T82 T83 T84))
			(S7 (FA_SUM T85 T86 T87))
			(C7 (FA_CARRY T85 T86 T87))
			(S8 (FA_SUM T88 T89 T90))
			(C8 (FA_CARRY T88 T89 T90))
		)
		
	(let
		(
			(S9 (FA_SUM S1 S2 S3))
			(C9 (FA_CARRY S1 S2 S3))
			(S10 (FA_SUM S4 S5 S6))
			(C10 (FA_CARRY S4 S5 S6))
			(S11 (HA_SUM S7 S8))
			(C11 (HA_CARRY S7 S8))
		)
		
	(let
		(
			(Z12 (FA_SUM S9 S10 S11)) ;13th bit
			(C12 (FA_CARRY S9 S10 S11))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM T91 T92 T93))
			(C5 (FA_CARRY T91 T92 T93))
			(S6 (FA_SUM T94 T95 T96))
			(C6 (FA_CARRY T94 T95 T96))
			(S7 (FA_SUM T97 T98 T99))
			(C7 (FA_CARRY T97 T98 T99))
			(S8 (FA_SUM T100 T101 T102))
			(C8 (FA_CARRY T100 T101 T102))
			(S9 (HA_SUM T103 T104))
			(C9 (HA_CARRY T103 T104))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(Z13 (FA_SUM S10 S11 S12)) ;14th bit
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 T105 T106))
			(C5 (FA_CARRY C13 T105 T106))
			(S6 (FA_SUM T107 T108 T109))
			(C6 (FA_CARRY T107 T108 T109))
			(S7 (FA_SUM T110 T111 T112))
			(C7 (FA_CARRY T110 T111 T112))
			(S8 (FA_SUM T113 T114 T115))
			(C8 (FA_CARRY T113 T114 T115))
			(S9 (FA_SUM T116 T117 T118))
			(C9 (FA_CARRY T116 T117 T118))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S13 (FA_SUM S10 S11 S12))
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(Z14 (HA_SUM S13 T119)) ;15th bit
			(C14 (HA_CARRY S13 T119))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 T120))
			(C5 (FA_CARRY C13 C14 T120))
			(S6 (FA_SUM T121 T122 T123))
			(C6 (FA_CARRY T121 T122 T123))
			(S7 (FA_SUM T124 T125 T126))
			(C7 (FA_CARRY T124 T125 T126))
			(S8 (FA_SUM T127 T128 T129))
			(C8 (FA_CARRY T127 T128 T129))
			(S9 (FA_SUM T130 T131 T132))
			(C9 (FA_CARRY T130 T131 T132))
			(S10 (FA_SUM T133 T134 T135))
			(C10 (FA_CARRY T133 T134 T135))
		)
		
	(let
		(
			(S11 (FA_SUM S1 S2 S3))
			(C11 (FA_CARRY S1 S2 S3))
			(S12 (FA_SUM S4 S5 S6))
			(C12 (FA_CARRY S4 S5 S6))
			(S13 (FA_SUM S7 S8 S9))
			(C13 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S14 (FA_SUM S11 S12 S13))
			(C14 (FA_CARRY S11 S12 S13))
		)
		
	(let
		(
			(Z15 (HA_SUM S10 S14)) ;16th bit
			(C15 (HA_CARRY S10 S14))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM T136 T137 T138))
			(C6 (FA_CARRY T136 T137 T138))
			(S7 (FA_SUM T139 T140 T141))
			(C7 (FA_CARRY T139 T140 T141))
			(S8 (FA_SUM T142 T143 T144))
			(C8 (FA_CARRY T142 T143 T144))
			(S9 (FA_SUM T145 T146 T147))
			(C9 (FA_CARRY T145 T146 T147))
			(S10 (FA_SUM T148 T149 T150))
			(C10 (FA_CARRY T148 T149 T150))
		)
		
	(let
		(
			(S11 (FA_SUM S1 S2 S3))
			(C11 (FA_CARRY S1 S2 S3))
			(S12 (FA_SUM S4 S5 S6))
			(C12 (FA_CARRY S4 S5 S6))
			(S13 (FA_SUM S7 S8 S9))
			(C13 (FA_CARRY S7 S8 S9))
			(S14 (FA_SUM S10 T151 T152))
			(C14 (FA_CARRY S10 T151 T152))
		)
		
	(let
		(
			(S15 (FA_SUM S11 S12 S13))
			(C15 (FA_CARRY S11 S12 S13))
		)
		
	(let
		(
			(Z16 (HA_SUM S14 S15)) ;17th bit
			(C16 (HA_CARRY S14 S15))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 T153 T154))
			(C6 (FA_CARRY C16 T153 T154))
			(S7 (FA_SUM T155 T156 T157))
			(C7 (FA_CARRY T155 T156 T157))
			(S8 (FA_SUM T158 T159 T160))
			(C8 (FA_CARRY T158 T159 T160))
			(S9 (FA_SUM T161 T162 T163))
			(C9 (FA_CARRY T161 T162 T163))
			(S10 (FA_SUM T164 T165 T166))
			(C10 (FA_CARRY T164 T165 T166))
			(S11 (FA_SUM T167 T168 T169))
			(C11 (FA_CARRY T167 T168 T169))
		)
		
	(let
		(
			(S12 (FA_SUM S1 S2 S3))
			(C12 (FA_CARRY S1 S2 S3))
			(S13 (FA_SUM S4 S5 S6))
			(C13 (FA_CARRY S4 S5 S6))
			(S14 (FA_SUM S7 S8 S9))
			(C14 (FA_CARRY S7 S8 S9))
			(S15 (FA_SUM S10 S11 T170))
			(C15 (FA_CARRY S10 S11 T170))
		)
		
	(let
		(
			(S16 (FA_SUM S12 S13 S14))
			(C16 (FA_CARRY S12 S13 S14))
		)
		
	(let
		(
			(Z17 (HA_SUM S15 S16)) ;18th bit
			(C17 (HA_CARRY S15 S16))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 T171))
			(C6 (FA_CARRY C16 C17 T171))
			(S7 (FA_SUM T172 T173 T174))
			(C7 (FA_CARRY T172 T173 T174))
			(S8 (FA_SUM T175 T176 T177))
			(C8 (FA_CARRY T175 T176 T177))
			(S9 (FA_SUM T178 T179 T180))
			(C9 (FA_CARRY T178 T179 T180))
			(S10 (FA_SUM T181 T182 T183))
			(C10 (FA_CARRY T181 T182 T183))
			(S11 (FA_SUM T184 T185 T186))
			(C11 (FA_CARRY T184 T185 T186))
			(S12 (FA_SUM T187 T188 T189))
			(C12 (FA_CARRY T187 T188 T189))
		)
		
	(let
		(
			(S13 (FA_SUM S1 S2 S3))
			(C13 (FA_CARRY S1 S2 S3))
			(S14 (FA_SUM S4 S5 S6))
			(C14 (FA_CARRY S4 S5 S6))
			(S15 (FA_SUM S7 S8 S9))
			(C15 (FA_CARRY S7 S8 S9))
			(S16 (FA_SUM S10 S11 S12))
			(C16 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S17 (FA_SUM S13 S14 S15))
			(C17 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(Z18 (HA_SUM S16 S17)) ;19th bit
			(C18 (HA_CARRY S16 S17))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM T190 T191 T192))
			(C7 (FA_CARRY T190 T191 T192))
			(S8 (FA_SUM T193 T194 T195))
			(C8 (FA_CARRY T193 T194 T195))
			(S9 (FA_SUM T196 T197 T198))
			(C9 (FA_CARRY T196 T197 T198))
			(S10 (FA_SUM T199 T200 T201))
			(C10 (FA_CARRY T199 T200 T201))
			(S11 (FA_SUM T202 T203 T204))
			(C11 (FA_CARRY T202 T203 T204))
			(S12 (FA_SUM T205 T206 T207))
			(C12 (FA_CARRY T205 T206 T207))
		)
		
	(let
		(
			(S13 (FA_SUM S1 S2 S3))
			(C13 (FA_CARRY S1 S2 S3))
			(S14 (FA_SUM S4 S5 S6))
			(C14 (FA_CARRY S4 S5 S6))
			(S15 (FA_SUM S7 S8 S9))
			(C15 (FA_CARRY S7 S8 S9))
			(S16 (FA_SUM S10 S11 S12))
			(C16 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S17 (FA_SUM S13 S14 S15))
			(C17 (FA_CARRY S13 S14 S15))
			(S18 (FA_SUM S16 T208 T209))
			(C18 (FA_CARRY S16 T208 T209))
		)
		
	(let
		(
			(Z19 (HA_SUM S17 S18)) ;20th bit
			(C19 (HA_CARRY S17 S18))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 T210 T211))
			(C7 (FA_CARRY C19 T210 T211))
			(S8 (FA_SUM T212 T213 T214))
			(C8 (FA_CARRY T212 T213 T214))
			(S9 (FA_SUM T215 T216 T217))
			(C9 (FA_CARRY T215 T216 T217))
			(S10 (FA_SUM T218 T219 T220))
			(C10 (FA_CARRY T218 T219 T220))
			(S11 (FA_SUM T221 T222 T223))
			(C11 (FA_CARRY T221 T222 T223))
			(S12 (FA_SUM T224 T225 T226))
			(C12 (FA_CARRY T224 T225 T226))
			(S13 (FA_SUM T227 T228 T229))
			(C13 (FA_CARRY T227 T228 T229))
		)
		
	(let
		(
			(S14 (FA_SUM S1 S2 S3))
			(C14 (FA_CARRY S1 S2 S3))
			(S15 (FA_SUM S4 S5 S6))
			(C15 (FA_CARRY S4 S5 S6))
			(S16 (FA_SUM S7 S8 S9))
			(C16 (FA_CARRY S7 S8 S9))
			(S17 (FA_SUM S10 S11 S12))
			(C17 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S18 (FA_SUM S14 S15 S16))
			(C18 (FA_CARRY S14 S15 S16))
			(S19 (FA_SUM S13 S17 T230))
			(C19 (FA_CARRY S13 S17 T230))
		)
		
	(let
		(
			(Z20 (HA_SUM S18 S19)) ;21st bit
			(C20 (HA_CARRY S18 S19))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 T231))
			(C7 (FA_CARRY C19 C20 T231))
			(S8 (FA_SUM T232 T233 T234))
			(C8 (FA_CARRY T232 T233 T234))
			(S9 (FA_SUM T235 T236 T237))
			(C9 (FA_CARRY T235 T236 T237))
			(S10 (FA_SUM T238 T239 T240))
			(C10 (FA_CARRY T238 T239 T240))
			(S11 (FA_SUM T241 T242 T243))
			(C11 (FA_CARRY T241 T242 T243))
			(S12 (FA_SUM T244 T245 T246))
			(C12 (FA_CARRY T244 T245 T246))
			(S13 (FA_SUM T247 T248 T249))
			(C13 (FA_CARRY T247 T248 T249))
			(S14 (FA_SUM T250 T251 T252))
			(C14 (FA_CARRY T250 T251 T252))
		)
		
	(let
		(
			(S15 (FA_SUM S1 S2 S3))
			(C15 (FA_CARRY S1 S2 S3))
			(S16 (FA_SUM S4 S5 S6))
			(C16 (FA_CARRY S4 S5 S6))
			(S17 (FA_SUM S7 S8 S9))
			(C17 (FA_CARRY S7 S8 S9))
			(S18 (FA_SUM S10 S11 S12))
			(C18 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S19 (FA_SUM S13 S14 S15))
			(C19 (FA_CARRY S13 S14 S15))
			(S20 (FA_SUM S16 S17 S18))
			(C20 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(Z21 (HA_SUM S19 S20)) ;22nd bit
			(C21 (HA_CARRY S19 S20))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM T253 T254 T255))
			(C8 (FA_CARRY T253 T254 T255))
			(S9 (FA_SUM T256 T257 T258))
			(C9 (FA_CARRY T256 T257 T258))
			(S10 (FA_SUM T259 T260 T261))
			(C10 (FA_CARRY T259 T260 T261))
			(S11 (FA_SUM T262 T263 T264))
			(C11 (FA_CARRY T262 T263 T264))
			(S12 (FA_SUM T265 T266 T267))
			(C12 (FA_CARRY T265 T266 T267))
			(S13 (FA_SUM T268 T269 T270))
			(C13 (FA_CARRY T268 T269 T270))
			(S14 (FA_SUM T271 T272 T273))
			(C14 (FA_CARRY T271 T272 T273))
		)
		
	(let
		(
			(S15 (FA_SUM S1 S2 S3))
			(C15 (FA_CARRY S1 S2 S3))
			(S16 (FA_SUM S4 S5 S6))
			(C16 (FA_CARRY S4 S5 S6))
			(S17 (FA_SUM S7 S8 S9))
			(C17 (FA_CARRY S7 S8 S9))
			(S18 (FA_SUM S10 S11 S12))
			(C18 (FA_CARRY S10 S11 S12))
			(S19 (FA_SUM S13 S14 T274))
			(C19 (FA_CARRY S13 S14 T274))
		)
		
	(let
		(
			(S20 (FA_SUM S15 S16 S17))
			(C20 (FA_CARRY S15 S16 S17))
			(S21 (FA_SUM S18 S19 T275))
			(C21 (FA_CARRY S18 S19 T275))
		)
		
	(let
		(
			(Z22 (HA_SUM S20 S21)) ;23rd bit
			(C22 (HA_CARRY S20 S21))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 T276 T277))
			(C8 (FA_CARRY C22 T276 T277))
			(S9 (FA_SUM T278 T279 T280))
			(C9 (FA_CARRY T278 T279 T280))
			(S10 (FA_SUM T281 T282 T283))
			(C10 (FA_CARRY T281 T282 T283))
			(S11 (FA_SUM T284 T285 T286))
			(C11 (FA_CARRY T284 T285 T286))
			(S12 (FA_SUM T287 T288 T289))
			(C12 (FA_CARRY T287 T288 T289))
			(S13 (FA_SUM T290 T291 T292))
			(C13 (FA_CARRY T290 T291 T292))
			(S14 (FA_SUM T293 T294 T295))
			(C14 (FA_CARRY T293 T294 T295))
			(S15 (FA_SUM T296 T297 T298))
			(C15 (FA_CARRY T296 T297 T298))
		)
		
	(let
		(
			(S16 (FA_SUM S1 S2 S3))
			(C16 (FA_CARRY S1 S2 S3))
			(S17 (FA_SUM S4 S5 S6))
			(C17 (FA_CARRY S4 S5 S6))
			(S18 (FA_SUM S7 S8 S9))
			(C18 (FA_CARRY S7 S8 S9))
			(S19 (FA_SUM S10 S11 S12))
			(C19 (FA_CARRY S10 S11 S12))
			(S20 (FA_SUM S13 S14 S15))
			(C20 (FA_CARRY S13 S14 S15))
		)
			
	(let
		(
			(S21 (FA_SUM S16 S17 S18))
			(C21 (FA_CARRY S16 S17 S18))
			(S22 (FA_SUM S19 S20 T299))
			(C22 (FA_CARRY S19 S20 T299))
		)
		
	(let
		(
			(Z23 (HA_SUM S21 S22)) ;24th bit
			(C23 (HA_CARRY S21 S22))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 T300))
			(C8 (FA_CARRY C22 C23 T300))
			(S9 (FA_SUM T301 T302 T303))
			(C9 (FA_CARRY T301 T302 T303))
			(S10 (FA_SUM T304 T305 T306))
			(C10 (FA_CARRY T304 T305 T306))
			(S11 (FA_SUM T307 T308 T309))
			(C11 (FA_CARRY T307 T308 T309))
			(S12 (FA_SUM T310 T311 T312))
			(C12 (FA_CARRY T310 T311 T312))
			(S13 (FA_SUM T313 T314 T315))
			(C13 (FA_CARRY T313 T314 T315))
			(S14 (FA_SUM T316 T317 T318))
			(C14 (FA_CARRY T316 T317 T318))
			(S15 (FA_SUM T319 T320 T321))
			(C15 (FA_CARRY T319 T320 T321))
		)
		
	(let
		(
			(S16 (FA_SUM S1 S2 S3))
			(C16 (FA_CARRY S1 S2 S3))
			(S17 (FA_SUM S4 S5 S6))
			(C17 (FA_CARRY S4 S5 S6))
			(S18 (FA_SUM S7 S8 S9))
			(C18 (FA_CARRY S7 S8 S9))
			(S19 (FA_SUM S10 S11 S12))
			(C19 (FA_CARRY S10 S11 S12))
			(S20 (FA_SUM S13 S14 S15))
			(C20 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(S21 (FA_SUM S16 S17 S18))
			(C21 (FA_CARRY S16 S17 S18))
			(S22 (FA_SUM S19 S20 T322))
			(C22 (FA_CARRY S19 S20 T322))
		)
		
	(let
		(
			(Z24 (HA_SUM S21 S22)) ;25th bit
			(C23 (HA_CARRY S21 S22))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 T323))
			(C8 (FA_CARRY C22 C23 T323))
			(S9 (FA_SUM T324 T325 T326))
			(C9 (FA_CARRY T324 T325 T326))
			(S10 (FA_SUM T327 T328 T329))
			(C10 (FA_CARRY T327 T328 T329))
			(S11 (FA_SUM T330 T331 T332))
			(C11 (FA_CARRY T330 T331 T332))
			(S12 (FA_SUM T333 T334 T335))
			(C12 (FA_CARRY T333 T334 T335))
			(S13 (FA_SUM T336 T337 T338))
			(C13 (FA_CARRY T336 T337 T338))
			(S14 (FA_SUM T339 T340 T341))
			(C14 (FA_CARRY T339 T340 T341))
			(S15 (FA_SUM T342 T343 T344))
			(C15 (FA_CARRY T342 T343 T344))
		)
		
	(let
		(
			(S16 (FA_SUM S1 S2 S3))
			(C16 (FA_CARRY S1 S2 S3))
			(S17 (FA_SUM S4 S5 S6))
			(C17 (FA_CARRY S4 S5 S6))
			(S18 (FA_SUM S7 S8 S9))
			(C18 (FA_CARRY S7 S8 S9))
			(S19 (FA_SUM S10 S11 S12))
			(C19 (FA_CARRY S10 S11 S12))
			(S20 (FA_SUM S13 S14 S15))
			(C20 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(S21 (FA_SUM S16 S17 S18))
			(C21 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(Z25 (FA_SUM S19 S20 S21)) ;26th bit
			(C22 (FA_CARRY S19 S20 S21))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 T345 T346))
			(C8 (FA_CARRY C22 T345 T346))
			(S9 (FA_SUM T347 T348 T349))
			(C9 (FA_CARRY T347 T348 T349))
			(S10 (FA_SUM T350 T351 T352))
			(C10 (FA_CARRY T350 T351 T352))
			(S11 (FA_SUM T353 T354 T355))
			(C11 (FA_CARRY T353 T354 T355))
			(S12 (FA_SUM T356 T357 T358))
			(C12 (FA_CARRY T356 T357 T358))
			(S13 (FA_SUM T359 T360 T361))
			(C13 (FA_CARRY T359 T360 T361))
			(S14 (FA_SUM T362 T363 T364))
			(C14 (FA_CARRY T362 T363 T364))
		)
		
	(let
		(
			(S15 (FA_SUM S1 S2 S3))
			(C15 (FA_CARRY S1 S2 S3))
			(S16 (FA_SUM S4 S5 S6))
			(C16 (FA_CARRY S4 S5 S6))
			(S17 (FA_SUM S7 S8 S9))
			(C17 (FA_CARRY S7 S8 S9))
			(S18 (FA_SUM S10 S11 S12))
			(C18 (FA_CARRY S10 S11 S12))
			(S19 (FA_SUM S13 S14 T365))
			(C19 (FA_CARRY S13 S14 T365))
		)
		
	(let
		(
			(S20 (FA_SUM S15 S16 S17))
			(C20 (FA_CARRY S15 S16 S17))
		)
		
	(let
		(
			(Z26 (FA_SUM S18 S19 S20)) ;27th bit
			(C21 (FA_CARRY S18 S19 S20))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM T366 T367 T368))
			(C8 (FA_CARRY T366 T367 T368))
			(S9 (FA_SUM T369 T370 T371))
			(C9 (FA_CARRY T369 T370 T371))
			(S10 (FA_SUM T372 T373 T374))
			(C10 (FA_CARRY T372 T373 T374))
			(S11 (FA_SUM T375 T376 T377))
			(C11 (FA_CARRY T375 T376 T377))
			(S12 (FA_SUM T378 T379 T380))
			(C12 (FA_CARRY T378 T379 T380))
			(S13 (FA_SUM T381 T382 T383))
			(C13 (FA_CARRY T381 T382 T383))
		)
		
	(let
		(
			(S14 (FA_SUM S1 S2 S3))
			(C14 (FA_CARRY S1 S2 S3))
			(S15 (FA_SUM S4 S5 S6))
			(C15 (FA_CARRY S4 S5 S6))
			(S16 (FA_SUM S7 S8 S9))
			(C16 (FA_CARRY S7 S8 S9))
			(S17 (FA_SUM S10 S11 S12))
			(C17 (FA_CARRY S10 S11 S12))
			(S18 (FA_SUM S13 T384 T385))
			(C18 (FA_CARRY S13 T384 T385))
		)
		
	(let
		(
			(S19 (FA_SUM S14 S15 S16))
			(C19 (FA_CARRY S14 S15 S16))
		)
		
	(let
		(
			(Z27 (FA_SUM S17 S18 S19)) ;28th bit
			(C20 (FA_CARRY S17 S18 S19))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 T386))
			(C7 (FA_CARRY C19 C20 T386))
			(S8 (FA_SUM T387 T388 T389))
			(C8 (FA_CARRY T387 T388 T389))
			(S9 (FA_SUM T390 T391 T392))
			(C9 (FA_CARRY T390 T391 T392))
			(S10 (FA_SUM T393 T394 T395))
			(C10 (FA_CARRY T393 T394 T395))
			(S11 (FA_SUM T396 T397 T398))
			(C11 (FA_CARRY T396 T397 T398))
			(S12 (FA_SUM T399 T400 T401))
			(C12 (FA_CARRY T399 T400 T401))
			(S13 (FA_SUM T402 T403 T404))
			(C13 (FA_CARRY T402 T403 T404))
		)
		
	(let
		(
			(S14 (FA_SUM S1 S2 S3))
			(C14 (FA_CARRY S1 S2 S3))
			(S15 (FA_SUM S4 S5 S6))
			(C15 (FA_CARRY S4 S5 S6))
			(S16 (FA_SUM S7 S8 S9))
			(C16 (FA_CARRY S7 S8 S9))
			(S17 (FA_SUM S10 S11 S12))
			(C17 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S18 (FA_SUM S14 S15 S16))
			(C18 (FA_CARRY S14 S15 S16))
		)
		
	(let
		(
			(Z28 (FA_SUM S13 S17 S18)) ;29th bit
			(C19 (FA_CARRY S13 S17 S18))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 T405 T406))
			(C7 (FA_CARRY C19 T405 T406))
			(S8 (FA_SUM T407 T408 T409))
			(C8 (FA_CARRY T407 T408 T409))
			(S9 (FA_SUM T410 T411 T412))
			(C9 (FA_CARRY T410 T411 T412))
			(S10 (FA_SUM T413 T414 T415))
			(C10 (FA_CARRY T413 T414 T415))
			(S11 (FA_SUM T416 T417 T418))
			(C11 (FA_CARRY T416 T417 T418))
			(S12 (FA_SUM T419 T420 T421))
			(C12 (FA_CARRY T419 T420 T421))
		)
		
	(let
		(
			(S13 (FA_SUM S1 S2 S3))
			(C13 (FA_CARRY S1 S2 S3))
			(S14 (FA_SUM S4 S5 S6))
			(C14 (FA_CARRY S4 S5 S6))
			(S15 (FA_SUM S7 S8 S9))
			(C15 (FA_CARRY S7 S8 S9))
			(S16 (FA_SUM S10 S11 S12))
			(C16 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S17 (FA_SUM S13 S14 S15))
			(C17 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(Z29 (FA_SUM S16 S17 T422)) ;30th bit
			(C18 (FA_CARRY S16 S17 T422))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM T423 T424 T425))
			(C7 (FA_CARRY T423 T424 T425))
			(S8 (FA_SUM T426 T427 T428))
			(C8 (FA_CARRY T426 T427 T428))
			(S9 (FA_SUM T429 T430 T431))
			(C9 (FA_CARRY T429 T430 T431))
			(S10 (FA_SUM T432 T433 T434))
			(C10 (FA_CARRY T432 T433 T434))
			(S11 (FA_SUM T435 T436 T437))
			(C11 (FA_CARRY T435 T436 T437))
		)
		
	(let
		(
			(S12 (FA_SUM S1 S2 S3))
			(C12 (FA_CARRY S1 S2 S3))
			(S13 (FA_SUM S4 S5 S6))
			(C13 (FA_CARRY S4 S5 S6))
			(S14 (FA_SUM S7 S8 S9))
			(C14 (FA_CARRY S7 S8 S9))
			(S15 (FA_SUM S10 S11 T438))
			(C15 (FA_CARRY S10 S11 T438))
		)
		
	(let
		(
			(S16 (FA_SUM S12 S13 S14))
			(C16 (FA_CARRY S12 S13 S14))
		)
		
	(let
		(
			(Z30 (FA_SUM S15 S16 T439)) ;31st bit
			(C17 (FA_CARRY S15 S16 T439))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 T440))
			(C6 (FA_CARRY C16 C17 T440))
			(S7 (FA_SUM T441 T442 T443))
			(C7 (FA_CARRY T441 T442 T443))
			(S8 (FA_SUM T444 T445 T446))
			(C8 (FA_CARRY T444 T445 T446))
			(S9 (FA_SUM T447 T448 T449))
			(C9 (FA_CARRY T447 T448 T449))
			(S10 (FA_SUM T450 T451 T452))
			(C10 (FA_CARRY T450 T451 T452))
			(S11 (FA_SUM T453 T454 T455))
			(C11 (FA_CARRY T453 T454 T455))
		)
		
	(let
		(
			(S12 (FA_SUM S1 S2 S3))
			(C12 (FA_CARRY S1 S2 S3))
			(S13 (FA_SUM S4 S5 S6))
			(C13 (FA_CARRY S4 S5 S6))
			(S14 (FA_SUM S7 S8 S9))
			(C14 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S15 (FA_SUM S12 S13 S14))
			(C15 (FA_CARRY S12 S13 S14))
		)
		
	(let
		(
			(Z31 (FA_SUM S10 S11 S15)) ;32nd bit
			(C16 (FA_CARRY S10 S11 S15))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 T456 T457))
			(C6 (FA_CARRY C16 T456 T457))
			(S7 (FA_SUM T458 T459 T460))
			(C7 (FA_CARRY T458 T459 T460))
			(S8 (FA_SUM T461 T462 T463))
			(C8 (FA_CARRY T461 T462 T463))
			(S9 (FA_SUM T464 T465 T466))
			(C9 (FA_CARRY T464 T465 T466))
			(S10 (FA_SUM T467 T468 T469))
			(C10 (FA_CARRY T467 T468 T469))
		)
		
	(let
		(
			(S11 (FA_SUM S1 S2 S3))
			(C11 (FA_CARRY S1 S2 S3))
			(S12 (FA_SUM S4 S5 S6))
			(C12 (FA_CARRY S4 S5 S6))
			(S13 (FA_SUM S7 S8 S9))
			(C13 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S14 (FA_SUM S11 S12 S13))
			(C14 (FA_CARRY S11 S12 S13))
		)
		
	(let
		(
			(Z32 (FA_SUM S10 S14 T470)) ;33rd bit
			(C15 (FA_CARRY S10 S14 T470))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM T471 T472 T473))
			(C6 (FA_CARRY T471 T472 T473))
			(S7 (FA_SUM T474 T475 T476))
			(C7 (FA_CARRY T474 T475 T476))
			(S8 (FA_SUM T477 T478 T479))
			(C8 (FA_CARRY T477 T478 T479))
			(S9 (FA_SUM T480 T481 T482))
			(C9 (FA_CARRY T480 T481 T482))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S13 (FA_SUM S10 S11 S12))
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(Z33 (FA_SUM S13 T483 T484)) ;34th bit
			(C14 (FA_CARRY S13 T483 T484))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 T485))
			(C5 (FA_CARRY C13 C14 T485))
			(S6 (FA_SUM T486 T487 T488))
			(C6 (FA_CARRY T486 T487 T488))
			(S7 (FA_SUM T489 T490 T491))
			(C7 (FA_CARRY T489 T490 T491))
			(S8 (FA_SUM T492 T493 T494))
			(C8 (FA_CARRY T492 T493 T494))
			(S9 (FA_SUM T495 T496 T497))
			(C9 (FA_CARRY T495 T496 T497))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(Z34 (FA_SUM S10 S11 S12)) ;35th bit
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 T498 T499))
			(C5 (FA_CARRY C13 T498 T499))
			(S6 (FA_SUM T500 T501 T502))
			(C6 (FA_CARRY T500 T501 T502))
			(S7 (FA_SUM T503 T504 T505))
			(C7 (FA_CARRY T503 T504 T505))
			(S8 (FA_SUM T506 T507 T508))
			(C8 (FA_CARRY T506 T507 T508))
		)
		
	(let
		(
			(S9 (FA_SUM S1 S2 S3))
			(C9 (FA_CARRY S1 S2 S3))
			(S10 (FA_SUM S4 S5 S6))
			(C10 (FA_CARRY S4 S5 S6))
			(S11 (FA_SUM S7 S8 T509))
			(C11 (FA_CARRY S7 S8 T509))
		)
		
	(let
		(
			(Z35 (FA_SUM S9 S10 S11)) ;36th bit
			(C12 (FA_CARRY S9 S10 S11))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM T510 T511 T512))
			(C5 (FA_CARRY T510 T511 T512))
			(S6 (FA_SUM T513 T514 T515))
			(C6 (FA_CARRY T513 T514 T515))
			(S7 (FA_SUM T516 T517 T518))
			(C7 (FA_CARRY T516 T517 T518))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
			(S10 (FA_SUM S7 T519 T520))
			(C10 (FA_CARRY S7 T519 T520))
		)
		
	(let
		(
			(Z36 (FA_SUM S8 S9 S10)) ;37th bit
			(C11 (FA_CARRY S8 S9 S10))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 T521))
			(C4 (FA_CARRY C10 C11 T521))
			(S5 (FA_SUM T522 T523 T524))
			(C5 (FA_CARRY T522 T523 T524))
			(S6 (FA_SUM T525 T526 T527))
			(C6 (FA_CARRY T525 T526 T527))
			(S7 (FA_SUM T528 T529 T530))
			(C7 (FA_CARRY T528 T529 T530))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z37 (FA_SUM S7 S8 S9)) ;38th bit
			(C10 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 T531 T532))
			(C4 (FA_CARRY C10 T531 T532))
			(S5 (FA_SUM T533 T534 T535))
			(C5 (FA_CARRY T533 T534 T535))
			(S6 (FA_SUM T536 T537 T538))
			(C6 (FA_CARRY T536 T537 T538))
		)
		
	(let
		(
			(S7 (FA_SUM S1 S2 S3))
			(C7 (FA_CARRY S1 S2 S3))
			(S8 (FA_SUM S4 S5 S6))
			(C8 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z38 (FA_SUM S7 S8 T539)) ;39th bit
			(C9 (FA_CARRY S7 S8 T539))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T540 T541 T542))
			(C4 (FA_CARRY T540 T541 T542))
			(S5 (FA_SUM T543 T544 T545))
			(C5 (FA_CARRY T543 T544 T545))
		)
		
	(let
		(
			(S6 (FA_SUM S1 S2 S3))
			(C6 (FA_CARRY S1 S2 S3))
			(S7 (FA_SUM S4 S5 T546))
			(C7 (FA_CARRY S4 S5 T546))
		)
		
	(let
		(
			(Z39 (FA_SUM S6 S7 T547)) ;40th bit
			(C8 (FA_CARRY S6 S7 T547))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 T548))
			(C3 (FA_CARRY C7 C8 T548))
			(S4 (FA_SUM T549 T550 T551))
			(C4 (FA_CARRY T549 T550 T551))
			(S5 (FA_SUM T552 T553 T554))
			(C5 (FA_CARRY T552 T553 T554))
		)
		
	(let
		(
			(S6 (FA_SUM S1 S2 S3))
			(C6 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z40 (FA_SUM S4 S5 S6)) ;41st bit
			(C7 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 T555 T556))
			(C3 (FA_CARRY C7 T555 T556))
			(S4 (FA_SUM T557 T558 T559))
			(C4 (FA_CARRY T557 T558 T559))
		)
		
	(let
		(
			(S5 (FA_SUM S1 S2 S3))
			(C5 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z41 (FA_SUM S4 S5 T560)) ;42nd bit
			(C6 (FA_CARRY S4 S5 T560))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T561 T562 T563))
			(C3 (FA_CARRY T561 T562 T563))
		)
		
	(let
		(
			(S4 (FA_SUM S1 S2 S3))
			(C4 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z42 (FA_SUM S4 T564 T565)) ;43rd bit
			(C5 (FA_CARRY S4 T564 T565))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 T566))
			(C2 (FA_CARRY C4 C5 T566))
			(S3 (FA_SUM T567 T568 T569))
			(C3 (FA_CARRY T567 T568 T569))
		)
		
	(let
		(
			(Z43 (FA_SUM S1 S2 S3)) ;44th bit
			(C4 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 T570 T571))
			(C2 (FA_CARRY C4 T570 T571))
		)
		
	(let
		(
			(Z44 (FA_SUM S1 S2 T572)) ;45th bit
			(C3 (FA_CARRY S1 S2 T572))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
		)
		
	(let
		(
			(Z45 (FA_SUM S1 T573 T574)) ;46th bit
			(C2 (FA_CARRY S1 T573 T574))
		)
		
	(let
		(
			(Z46 (FA_SUM C1 C2 T575)) ;47th bit
			(Z47 (FA_CARRY C1 C2 T575)) ;48th bit
		)
		
	(let
		(
			(Z (concat Z47 Z46 Z45 Z44 Z43 Z42 Z41 Z40 Z39 Z38 Z37 Z36 Z35 Z34 Z33 Z32 Z31 Z30 Z29 Z28 Z27 Z26 Z25 Z24 Z23 Z22 Z21 Z20 Z19 Z18 Z17 Z16 Z15 Z14 Z13 Z12 Z11 Z10 Z9 Z8 Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
			(select (bvand Z47 Z46 Z45 Z44 Z43 Z42 Z41 Z40 Z39 Z38 Z37 Z36 Z35 Z34 Z33 Z32 Z31 Z30 Z29 Z28 Z27 Z26 Z25 Z24 Z23 Z22 Z21 Z20 Z19 Z18 Z17 Z16 Z15 Z14 Z13 Z12 Z11 Z10 Z9 Z8 Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
		)
		
	(let
		(
			(muxout (mux2to1 Z trojanout select))
		)
	
	(let
		(
			(F (concat trojanout muxout))
		)
	))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))
)

(assert
	(not
		(let
			(
				(te2 ((_ extract 2 2) trojan))
				(te1 ((_ extract 1 1) trojan))
				(te0 ((_ extract 0 0) trojan))
			)
		(let
			(
				(trojan1 (concat te2 te1 te0 t0))
				(trojan2 (concat te2 te1 te0 t1))
			)
		(let
			(
				(trojanew1 (concat trojan1 trojan1 trojan1 trojan1 trojan1 trojan1 trojan1 trojan1 trojan1 trojan1 trojan1 trojan1))
				(trojanew2 (concat trojan2 trojan2 trojan2 trojan2 trojan2 trojan2 trojan2 trojan2 trojan2 trojan2 trojan2 trojan2))
			)
		(let
			(
				(state0 (ARRAY_MUL Xmul Ymul trojanew1))
				(state1 (ARRAY_MUL Xmul Ymul trojanew2))
			) 
		(let
			(
				(LHS ((_ extract 47 0) state0))
				(RHS ((_ extract 47 0) state1))
			)
		(=>
			(and (= t0 (_ bv0 1)) (= t1 (_ bv1 1)))
			(= LHS RHS)
		)))))))
)
			
(check-sat)
(get-model)