(set-logic QF_BV)
(declare-fun Xmul () (_ BitVec 32))
(declare-fun Ymul () (_ BitVec 31))
(declare-fun a0 () (_ BitVec 1))
(declare-fun a1 () (_ BitVec 1))
(declare-fun trojan () (_ BitVec 4))

(define-fun mux2to1 ( (s0(_ BitVec 64)) (s1(_ BitVec 64)) (s(_ BitVec 1))) (_ BitVec 64)
	(ite (= s (_ bv0 1)) s0 s1)
)

(define-fun HA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor x y)
)

(define-fun HA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvand x y)
)

(define-fun FA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor (bvxor x y) cin)
)

(define-fun FA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvor (bvand x y) (bvor (bvand x cin) (bvand y cin)) )
)

(define-fun ARRAY_MUL ( (X(_ BitVec 32)) (Y(_ BitVec 32)) (trojan(_ BitVec 64))) (_ BitVec 128) 
	(let
		(
			(X31 ((_ extract 31 31) X))
			(X30 ((_ extract 30 30) X))
			(X29 ((_ extract 29 29) X))
			(X28 ((_ extract 28 28) X))
			(X27 ((_ extract 27 27) X))
			(X26 ((_ extract 26 26) X))
			(X25 ((_ extract 25 25) X))
			(X24 ((_ extract 24 24) X))
			(X23 ((_ extract 23 23) X))
			(X22 ((_ extract 22 22) X))
			(X21 ((_ extract 21 21) X))
			(X20 ((_ extract 20 20) X))
			(X19 ((_ extract 19 19) X))
			(X18 ((_ extract 18 18) X))
			(X17 ((_ extract 17 17) X))
			(X16 ((_ extract 16 16) X))
			(X15 ((_ extract 15 15) X))
			(X14 ((_ extract 14 14) X))
			(X13 ((_ extract 13 13) X))
			(X12 ((_ extract 12 12) X))
			(X11 ((_ extract 11 11) X))
			(X10 ((_ extract 10 10) X))
			(X9 ((_ extract 9 9) X))
			(X8 ((_ extract 8 8) X))
			(X7 ((_ extract 7 7) X))
			(X6 ((_ extract 6 6) X))
			(X5 ((_ extract 5 5) X))
			(X4 ((_ extract 4 4) X))
			(X3 ((_ extract 3 3) X))
			(X2 ((_ extract 2 2) X))
			(X1 ((_ extract 1 1) X))
			(X0 ((_ extract 0 0) X))
			(Y31 ((_ extract 31 31) Y))
			(Y30 ((_ extract 30 30) Y))
			(Y29 ((_ extract 29 29) Y))
			(Y28 ((_ extract 28 28) Y))
			(Y27 ((_ extract 27 27) Y))
			(Y26 ((_ extract 26 26) Y))
			(Y25 ((_ extract 25 25) Y))
			(Y24 ((_ extract 24 24) Y))
			(Y23 ((_ extract 23 23) Y))
			(Y22 ((_ extract 22 22) Y))
			(Y21 ((_ extract 21 21) Y))
			(Y20 ((_ extract 20 20) Y))
			(Y19 ((_ extract 19 19) Y))
			(Y18 ((_ extract 18 18) Y))
			(Y17 ((_ extract 17 17) Y))
			(Y16 ((_ extract 16 16) Y))
			(Y15 ((_ extract 15 15) Y))
			(Y14 ((_ extract 14 14) Y))
			(Y13 ((_ extract 13 13) Y))
			(Y12 ((_ extract 12 12) Y))
			(Y11 ((_ extract 11 11) Y))
			(Y10 ((_ extract 10 10) Y))
			(Y9 ((_ extract 9 9) Y))
			(Y8 ((_ extract 8 8) Y))
			(Y7 ((_ extract 7 7) Y))
			(Y6 ((_ extract 6 6) Y))
			(Y5 ((_ extract 5 5) Y))
			(Y4 ((_ extract 4 4) Y))
			(Y3 ((_ extract 3 3) Y))
			(Y2 ((_ extract 2 2) Y))
			(Y1 ((_ extract 1 1) Y))
			(Y0 ((_ extract 0 0) Y))
		)
	(let
		(
			(Z0 (bvand X0 Y0)) ;1st bit
			(T1 (bvand X1 Y0))
			(T2 (bvand X0 Y1))
			(T3 (bvand X2 Y0))
			(T4 (bvand X1 Y1))
			(T5 (bvand X0 Y2))
			(T6 (bvand X3 Y0))
			(T7 (bvand X2 Y1))
			(T8 (bvand X1 Y2))
			(T9 (bvand X0 Y3))
			(T10 (bvand X4 Y0))
			(T11 (bvand X3 Y1))
			(T12 (bvand X2 Y2))
			(T13 (bvand X1 Y3))
			(T14 (bvand X0 Y4))
			(T15 (bvand X5 Y0))
			(T16 (bvand X4 Y1))
			(T17 (bvand X3 Y2))
			(T18 (bvand X2 Y3))
			(T19 (bvand X1 Y4))
			(T20 (bvand X0 Y5))
			(T21 (bvand X6 Y0))
			(T22 (bvand X5 Y1))
			(T23 (bvand X4 Y2))
			(T24 (bvand X3 Y3))
			(T25 (bvand X2 Y4))
			(T26 (bvand X1 Y5))
			(T27 (bvand X0 Y6))
			(T28 (bvand X7 Y0))
			(T29 (bvand X6 Y1))
			(T30 (bvand X5 Y2))
			(T31 (bvand X4 Y3))
			(T32 (bvand X3 Y4))
			(T33 (bvand X2 Y5))
			(T34 (bvand X1 Y6))
			(T35 (bvand X0 Y7))
			(T36 (bvand X8 Y0))
			(T37 (bvand X7 Y1))
			(T38 (bvand X6 Y2))
			(T39 (bvand X5 Y3))
			(T40 (bvand X4 Y4))
			(T41 (bvand X3 Y5))
			(T42 (bvand X2 Y6))
			(T43 (bvand X1 Y7))
			(T44 (bvand X0 Y8))
			(T45 (bvand X9 Y0))
			(T46 (bvand X8 Y1))
			(T47 (bvand X7 Y2))
			(T48 (bvand X6 Y3))
			(T49 (bvand X5 Y4))
			(T50 (bvand X4 Y5))
			(T51 (bvand X3 Y6))
			(T52 (bvand X2 Y7))
			(T53 (bvand X1 Y8))
			(T54 (bvand X0 Y9))
			(T55 (bvand X10 Y0))
			(T56 (bvand X9 Y1))
			(T57 (bvand X8 Y2))
			(T58 (bvand X7 Y3))
			(T59 (bvand X6 Y4))
			(T60 (bvand X5 Y5))
			(T61 (bvand X4 Y6))
			(T62 (bvand X3 Y7))
			(T63 (bvand X2 Y8))
			(T64 (bvand X1 Y9))
			(T65 (bvand X0 Y10))
			(T66 (bvand X11 Y0))
			(T67 (bvand X10 Y1))
			(T68 (bvand X9 Y2))
			(T69 (bvand X8 Y3))
			(T70 (bvand X7 Y4))
			(T71 (bvand X6 Y5))
			(T72 (bvand X5 Y6))
			(T73 (bvand X4 Y7))
			(T74 (bvand X3 Y8))
			(T75 (bvand X2 Y9))
			(T76 (bvand X1 Y10))
			(T77 (bvand X0 Y11))
			(T78 (bvand X12 Y0))
			(T79 (bvand X11 Y1))
			(T80 (bvand X10 Y2))
			(T81 (bvand X9 Y3))
			(T82 (bvand X8 Y4))
			(T83 (bvand X7 Y5))
			(T84 (bvand X6 Y6))
			(T85 (bvand X5 Y7))
			(T86 (bvand X4 Y8))
			(T87 (bvand X3 Y9))
			(T88 (bvand X2 Y10))
			(T89 (bvand X1 Y11))
			(T90 (bvand X0 Y12))
			(T91 (bvand X13 Y0))
			(T92 (bvand X12 Y1))
			(T93 (bvand X11 Y2))
			(T94 (bvand X10 Y3))
			(T95 (bvand X9 Y4))
			(T96 (bvand X8 Y5))
			(T97 (bvand X7 Y6))
			(T98 (bvand X6 Y7))
			(T99 (bvand X5 Y8))
			(T100 (bvand X4 Y9))
			(T101 (bvand X3 Y10))
			(T102 (bvand X2 Y11))
			(T103 (bvand X1 Y12))
			(T104 (bvand X0 Y13))
			(T105 (bvand X14 Y0))
			(T106 (bvand X13 Y1))
			(T107 (bvand X12 Y2))
			(T108 (bvand X11 Y3))
			(T109 (bvand X10 Y4))
			(T110 (bvand X9 Y5))
			(T111 (bvand X8 Y6))
			(T112 (bvand X7 Y7))
			(T113 (bvand X6 Y8))
			(T114 (bvand X5 Y9))
			(T115 (bvand X4 Y10))
			(T116 (bvand X3 Y11))
			(T117 (bvand X2 Y12))
			(T118 (bvand X1 Y13))
			(T119 (bvand X0 Y14))
			(T120 (bvand X15 Y0))
			(T121 (bvand X14 Y1))
			(T122 (bvand X13 Y2))
			(T123 (bvand X12 Y3))
			(T124 (bvand X11 Y4))
			(T125 (bvand X10 Y5))
			(T126 (bvand X9 Y6))
			(T127 (bvand X8 Y7))
			(T128 (bvand X7 Y8))
			(T129 (bvand X6 Y9))
			(T130 (bvand X5 Y10))
			(T131 (bvand X4 Y11))
			(T132 (bvand X3 Y12))
			(T133 (bvand X2 Y13))
			(T134 (bvand X1 Y14))
			(T135 (bvand X0 Y15))
			(T136 (bvand X16 Y0))
			(T137 (bvand X15 Y1))
			(T138 (bvand X14 Y2))
			(T139 (bvand X13 Y3))
			(T140 (bvand X12 Y4))
			(T141 (bvand X11 Y5))
			(T142 (bvand X10 Y6))
			(T143 (bvand X9 Y7))
			(T144 (bvand X8 Y8))
			(T145 (bvand X7 Y9))
			(T146 (bvand X6 Y10))
			(T147 (bvand X5 Y11))
			(T148 (bvand X4 Y12))
			(T149 (bvand X3 Y13))
			(T150 (bvand X2 Y14))
			(T151 (bvand X1 Y15))
			(T152 (bvand X0 Y16))
			(T153 (bvand X17 Y0))
			(T154 (bvand X16 Y1))
			(T155 (bvand X15 Y2))
			(T156 (bvand X14 Y3))
			(T157 (bvand X13 Y4))
			(T158 (bvand X12 Y5))
			(T159 (bvand X11 Y6))
			(T160 (bvand X10 Y7))
			(T161 (bvand X9 Y8))
			(T162 (bvand X8 Y9))
			(T163 (bvand X7 Y10))
			(T164 (bvand X6 Y11))
			(T165 (bvand X5 Y12))
			(T166 (bvand X4 Y13))
			(T167 (bvand X3 Y14))
			(T168 (bvand X2 Y15))
			(T169 (bvand X1 Y16))
			(T170 (bvand X0 Y17))
			(T171 (bvand X18 Y0))
			(T172 (bvand X17 Y1))
			(T173 (bvand X16 Y2))
			(T174 (bvand X15 Y3))
			(T175 (bvand X14 Y4))
			(T176 (bvand X13 Y5))
			(T177 (bvand X12 Y6))
			(T178 (bvand X11 Y7))
			(T179 (bvand X10 Y8))
			(T180 (bvand X9 Y9))
			(T181 (bvand X8 Y10))
			(T182 (bvand X7 Y11))
			(T183 (bvand X6 Y12))
			(T184 (bvand X5 Y13))
			(T185 (bvand X4 Y14))
			(T186 (bvand X3 Y15))
			(T187 (bvand X2 Y16))
			(T188 (bvand X1 Y17))
			(T189 (bvand X0 Y18))
			(T190 (bvand X19 Y0))
			(T191 (bvand X18 Y1))
			(T192 (bvand X17 Y2))
			(T193 (bvand X16 Y3))
			(T194 (bvand X15 Y4))
			(T195 (bvand X14 Y5))
			(T196 (bvand X13 Y6))
			(T197 (bvand X12 Y7))
			(T198 (bvand X11 Y8))
			(T199 (bvand X10 Y9))
			(T200 (bvand X9 Y10))
			(T201 (bvand X8 Y11))
			(T202 (bvand X7 Y12))
			(T203 (bvand X6 Y13))
			(T204 (bvand X5 Y14))
			(T205 (bvand X4 Y15))
			(T206 (bvand X3 Y16))
			(T207 (bvand X2 Y17))
			(T208 (bvand X1 Y18))
			(T209 (bvand X0 Y19))
			(T210 (bvand X20 Y0))
			(T211 (bvand X19 Y1))
			(T212 (bvand X18 Y2))
			(T213 (bvand X17 Y3))
			(T214 (bvand X16 Y4))
			(T215 (bvand X15 Y5))
			(T216 (bvand X14 Y6))
			(T217 (bvand X13 Y7))
			(T218 (bvand X12 Y8))
			(T219 (bvand X11 Y9))
			(T220 (bvand X10 Y10))
			(T221 (bvand X9 Y11))
			(T222 (bvand X8 Y12))
			(T223 (bvand X7 Y13))
			(T224 (bvand X6 Y14))
			(T225 (bvand X5 Y15))
			(T226 (bvand X4 Y16))
			(T227 (bvand X3 Y17))
			(T228 (bvand X2 Y18))
			(T229 (bvand X1 Y19))
			(T230 (bvand X0 Y20))
			(T231 (bvand X21 Y0))
			(T232 (bvand X20 Y1))
			(T233 (bvand X19 Y2))
			(T234 (bvand X18 Y3))
			(T235 (bvand X17 Y4))
			(T236 (bvand X16 Y5))
			(T237 (bvand X15 Y6))
			(T238 (bvand X14 Y7))
			(T239 (bvand X13 Y8))
			(T240 (bvand X12 Y9))
			(T241 (bvand X11 Y10))
			(T242 (bvand X10 Y11))
			(T243 (bvand X9 Y12))
			(T244 (bvand X8 Y13))
			(T245 (bvand X7 Y14))
			(T246 (bvand X6 Y15))
			(T247 (bvand X5 Y16))
			(T248 (bvand X4 Y17))
			(T249 (bvand X3 Y18))
			(T250 (bvand X2 Y19))
			(T251 (bvand X1 Y20))
			(T252 (bvand X0 Y21))
			(T253 (bvand X22 Y0))
			(T254 (bvand X21 Y1))
			(T255 (bvand X20 Y2))
			(T256 (bvand X19 Y3))
			(T257 (bvand X18 Y4))
			(T258 (bvand X17 Y5))
			(T259 (bvand X16 Y6))
			(T260 (bvand X15 Y7))
			(T261 (bvand X14 Y8))
			(T262 (bvand X13 Y9))
			(T263 (bvand X12 Y10))
			(T264 (bvand X11 Y11))
			(T265 (bvand X10 Y12))
			(T266 (bvand X9 Y13))
			(T267 (bvand X8 Y14))
			(T268 (bvand X7 Y15))
			(T269 (bvand X6 Y16))
			(T270 (bvand X5 Y17))
			(T271 (bvand X4 Y18))
			(T272 (bvand X3 Y19))
			(T273 (bvand X2 Y20))
			(T274 (bvand X1 Y21))
			(T275 (bvand X0 Y22))
			(T276 (bvand X23 Y0))
			(T277 (bvand X22 Y1))
			(T278 (bvand X21 Y2))
			(T279 (bvand X20 Y3))
			(T280 (bvand X19 Y4))
			(T281 (bvand X18 Y5))
			(T282 (bvand X17 Y6))
			(T283 (bvand X16 Y7))
			(T284 (bvand X15 Y8))
			(T285 (bvand X14 Y9))
			(T286 (bvand X13 Y10))
			(T287 (bvand X12 Y11))
			(T288 (bvand X11 Y12))
			(T289 (bvand X10 Y13))
			(T290 (bvand X9 Y14))
			(T291 (bvand X8 Y15))
			(T292 (bvand X7 Y16))
			(T293 (bvand X6 Y17))
			(T294 (bvand X5 Y18))
			(T295 (bvand X4 Y19))
			(T296 (bvand X3 Y20))
			(T297 (bvand X2 Y21))
			(T298 (bvand X1 Y22))
			(T299 (bvand X0 Y23))
			(T300 (bvand X24 Y0))
			(T301 (bvand X23 Y1))
			(T302 (bvand X22 Y2))
			(T303 (bvand X21 Y3))
			(T304 (bvand X20 Y4))
			(T305 (bvand X19 Y5))
			(T306 (bvand X18 Y6))
			(T307 (bvand X17 Y7))
			(T308 (bvand X16 Y8))
			(T309 (bvand X15 Y9))
			(T310 (bvand X14 Y10))
			(T311 (bvand X13 Y11))
			(T312 (bvand X12 Y12))
			(T313 (bvand X11 Y13))
			(T314 (bvand X10 Y14))
			(T315 (bvand X9 Y15))
			(T316 (bvand X8 Y16))
			(T317 (bvand X7 Y17))
			(T318 (bvand X6 Y18))
			(T319 (bvand X5 Y19))
			(T320 (bvand X4 Y20))
			(T321 (bvand X3 Y21))
			(T322 (bvand X2 Y22))
			(T323 (bvand X1 Y23))
			(T324 (bvand X0 Y24))
			(T325 (bvand X25 Y0))
			(T326 (bvand X24 Y1))
			(T327 (bvand X23 Y2))
			(T328 (bvand X22 Y3))
			(T329 (bvand X21 Y4))
			(T330 (bvand X20 Y5))
			(T331 (bvand X19 Y6))
			(T332 (bvand X18 Y7))
			(T333 (bvand X17 Y8))
			(T334 (bvand X16 Y9))
			(T335 (bvand X15 Y10))
			(T336 (bvand X14 Y11))
			(T337 (bvand X13 Y12))
			(T338 (bvand X12 Y13))
			(T339 (bvand X11 Y14))
			(T340 (bvand X10 Y15))
			(T341 (bvand X9 Y16))
			(T342 (bvand X8 Y17))
			(T343 (bvand X7 Y18))
			(T344 (bvand X6 Y19))
			(T345 (bvand X5 Y20))
			(T346 (bvand X4 Y21))
			(T347 (bvand X3 Y22))
			(T348 (bvand X2 Y23))
			(T349 (bvand X1 Y24))
			(T350 (bvand X0 Y25))
			(T351 (bvand X26 Y0))
			(T352 (bvand X25 Y1))
			(T353 (bvand X24 Y2))
			(T354 (bvand X23 Y3))
			(T355 (bvand X22 Y4))
			(T356 (bvand X21 Y5))
			(T357 (bvand X20 Y6))
			(T358 (bvand X19 Y7))
			(T359 (bvand X18 Y8))
			(T360 (bvand X17 Y9))
			(T361 (bvand X16 Y10))
			(T362 (bvand X15 Y11))
			(T363 (bvand X14 Y12))
			(T364 (bvand X13 Y13))
			(T365 (bvand X12 Y14))
			(T366 (bvand X11 Y15))
			(T367 (bvand X10 Y16))
			(T368 (bvand X9 Y17))
			(T369 (bvand X8 Y18))
			(T370 (bvand X7 Y19))
			(T371 (bvand X6 Y20))
			(T372 (bvand X5 Y21))
			(T373 (bvand X4 Y22))
			(T374 (bvand X3 Y23))
			(T375 (bvand X2 Y24))
			(T376 (bvand X1 Y25))
			(T377 (bvand X0 Y26))
			(T378 (bvand X27 Y0))
			(T379 (bvand X26 Y1))
			(T380 (bvand X25 Y2))
			(T381 (bvand X24 Y3))
			(T382 (bvand X23 Y4))
			(T383 (bvand X22 Y5))
			(T384 (bvand X21 Y6))
			(T385 (bvand X20 Y7))
			(T386 (bvand X19 Y8))
			(T387 (bvand X18 Y9))
			(T388 (bvand X17 Y10))
			(T389 (bvand X16 Y11))
			(T390 (bvand X15 Y12))
			(T391 (bvand X14 Y13))
			(T392 (bvand X13 Y14))
			(T393 (bvand X12 Y15))
			(T394 (bvand X11 Y16))
			(T395 (bvand X10 Y17))
			(T396 (bvand X9 Y18))
			(T397 (bvand X8 Y19))
			(T398 (bvand X7 Y20))
			(T399 (bvand X6 Y21))
			(T400 (bvand X5 Y22))
			(T401 (bvand X4 Y23))
			(T402 (bvand X3 Y24))
			(T403 (bvand X2 Y25))
			(T404 (bvand X1 Y26))
			(T405 (bvand X0 Y27))
			(T406 (bvand X28 Y0))
			(T407 (bvand X27 Y1))
			(T408 (bvand X26 Y2))
			(T409 (bvand X25 Y3))
			(T410 (bvand X24 Y4))
			(T411 (bvand X23 Y5))
			(T412 (bvand X22 Y6))
			(T413 (bvand X21 Y7))
			(T414 (bvand X20 Y8))
			(T415 (bvand X19 Y9))
			(T416 (bvand X18 Y10))
			(T417 (bvand X17 Y11))
			(T418 (bvand X16 Y12))
			(T419 (bvand X15 Y13))
			(T420 (bvand X14 Y14))
			(T421 (bvand X13 Y15))
			(T422 (bvand X12 Y16))
			(T423 (bvand X11 Y17))
			(T424 (bvand X10 Y18))
			(T425 (bvand X9 Y19))
			(T426 (bvand X8 Y20))
			(T427 (bvand X7 Y21))
			(T428 (bvand X6 Y22))
			(T429 (bvand X5 Y23))
			(T430 (bvand X4 Y24))
			(T431 (bvand X3 Y25))
			(T432 (bvand X2 Y26))
			(T433 (bvand X1 Y27))
			(T434 (bvand X0 Y28))
			(T435 (bvand X29 Y0))
			(T436 (bvand X28 Y1))
			(T437 (bvand X27 Y2))
			(T438 (bvand X26 Y3))
			(T439 (bvand X25 Y4))
			(T440 (bvand X24 Y5))
			(T441 (bvand X23 Y6))
			(T442 (bvand X22 Y7))
			(T443 (bvand X21 Y8))
			(T444 (bvand X20 Y9))
			(T445 (bvand X19 Y10))
			(T446 (bvand X18 Y11))
			(T447 (bvand X17 Y12))
			(T448 (bvand X16 Y13))
			(T449 (bvand X15 Y14))
			(T450 (bvand X14 Y15))
			(T451 (bvand X13 Y16))
			(T452 (bvand X12 Y17))
			(T453 (bvand X11 Y18))
			(T454 (bvand X10 Y19))
			(T455 (bvand X9 Y20))
			(T456 (bvand X8 Y21))
			(T457 (bvand X7 Y22))
			(T458 (bvand X6 Y23))
			(T459 (bvand X5 Y24))
			(T460 (bvand X4 Y25))
			(T461 (bvand X3 Y26))
			(T462 (bvand X2 Y27))
			(T463 (bvand X1 Y28))
			(T464 (bvand X0 Y29))
			(T465 (bvand X30 Y0))
			(T466 (bvand X29 Y1))
			(T467 (bvand X28 Y2))
			(T468 (bvand X27 Y3))
			(T469 (bvand X26 Y4))
			(T470 (bvand X25 Y5))
			(T471 (bvand X24 Y6))
			(T472 (bvand X23 Y7))
			(T473 (bvand X22 Y8))
			(T474 (bvand X21 Y9))
			(T475 (bvand X20 Y10))
			(T476 (bvand X19 Y11))
			(T477 (bvand X18 Y12))
			(T478 (bvand X17 Y13))
			(T479 (bvand X16 Y14))
			(T480 (bvand X15 Y15))
			(T481 (bvand X14 Y16))
			(T482 (bvand X13 Y17))
			(T483 (bvand X12 Y18))
			(T484 (bvand X11 Y19))
			(T485 (bvand X10 Y20))
			(T486 (bvand X9 Y21))
			(T487 (bvand X8 Y22))
			(T488 (bvand X7 Y23))
			(T489 (bvand X6 Y24))
			(T490 (bvand X5 Y25))
			(T491 (bvand X4 Y26))
			(T492 (bvand X3 Y27))
			(T493 (bvand X2 Y28))
			(T494 (bvand X1 Y29))
			(T495 (bvand X0 Y30))
			(T496 (bvand X31 Y0))
			(T497 (bvand X30 Y1))
			(T498 (bvand X29 Y2))
			(T499 (bvand X28 Y3))
			(T500 (bvand X27 Y4))
			(T501 (bvand X26 Y5))
			(T502 (bvand X25 Y6))
			(T503 (bvand X24 Y7))
			(T504 (bvand X23 Y8))
			(T505 (bvand X22 Y9))
			(T506 (bvand X21 Y10))
			(T507 (bvand X20 Y11))
			(T508 (bvand X19 Y12))
			(T509 (bvand X18 Y13))
			(T510 (bvand X17 Y14))
			(T511 (bvand X16 Y15))
			(T512 (bvand X15 Y16))
			(T513 (bvand X14 Y17))
			(T514 (bvand X13 Y18))
			(T515 (bvand X12 Y19))
			(T516 (bvand X11 Y20))
			(T517 (bvand X10 Y21))
			(T518 (bvand X9 Y22))
			(T519 (bvand X8 Y23))
			(T520 (bvand X7 Y24))
			(T521 (bvand X6 Y25))
			(T522 (bvand X5 Y26))
			(T523 (bvand X4 Y27))
			(T524 (bvand X3 Y28))
			(T525 (bvand X2 Y29))
			(T526 (bvand X1 Y30))
			(T527 (bvand X0 Y31))
			(T528 (bvand X31 Y1))
			(T529 (bvand X30 Y2))
			(T530 (bvand X29 Y3))
			(T531 (bvand X28 Y4))
			(T532 (bvand X27 Y5))
			(T533 (bvand X26 Y6))
			(T534 (bvand X25 Y7))
			(T535 (bvand X24 Y8))
			(T536 (bvand X23 Y9))
			(T537 (bvand X22 Y10))
			(T538 (bvand X21 Y11))
			(T539 (bvand X20 Y12))
			(T540 (bvand X19 Y13))
			(T541 (bvand X18 Y14))
			(T542 (bvand X17 Y15))
			(T543 (bvand X16 Y16))
			(T544 (bvand X15 Y17))
			(T545 (bvand X14 Y18))
			(T546 (bvand X13 Y19))
			(T547 (bvand X12 Y20))
			(T548 (bvand X11 Y21))
			(T549 (bvand X10 Y22))
			(T550 (bvand X9 Y23))
			(T551 (bvand X8 Y24))
			(T552 (bvand X7 Y25))
			(T553 (bvand X6 Y26))
			(T554 (bvand X5 Y27))
			(T555 (bvand X4 Y28))
			(T556 (bvand X3 Y29))
			(T557 (bvand X2 Y30))
			(T558 (bvand X1 Y31))
			(T559 (bvand X31 Y2))
			(T560 (bvand X30 Y3))
			(T561 (bvand X29 Y4))
			(T562 (bvand X28 Y5))
			(T563 (bvand X27 Y6))
			(T564 (bvand X26 Y7))
			(T565 (bvand X25 Y8))
			(T566 (bvand X24 Y9))
			(T567 (bvand X23 Y10))
			(T568 (bvand X22 Y11))
			(T569 (bvand X21 Y12))
			(T570 (bvand X20 Y13))
			(T571 (bvand X19 Y14))
			(T572 (bvand X18 Y15))
			(T573 (bvand X17 Y16))
			(T574 (bvand X16 Y17))
			(T575 (bvand X15 Y18))
			(T576 (bvand X14 Y19))
			(T577 (bvand X13 Y20))
			(T578 (bvand X12 Y21))
			(T579 (bvand X11 Y22))
			(T580 (bvand X10 Y23))
			(T581 (bvand X9 Y24))
			(T582 (bvand X8 Y25))
			(T583 (bvand X7 Y26))
			(T584 (bvand X6 Y27))
			(T585 (bvand X5 Y28))
			(T586 (bvand X4 Y29))
			(T587 (bvand X3 Y30))
			(T588 (bvand X2 Y31))
			(T589 (bvand X31 Y3))
			(T590 (bvand X30 Y4))
			(T591 (bvand X29 Y5))
			(T592 (bvand X28 Y6))
			(T593 (bvand X27 Y7))
			(T594 (bvand X26 Y8))
			(T595 (bvand X25 Y9))
			(T596 (bvand X24 Y10))
			(T597 (bvand X23 Y11))
			(T598 (bvand X22 Y12))
			(T599 (bvand X21 Y13))
			(T600 (bvand X20 Y14))
			(T601 (bvand X19 Y15))
			(T602 (bvand X18 Y16))
			(T603 (bvand X17 Y17))
			(T604 (bvand X16 Y18))
			(T605 (bvand X15 Y19))
			(T606 (bvand X14 Y20))
			(T607 (bvand X13 Y21))
			(T608 (bvand X12 Y22))
			(T609 (bvand X11 Y23))
			(T610 (bvand X10 Y24))
			(T611 (bvand X9 Y25))
			(T612 (bvand X8 Y26))
			(T613 (bvand X7 Y27))
			(T614 (bvand X6 Y28))
			(T615 (bvand X5 Y29))
			(T616 (bvand X4 Y30))
			(T617 (bvand X3 Y31))
			(T618 (bvand X31 Y4))
			(T619 (bvand X30 Y5))
			(T620 (bvand X29 Y6))
			(T621 (bvand X28 Y7))
			(T622 (bvand X27 Y8))
			(T623 (bvand X26 Y9))
			(T624 (bvand X25 Y10))
			(T625 (bvand X24 Y11))
			(T626 (bvand X23 Y12))
			(T627 (bvand X22 Y13))
			(T628 (bvand X21 Y14))
			(T629 (bvand X20 Y15))
			(T630 (bvand X19 Y16))
			(T631 (bvand X18 Y17))
			(T632 (bvand X17 Y18))
			(T633 (bvand X16 Y19))
			(T634 (bvand X15 Y20))
			(T635 (bvand X14 Y21))
			(T636 (bvand X13 Y22))
			(T637 (bvand X12 Y23))
			(T638 (bvand X11 Y24))
			(T639 (bvand X10 Y25))
			(T640 (bvand X9 Y26))
			(T641 (bvand X8 Y27))
			(T642 (bvand X7 Y28))
			(T643 (bvand X6 Y29))
			(T644 (bvand X5 Y30))
			(T645 (bvand X4 Y31))
			(T646 (bvand X31 Y5))
			(T647 (bvand X30 Y6))
			(T648 (bvand X29 Y7))
			(T649 (bvand X28 Y8))
			(T650 (bvand X27 Y9))
			(T651 (bvand X26 Y10))
			(T652 (bvand X25 Y11))
			(T653 (bvand X24 Y12))
			(T654 (bvand X23 Y13))
			(T655 (bvand X22 Y14))
			(T656 (bvand X21 Y15))
			(T657 (bvand X20 Y16))
			(T658 (bvand X19 Y17))
			(T659 (bvand X18 Y18))
			(T660 (bvand X17 Y19))
			(T661 (bvand X16 Y20))
			(T662 (bvand X15 Y21))
			(T663 (bvand X14 Y22))
			(T664 (bvand X13 Y23))
			(T665 (bvand X12 Y24))
			(T666 (bvand X11 Y25))
			(T667 (bvand X10 Y26))
			(T668 (bvand X9 Y27))
			(T669 (bvand X8 Y28))
			(T670 (bvand X7 Y29))
			(T671 (bvand X6 Y30))
			(T672 (bvand X5 Y31))
			(T673 (bvand X31 Y6))
			(T674 (bvand X30 Y7))
			(T675 (bvand X29 Y8))
			(T676 (bvand X28 Y9))
			(T677 (bvand X27 Y10))
			(T678 (bvand X26 Y11))
			(T679 (bvand X25 Y12))
			(T680 (bvand X24 Y13))
			(T681 (bvand X23 Y14))
			(T682 (bvand X22 Y15))
			(T683 (bvand X21 Y16))
			(T684 (bvand X20 Y17))
			(T685 (bvand X19 Y18))
			(T686 (bvand X18 Y19))
			(T687 (bvand X17 Y20))
			(T688 (bvand X16 Y21))
			(T689 (bvand X15 Y22))
			(T690 (bvand X14 Y23))
			(T691 (bvand X13 Y24))
			(T692 (bvand X12 Y25))
			(T693 (bvand X11 Y26))
			(T694 (bvand X10 Y27))
			(T695 (bvand X9 Y28))
			(T696 (bvand X8 Y29))
			(T697 (bvand X7 Y30))
			(T698 (bvand X6 Y31))
			(T699 (bvand X31 Y7))
			(T700 (bvand X30 Y8))
			(T701 (bvand X29 Y9))
			(T702 (bvand X28 Y10))
			(T703 (bvand X27 Y11))
			(T704 (bvand X26 Y12))
			(T705 (bvand X25 Y13))
			(T706 (bvand X24 Y14))
			(T707 (bvand X23 Y15))
			(T708 (bvand X22 Y16))
			(T709 (bvand X21 Y17))
			(T710 (bvand X20 Y18))
			(T711 (bvand X19 Y19))
			(T712 (bvand X18 Y20))
			(T713 (bvand X17 Y21))
			(T714 (bvand X16 Y22))
			(T715 (bvand X15 Y23))
			(T716 (bvand X14 Y24))
			(T717 (bvand X13 Y25))
			(T718 (bvand X12 Y26))
			(T719 (bvand X11 Y27))
			(T720 (bvand X10 Y28))
			(T721 (bvand X9 Y29))
			(T722 (bvand X8 Y30))
			(T723 (bvand X7 Y31))
			(T724 (bvand X31 Y8))
			(T725 (bvand X30 Y9))
			(T726 (bvand X29 Y10))
			(T727 (bvand X28 Y11))
			(T728 (bvand X27 Y12))
			(T729 (bvand X26 Y13))
			(T730 (bvand X25 Y14))
			(T731 (bvand X24 Y15))
			(T732 (bvand X23 Y16))
			(T733 (bvand X22 Y17))
			(T734 (bvand X21 Y18))
			(T735 (bvand X20 Y19))
			(T736 (bvand X19 Y20))
			(T737 (bvand X18 Y21))
			(T738 (bvand X17 Y22))
			(T739 (bvand X16 Y23))
			(T740 (bvand X15 Y24))
			(T741 (bvand X14 Y25))
			(T742 (bvand X13 Y26))
			(T743 (bvand X12 Y27))
			(T744 (bvand X11 Y28))
			(T745 (bvand X10 Y29))
			(T746 (bvand X9 Y30))
			(T747 (bvand X8 Y31))
			(T748 (bvand X31 Y9))
			(T749 (bvand X30 Y10))
			(T750 (bvand X29 Y11))
			(T751 (bvand X28 Y12))
			(T752 (bvand X27 Y13))
			(T753 (bvand X26 Y14))
			(T754 (bvand X25 Y15))
			(T755 (bvand X24 Y16))
			(T756 (bvand X23 Y17))
			(T757 (bvand X22 Y18))
			(T758 (bvand X21 Y19))
			(T759 (bvand X20 Y20))
			(T760 (bvand X19 Y21))
			(T761 (bvand X18 Y22))
			(T762 (bvand X17 Y23))
			(T763 (bvand X16 Y24))
			(T764 (bvand X15 Y25))
			(T765 (bvand X14 Y26))
			(T766 (bvand X13 Y27))
			(T767 (bvand X12 Y28))
			(T768 (bvand X11 Y29))
			(T769 (bvand X10 Y30))
			(T770 (bvand X9 Y31))
			(T771 (bvand X31 Y10))
			(T772 (bvand X30 Y11))
			(T773 (bvand X29 Y12))
			(T774 (bvand X28 Y13))
			(T775 (bvand X27 Y14))
			(T776 (bvand X26 Y15))
			(T777 (bvand X25 Y16))
			(T778 (bvand X24 Y17))
			(T779 (bvand X23 Y18))
			(T780 (bvand X22 Y19))
			(T781 (bvand X21 Y20))
			(T782 (bvand X20 Y21))
			(T783 (bvand X19 Y22))
			(T784 (bvand X18 Y23))
			(T785 (bvand X17 Y24))
			(T786 (bvand X16 Y25))
			(T787 (bvand X15 Y26))
			(T788 (bvand X14 Y27))
			(T789 (bvand X13 Y28))
			(T790 (bvand X12 Y29))
			(T791 (bvand X11 Y30))
			(T792 (bvand X10 Y31))
			(T793 (bvand X31 Y11))
			(T794 (bvand X30 Y12))
			(T795 (bvand X29 Y13))
			(T796 (bvand X28 Y14))
			(T797 (bvand X27 Y15))
			(T798 (bvand X26 Y16))
			(T799 (bvand X25 Y17))
			(T800 (bvand X24 Y18))
			(T801 (bvand X23 Y19))
			(T802 (bvand X22 Y20))
			(T803 (bvand X21 Y21))
			(T804 (bvand X20 Y22))
			(T805 (bvand X19 Y23))
			(T806 (bvand X18 Y24))
			(T807 (bvand X17 Y25))
			(T808 (bvand X16 Y26))
			(T809 (bvand X15 Y27))
			(T810 (bvand X14 Y28))
			(T811 (bvand X13 Y29))
			(T812 (bvand X12 Y30))
			(T813 (bvand X11 Y31))
			(T814 (bvand X31 Y12))
			(T815 (bvand X30 Y13))
			(T816 (bvand X29 Y14))
			(T817 (bvand X28 Y15))
			(T818 (bvand X27 Y16))
			(T819 (bvand X26 Y17))
			(T820 (bvand X25 Y18))
			(T821 (bvand X24 Y19))
			(T822 (bvand X23 Y20))
			(T823 (bvand X22 Y21))
			(T824 (bvand X21 Y22))
			(T825 (bvand X20 Y23))
			(T826 (bvand X19 Y24))
			(T827 (bvand X18 Y25))
			(T828 (bvand X17 Y26))
			(T829 (bvand X16 Y27))
			(T830 (bvand X15 Y28))
			(T831 (bvand X14 Y29))
			(T832 (bvand X13 Y30))
			(T833 (bvand X12 Y31))
			(T834 (bvand X31 Y13))
			(T835 (bvand X30 Y14))
			(T836 (bvand X29 Y15))
			(T837 (bvand X28 Y16))
			(T838 (bvand X27 Y17))
			(T839 (bvand X26 Y18))
			(T840 (bvand X25 Y19))
			(T841 (bvand X24 Y20))
			(T842 (bvand X23 Y21))
			(T843 (bvand X22 Y22))
			(T844 (bvand X21 Y23))
			(T845 (bvand X20 Y24))
			(T846 (bvand X19 Y25))
			(T847 (bvand X18 Y26))
			(T848 (bvand X17 Y27))
			(T849 (bvand X16 Y28))
			(T850 (bvand X15 Y29))
			(T851 (bvand X14 Y30))
			(T852 (bvand X13 Y31))
			(T853 (bvand X31 Y14))
			(T854 (bvand X30 Y15))
			(T855 (bvand X29 Y16))
			(T856 (bvand X28 Y17))
			(T857 (bvand X27 Y18))
			(T858 (bvand X26 Y19))
			(T859 (bvand X25 Y20))
			(T860 (bvand X24 Y21))
			(T861 (bvand X23 Y22))
			(T862 (bvand X22 Y23))
			(T863 (bvand X21 Y24))
			(T864 (bvand X20 Y25))
			(T865 (bvand X19 Y26))
			(T866 (bvand X18 Y27))
			(T867 (bvand X17 Y28))
			(T868 (bvand X16 Y29))
			(T869 (bvand X15 Y30))
			(T870 (bvand X14 Y31))
			(T871 (bvand X31 Y15))
			(T872 (bvand X30 Y16))
			(T873 (bvand X29 Y17))
			(T874 (bvand X28 Y18))
			(T875 (bvand X27 Y19))
			(T876 (bvand X26 Y20))
			(T877 (bvand X25 Y21))
			(T878 (bvand X24 Y22))
			(T879 (bvand X23 Y23))
			(T880 (bvand X22 Y24))
			(T881 (bvand X21 Y25))
			(T882 (bvand X20 Y26))
			(T883 (bvand X19 Y27))
			(T884 (bvand X18 Y28))
			(T885 (bvand X17 Y29))
			(T886 (bvand X16 Y30))
			(T887 (bvand X15 Y31))
			(T888 (bvand X31 Y16))
			(T889 (bvand X30 Y17))
			(T890 (bvand X29 Y18))
			(T891 (bvand X28 Y19))
			(T892 (bvand X27 Y20))
			(T893 (bvand X26 Y21))
			(T894 (bvand X25 Y22))
			(T895 (bvand X24 Y23))
			(T896 (bvand X23 Y24))
			(T897 (bvand X22 Y25))
			(T898 (bvand X21 Y26))
			(T899 (bvand X20 Y27))
			(T900 (bvand X19 Y28))
			(T901 (bvand X18 Y29))
			(T902 (bvand X17 Y30))
			(T903 (bvand X16 Y31))
			(T904 (bvand X31 Y17))
			(T905 (bvand X30 Y18))
			(T906 (bvand X29 Y19))
			(T907 (bvand X28 Y20))
			(T908 (bvand X27 Y21))
			(T909 (bvand X26 Y22))
			(T910 (bvand X25 Y23))
			(T911 (bvand X24 Y24))
			(T912 (bvand X23 Y25))
			(T913 (bvand X22 Y26))
			(T914 (bvand X21 Y27))
			(T915 (bvand X20 Y28))
			(T916 (bvand X19 Y29))
			(T917 (bvand X18 Y30))
			(T918 (bvand X17 Y31))
			(T919 (bvand X31 Y18))
			(T920 (bvand X30 Y19))
			(T921 (bvand X29 Y20))
			(T922 (bvand X28 Y21))
			(T923 (bvand X27 Y22))
			(T924 (bvand X26 Y23))
			(T925 (bvand X25 Y24))
			(T926 (bvand X24 Y25))
			(T927 (bvand X23 Y26))
			(T928 (bvand X22 Y27))
			(T929 (bvand X21 Y28))
			(T930 (bvand X20 Y29))
			(T931 (bvand X19 Y30))
			(T932 (bvand X18 Y31))
			(T933 (bvand X31 Y19))
			(T934 (bvand X30 Y20))
			(T935 (bvand X29 Y21))
			(T936 (bvand X28 Y22))
			(T937 (bvand X27 Y23))
			(T938 (bvand X26 Y24))
			(T939 (bvand X25 Y25))
			(T940 (bvand X24 Y26))
			(T941 (bvand X23 Y27))
			(T942 (bvand X22 Y28))
			(T943 (bvand X21 Y29))
			(T944 (bvand X20 Y30))
			(T945 (bvand X19 Y31))
			(T946 (bvand X31 Y20))
			(T947 (bvand X30 Y21))
			(T948 (bvand X29 Y22))
			(T949 (bvand X28 Y23))
			(T950 (bvand X27 Y24))
			(T951 (bvand X26 Y25))
			(T952 (bvand X25 Y26))
			(T953 (bvand X24 Y27))
			(T954 (bvand X23 Y28))
			(T955 (bvand X22 Y29))
			(T956 (bvand X21 Y30))
			(T957 (bvand X20 Y31))
			(T958 (bvand X31 Y21))
			(T959 (bvand X30 Y22))
			(T960 (bvand X29 Y23))
			(T961 (bvand X28 Y24))
			(T962 (bvand X27 Y25))
			(T963 (bvand X26 Y26))
			(T964 (bvand X25 Y27))
			(T965 (bvand X24 Y28))
			(T966 (bvand X23 Y29))
			(T967 (bvand X22 Y30))
			(T968 (bvand X21 Y31))
			(T969 (bvand X31 Y22))
			(T970 (bvand X30 Y23))
			(T971 (bvand X29 Y24))
			(T972 (bvand X28 Y25))
			(T973 (bvand X27 Y26))
			(T974 (bvand X26 Y27))
			(T975 (bvand X25 Y28))
			(T976 (bvand X24 Y29))
			(T977 (bvand X23 Y30))
			(T978 (bvand X22 Y31))
			(T979 (bvand X31 Y23))
			(T980 (bvand X30 Y24))
			(T981 (bvand X29 Y25))
			(T982 (bvand X28 Y26))
			(T983 (bvand X27 Y27))
			(T984 (bvand X26 Y28))
			(T985 (bvand X25 Y29))
			(T986 (bvand X24 Y30))
			(T987 (bvand X23 Y31))
			(T988 (bvand X31 Y24))
			(T989 (bvand X30 Y25))
			(T990 (bvand X29 Y26))
			(T991 (bvand X28 Y27))
			(T992 (bvand X27 Y28))
			(T993 (bvand X26 Y29))
			(T994 (bvand X25 Y30))
			(T995 (bvand X24 Y31))
			(T996 (bvand X31 Y25))
			(T997 (bvand X30 Y26))
			(T998 (bvand X29 Y27))
			(T999 (bvand X28 Y28))
			(T1000 (bvand X27 Y29))
			(T1001 (bvand X26 Y30))
			(T1002 (bvand X25 Y31))
			(T1003 (bvand X31 Y26))
			(T1004 (bvand X30 Y27))
			(T1005 (bvand X29 Y28))
			(T1006 (bvand X28 Y29))
			(T1007 (bvand X27 Y30))
			(T1008 (bvand X26 Y31))
			(T1009 (bvand X31 Y27))
			(T1010 (bvand X30 Y28))
			(T1011 (bvand X29 Y29))
			(T1012 (bvand X28 Y30))
			(T1013 (bvand X27 Y31))
			(T1014 (bvand X31 Y28))
			(T1015 (bvand X30 Y29))
			(T1016 (bvand X29 Y30))
			(T1017 (bvand X28 Y31))
			(T1018 (bvand X31 Y29))
			(T1019 (bvand X30 Y30))
			(T1020 (bvand X29 Y31))
			(T1021 (bvand X31 Y30))
			(T1022 (bvand X30 Y31))
			(T1023 (bvand X31 Y31))
		)

	(let
		(
			(Z1 (HA_SUM T1 T2)) ;2nd bit
			(C0 (HA_CARRY T1 T2))
			(trojanout (bvshl trojan (_ bv1 64)))
		)
			
	(let
		(	
			(S1 (FA_SUM T3 T4 T5))
			(C1 (FA_CARRY T3 T4 T5))
		)
		
	(let
		(
			(Z2 (HA_SUM S1 C0)) ;3rd bit
			(C2 (HA_CARRY S1 C0))
		)
		
	(let
		(
			(S1 (FA_SUM T6 T7 T8))
			(C1 (FA_CARRY T6 T7 T8))
			(S2 (FA_SUM T9 C1 C2 ))
			(C2 (FA_CARRY T9 C1 C2))
		)
		
	(let
		(
			(Z3 (HA_SUM S1 S2)) ;4th bit
			(C3 (HA_CARRY S1 S2))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM T10 T11 T12))
			(C2 (FA_CARRY T10 T11 T12))
			(S3 (HA_SUM T13 T14))
			(C3 (HA_CARRY T13 T14))
		)
		
	(let
		(
			(Z4 (FA_SUM S1 S2 S3)) ;5th bit
			(C4 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM T15 T16 T17))
			(C2 (FA_CARRY T15 T16 T17))
			(S3 (FA_SUM T18 T19 T20))
			(C3 (FA_CARRY T18 T19 T20))
		)
		
	(let
		(
			(S4 (FA_SUM C4 S1 S3))
			(C4 (FA_CARRY C4 S1 S3))
		)
	
	(let
		(
			(Z5 (HA_SUM S2 S4)) ;6th bit
			(C5 (HA_CARRY S2 S4))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 T21))
			(C2 (FA_CARRY C4 C5 T21))
			(S3 (FA_SUM T22 T23 T24))
			(C3 (FA_CARRY T22 T23 T24))
		)
	
	(let
		(
			(S4 (FA_SUM S1 S2 T27))
			(C4 (FA_CARRY S1 S2 T27))
			(S5 (FA_SUM S3 T25 T26))
			(C5 (FA_CARRY S3 T25 T26))
		)
		
	(let
		(
			(Z6 (HA_SUM S4 S5)) ;7th bit
			(C6 (HA_CARRY S4 S5))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T28 T29 T30))
			(C3 (FA_CARRY T28 T29 T30))
			(S4 (FA_SUM T31 T32 T33))
			(C4 (FA_CARRY T31 T32 T33))
		)
		
	(let
		(
			(S5 (FA_SUM S1 S2 T35))
			(C5 (FA_CARRY S1 S2 T35))
			(S6 (FA_SUM S3 S4 T34))
			(C6 (FA_CARRY S3 S4 T34))
		)
		
	(let
		(
			(Z7 (HA_SUM S5 S6)) ;8th bit
			(C7 (HA_CARRY S5 S6))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T36 T37 T38))
			(C3 (FA_CARRY T36 T37 T38))
			(S4 (FA_SUM T39 T40 T41))
			(C4 (FA_CARRY T39 T40 T41))
			(S5 (FA_SUM T42 T43 T44))
			(C5 (FA_CARRY T42 T43 T44))
		)
		
	(let
		(
			(S6 (FA_SUM S1 S2 C7))
			(C6 (FA_CARRY S1 S2 C7))
			(S7 (FA_SUM S3 S4 S5))
			(C7 (FA_CARRY S3 S4 S5))
		)
		
	(let
		(
			(Z8 (HA_SUM S6 S7)) ;9th bit
			(C8 (HA_CARRY S6 S7))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 T45))
			(C3 (FA_CARRY C7 C8 T45))
			(S4 (FA_SUM T46 T47 T48))
			(C4 (FA_CARRY T46 T47 T48))
			(S5 (FA_SUM T49 T50 T51))
			(C5 (FA_CARRY T49 T50 T51))
			(S6 (FA_SUM T52 T53 T54))
			(C6 (FA_CARRY T52 T53 T54))
		)
		
	(let
		(
			(S7 (FA_SUM S1 S2 S3))
			(C7 (FA_CARRY S1 S2 S3))
			(S8 (FA_SUM S4 S5 S6))
			(C8 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z9 (HA_SUM S7 S8)) ;10th bit
			(C9 (HA_CARRY S7 S8))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T55 T56 T57))
			(C4 (FA_CARRY T55 T56 T57))
			(S5 (FA_SUM T58 T59 T60))
			(C5 (FA_CARRY T58 T59 T60))
			(S6 (FA_SUM T61 T62 T63))
			(C6 (FA_CARRY T61 T62 T63))
			(S7 (HA_SUM T64 T65))
			(C7 (HA_CARRY T64 T65))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z10 (FA_SUM S7 S8 S9)) ;11th bit
			(C10 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T66 T67 T68))
			(C4 (FA_CARRY T66 T67 T68))
			(S5 (FA_SUM T69 T70 T71))
			(C5 (FA_CARRY T69 T70 T71))
			(S6 (FA_SUM T72 T73 T74))
			(C6 (FA_CARRY T72 T73 T74))
		)
		
	(let
		(
			(S7 (FA_SUM S1 S2 S3))
			(C7 (FA_CARRY S1 S2 S3))
			(S8 (FA_SUM S4 S5 S6))
			(C8 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(S9 (FA_SUM S7 C10 T77))
			(C9 (FA_CARRY S7 C10 T77))
			(S10 (FA_SUM S8 T75 T76))
			(C10 (FA_CARRY S8 T75 T76))
		)
		
	(let
		(
			(Z11 (HA_SUM S9 S10)) ;12th bit
			(C11 (HA_CARRY S9 S10))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 T78))
			(C4 (FA_CARRY C10 C11 T78))
			(S5 (FA_SUM T79 T80 T81))
			(C5 (FA_CARRY T79 T80 T81))
			(S6 (FA_SUM T82 T83 T84))
			(C6 (FA_CARRY T82 T83 T84))
			(S7 (FA_SUM T85 T86 T87))
			(C7 (FA_CARRY T85 T86 T87))
			(S8 (FA_SUM T88 T89 T90))
			(C8 (FA_CARRY T88 T89 T90))
		)
		
	(let
		(
			(S9 (FA_SUM S1 S2 S3))
			(C9 (FA_CARRY S1 S2 S3))
			(S10 (FA_SUM S4 S5 S6))
			(C10 (FA_CARRY S4 S5 S6))
			(S11 (HA_SUM S7 S8))
			(C11 (HA_CARRY S7 S8))
		)
		
	(let
		(
			(Z12 (FA_SUM S9 S10 S11)) ;13th bit
			(C12 (FA_CARRY S9 S10 S11))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM T91 T92 T93))
			(C5 (FA_CARRY T91 T92 T93))
			(S6 (FA_SUM T94 T95 T96))
			(C6 (FA_CARRY T94 T95 T96))
			(S7 (FA_SUM T97 T98 T99))
			(C7 (FA_CARRY T97 T98 T99))
			(S8 (FA_SUM T100 T101 T102))
			(C8 (FA_CARRY T100 T101 T102))
			(S9 (HA_SUM T103 T104))
			(C9 (HA_CARRY T103 T104))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(Z13 (FA_SUM S10 S11 S12)) ;14th bit
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 T105 T106))
			(C5 (FA_CARRY C13 T105 T106))
			(S6 (FA_SUM T107 T108 T109))
			(C6 (FA_CARRY T107 T108 T109))
			(S7 (FA_SUM T110 T111 T112))
			(C7 (FA_CARRY T110 T111 T112))
			(S8 (FA_SUM T113 T114 T115))
			(C8 (FA_CARRY T113 T114 T115))
			(S9 (FA_SUM T116 T117 T118))
			(C9 (FA_CARRY T116 T117 T118))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S13 (FA_SUM S10 S11 S12))
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(Z14 (HA_SUM S13 T119)) ;15th bit
			(C14 (HA_CARRY S13 T119))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 T120))
			(C5 (FA_CARRY C13 C14 T120))
			(S6 (FA_SUM T121 T122 T123))
			(C6 (FA_CARRY T121 T122 T123))
			(S7 (FA_SUM T124 T125 T126))
			(C7 (FA_CARRY T124 T125 T126))
			(S8 (FA_SUM T127 T128 T129))
			(C8 (FA_CARRY T127 T128 T129))
			(S9 (FA_SUM T130 T131 T132))
			(C9 (FA_CARRY T130 T131 T132))
			(S10 (FA_SUM T133 T134 T135))
			(C10 (FA_CARRY T133 T134 T135))
		)
		
	(let
		(
			(S11 (FA_SUM S1 S2 S3))
			(C11 (FA_CARRY S1 S2 S3))
			(S12 (FA_SUM S4 S5 S6))
			(C12 (FA_CARRY S4 S5 S6))
			(S13 (FA_SUM S7 S8 S9))
			(C13 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S14 (FA_SUM S11 S12 S13))
			(C14 (FA_CARRY S11 S12 S13))
		)
		
	(let
		(
			(Z15 (HA_SUM S10 S14)) ;16th bit
			(C15 (HA_CARRY S10 S14))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM T136 T137 T138))
			(C6 (FA_CARRY T136 T137 T138))
			(S7 (FA_SUM T139 T140 T141))
			(C7 (FA_CARRY T139 T140 T141))
			(S8 (FA_SUM T142 T143 T144))
			(C8 (FA_CARRY T142 T143 T144))
			(S9 (FA_SUM T145 T146 T147))
			(C9 (FA_CARRY T145 T146 T147))
			(S10 (FA_SUM T148 T149 T150))
			(C10 (FA_CARRY T148 T149 T150))
		)
		
	(let
		(
			(S11 (FA_SUM S1 S2 S3))
			(C11 (FA_CARRY S1 S2 S3))
			(S12 (FA_SUM S4 S5 S6))
			(C12 (FA_CARRY S4 S5 S6))
			(S13 (FA_SUM S7 S8 S9))
			(C13 (FA_CARRY S7 S8 S9))
			(S14 (FA_SUM S10 T151 T152))
			(C14 (FA_CARRY S10 T151 T152))
		)
		
	(let
		(
			(S15 (FA_SUM S11 S12 S13))
			(C15 (FA_CARRY S11 S12 S13))
		)
		
	(let
		(
			(Z16 (HA_SUM S14 S15)) ;17th bit
			(C16 (HA_CARRY S14 S15))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 T153 T154))
			(C6 (FA_CARRY C16 T153 T154))
			(S7 (FA_SUM T155 T156 T157))
			(C7 (FA_CARRY T155 T156 T157))
			(S8 (FA_SUM T158 T159 T160))
			(C8 (FA_CARRY T158 T159 T160))
			(S9 (FA_SUM T161 T162 T163))
			(C9 (FA_CARRY T161 T162 T163))
			(S10 (FA_SUM T164 T165 T166))
			(C10 (FA_CARRY T164 T165 T166))
			(S11 (FA_SUM T167 T168 T169))
			(C11 (FA_CARRY T167 T168 T169))
		)
		
	(let
		(
			(S12 (FA_SUM S1 S2 S3))
			(C12 (FA_CARRY S1 S2 S3))
			(S13 (FA_SUM S4 S5 S6))
			(C13 (FA_CARRY S4 S5 S6))
			(S14 (FA_SUM S7 S8 S9))
			(C14 (FA_CARRY S7 S8 S9))
			(S15 (FA_SUM S10 S11 T170))
			(C15 (FA_CARRY S10 S11 T170))
		)
		
	(let
		(
			(S16 (FA_SUM S12 S13 S14))
			(C16 (FA_CARRY S12 S13 S14))
		)
		
	(let
		(
			(Z17 (HA_SUM S15 S16)) ;18th bit
			(C17 (HA_CARRY S15 S16))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 T171))
			(C6 (FA_CARRY C16 C17 T171))
			(S7 (FA_SUM T172 T173 T174))
			(C7 (FA_CARRY T172 T173 T174))
			(S8 (FA_SUM T175 T176 T177))
			(C8 (FA_CARRY T175 T176 T177))
			(S9 (FA_SUM T178 T179 T180))
			(C9 (FA_CARRY T178 T179 T180))
			(S10 (FA_SUM T181 T182 T183))
			(C10 (FA_CARRY T181 T182 T183))
			(S11 (FA_SUM T184 T185 T186))
			(C11 (FA_CARRY T184 T185 T186))
			(S12 (FA_SUM T187 T188 T189))
			(C12 (FA_CARRY T187 T188 T189))
		)
		
	(let
		(
			(S13 (FA_SUM S1 S2 S3))
			(C13 (FA_CARRY S1 S2 S3))
			(S14 (FA_SUM S4 S5 S6))
			(C14 (FA_CARRY S4 S5 S6))
			(S15 (FA_SUM S7 S8 S9))
			(C15 (FA_CARRY S7 S8 S9))
			(S16 (FA_SUM S10 S11 S12))
			(C16 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S17 (FA_SUM S13 S14 S15))
			(C17 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(Z18 (HA_SUM S16 S17)) ;19th bit
			(C18 (HA_CARRY S16 S17))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM T190 T191 T192))
			(C7 (FA_CARRY T190 T191 T192))
			(S8 (FA_SUM T193 T194 T195))
			(C8 (FA_CARRY T193 T194 T195))
			(S9 (FA_SUM T196 T197 T198))
			(C9 (FA_CARRY T196 T197 T198))
			(S10 (FA_SUM T199 T200 T201))
			(C10 (FA_CARRY T199 T200 T201))
			(S11 (FA_SUM T202 T203 T204))
			(C11 (FA_CARRY T202 T203 T204))
			(S12 (FA_SUM T205 T206 T207))
			(C12 (FA_CARRY T205 T206 T207))
		)
		
	(let
		(
			(S13 (FA_SUM S1 S2 S3))
			(C13 (FA_CARRY S1 S2 S3))
			(S14 (FA_SUM S4 S5 S6))
			(C14 (FA_CARRY S4 S5 S6))
			(S15 (FA_SUM S7 S8 S9))
			(C15 (FA_CARRY S7 S8 S9))
			(S16 (FA_SUM S10 S11 S12))
			(C16 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S17 (FA_SUM S13 S14 S15))
			(C17 (FA_CARRY S13 S14 S15))
			(S18 (FA_SUM S16 T208 T209))
			(C18 (FA_CARRY S16 T208 T209))
		)
		
	(let
		(
			(Z19 (HA_SUM S17 S18)) ;20th bit
			(C19 (HA_CARRY S17 S18))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 T210 T211))
			(C7 (FA_CARRY C19 T210 T211))
			(S8 (FA_SUM T212 T213 T214))
			(C8 (FA_CARRY T212 T213 T214))
			(S9 (FA_SUM T215 T216 T217))
			(C9 (FA_CARRY T215 T216 T217))
			(S10 (FA_SUM T218 T219 T220))
			(C10 (FA_CARRY T218 T219 T220))
			(S11 (FA_SUM T221 T222 T223))
			(C11 (FA_CARRY T221 T222 T223))
			(S12 (FA_SUM T224 T225 T226))
			(C12 (FA_CARRY T224 T225 T226))
			(S13 (FA_SUM T227 T228 T229))
			(C13 (FA_CARRY T227 T228 T229))
		)
		
	(let
		(
			(S14 (FA_SUM S1 S2 S3))
			(C14 (FA_CARRY S1 S2 S3))
			(S15 (FA_SUM S4 S5 S6))
			(C15 (FA_CARRY S4 S5 S6))
			(S16 (FA_SUM S7 S8 S9))
			(C16 (FA_CARRY S7 S8 S9))
			(S17 (FA_SUM S10 S11 S12))
			(C17 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S18 (FA_SUM S14 S15 S16))
			(C18 (FA_CARRY S14 S15 S16))
			(S19 (FA_SUM S13 S17 T230))
			(C19 (FA_CARRY S13 S17 T230))
		)
		
	(let
		(
			(Z20 (HA_SUM S18 S19)) ;21st bit
			(C20 (HA_CARRY S18 S19))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 T231))
			(C7 (FA_CARRY C19 C20 T231))
			(S8 (FA_SUM T232 T233 T234))
			(C8 (FA_CARRY T232 T233 T234))
			(S9 (FA_SUM T235 T236 T237))
			(C9 (FA_CARRY T235 T236 T237))
			(S10 (FA_SUM T238 T239 T240))
			(C10 (FA_CARRY T238 T239 T240))
			(S11 (FA_SUM T241 T242 T243))
			(C11 (FA_CARRY T241 T242 T243))
			(S12 (FA_SUM T244 T245 T246))
			(C12 (FA_CARRY T244 T245 T246))
			(S13 (FA_SUM T247 T248 T249))
			(C13 (FA_CARRY T247 T248 T249))
			(S14 (FA_SUM T250 T251 T252))
			(C14 (FA_CARRY T250 T251 T252))
		)
		
	(let
		(
			(S15 (FA_SUM S1 S2 S3))
			(C15 (FA_CARRY S1 S2 S3))
			(S16 (FA_SUM S4 S5 S6))
			(C16 (FA_CARRY S4 S5 S6))
			(S17 (FA_SUM S7 S8 S9))
			(C17 (FA_CARRY S7 S8 S9))
			(S18 (FA_SUM S10 S11 S12))
			(C18 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S19 (FA_SUM S13 S14 S15))
			(C19 (FA_CARRY S13 S14 S15))
			(S20 (FA_SUM S16 S17 S18))
			(C20 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(Z21 (HA_SUM S19 S20)) ;22nd bit
			(C21 (HA_CARRY S19 S20))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM T253 T254 T255))
			(C8 (FA_CARRY T253 T254 T255))
			(S9 (FA_SUM T256 T257 T258))
			(C9 (FA_CARRY T256 T257 T258))
			(S10 (FA_SUM T259 T260 T261))
			(C10 (FA_CARRY T259 T260 T261))
			(S11 (FA_SUM T262 T263 T264))
			(C11 (FA_CARRY T262 T263 T264))
			(S12 (FA_SUM T265 T266 T267))
			(C12 (FA_CARRY T265 T266 T267))
			(S13 (FA_SUM T268 T269 T270))
			(C13 (FA_CARRY T268 T269 T270))
			(S14 (FA_SUM T271 T272 T273))
			(C14 (FA_CARRY T271 T272 T273))
		)
		
	(let
		(
			(S15 (FA_SUM S1 S2 S3))
			(C15 (FA_CARRY S1 S2 S3))
			(S16 (FA_SUM S4 S5 S6))
			(C16 (FA_CARRY S4 S5 S6))
			(S17 (FA_SUM S7 S8 S9))
			(C17 (FA_CARRY S7 S8 S9))
			(S18 (FA_SUM S10 S11 S12))
			(C18 (FA_CARRY S10 S11 S12))
			(S19 (FA_SUM S13 S14 T274))
			(C19 (FA_CARRY S13 S14 T274))
		)
		
	(let
		(
			(S20 (FA_SUM S15 S16 S17))
			(C20 (FA_CARRY S15 S16 S17))
			(S21 (FA_SUM S18 S19 T275))
			(C21 (FA_CARRY S18 S19 T275))
		)
		
	(let
		(
			(Z22 (HA_SUM S20 S21)) ;23rd bit
			(C22 (HA_CARRY S20 S21))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 T276 T277))
			(C8 (FA_CARRY C22 T276 T277))
			(S9 (FA_SUM T278 T279 T280))
			(C9 (FA_CARRY T278 T279 T280))
			(S10 (FA_SUM T281 T282 T283))
			(C10 (FA_CARRY T281 T282 T283))
			(S11 (FA_SUM T284 T285 T286))
			(C11 (FA_CARRY T284 T285 T286))
			(S12 (FA_SUM T287 T288 T289))
			(C12 (FA_CARRY T287 T288 T289))
			(S13 (FA_SUM T290 T291 T292))
			(C13 (FA_CARRY T290 T291 T292))
			(S14 (FA_SUM T293 T294 T295))
			(C14 (FA_CARRY T293 T294 T295))
			(S15 (FA_SUM T296 T297 T298))
			(C15 (FA_CARRY T296 T297 T298))
		)
		
	(let
		(
			(S16 (FA_SUM S1 S2 S3))
			(C16 (FA_CARRY S1 S2 S3))
			(S17 (FA_SUM S4 S5 S6))
			(C17 (FA_CARRY S4 S5 S6))
			(S18 (FA_SUM S7 S8 S9))
			(C18 (FA_CARRY S7 S8 S9))
			(S19 (FA_SUM S10 S11 S12))
			(C19 (FA_CARRY S10 S11 S12))
			(S20 (FA_SUM S13 S14 S15))
			(C20 (FA_CARRY S13 S14 S15))
		)
			
	(let
		(
			(S21 (FA_SUM S16 S17 S18))
			(C21 (FA_CARRY S16 S17 S18))
			(S22 (FA_SUM S19 S20 T299))
			(C22 (FA_CARRY S19 S20 T299))
		)
		
	(let
		(
			(Z23 (HA_SUM S21 S22)) ;24th bit
			(C23 (HA_CARRY S21 S22))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 T300))
			(C8 (FA_CARRY C22 C23 T300))
			(S9 (FA_SUM T301 T302 T303))
			(C9 (FA_CARRY T301 T302 T303))
			(S10 (FA_SUM T304 T305 T306))
			(C10 (FA_CARRY T304 T305 T306))
			(S11 (FA_SUM T307 T308 T309))
			(C11 (FA_CARRY T307 T308 T309))
			(S12 (FA_SUM T310 T311 T312))
			(C12 (FA_CARRY T310 T311 T312))
			(S13 (FA_SUM T313 T314 T315))
			(C13 (FA_CARRY T313 T314 T315))
			(S14 (FA_SUM T316 T317 T318))
			(C14 (FA_CARRY T316 T317 T318))
			(S15 (FA_SUM T319 T320 T321))
			(C15 (FA_CARRY T319 T320 T321))
			(S16 (FA_SUM T322 T323 T324))
			(C16 (FA_CARRY T322 T323 T324))
		)
		
	(let
		(
			(S17 (FA_SUM S1 S2 S3))
			(C17 (FA_CARRY S1 S2 S3))
			(S18 (FA_SUM S4 S5 S6))
			(C18 (FA_CARRY S4 S5 S6))
			(S19 (FA_SUM S7 S8 S9))
			(C19 (FA_CARRY S7 S8 S9))
			(S20 (FA_SUM S10 S11 S12))
			(C20 (FA_CARRY S10 S11 S12))
			(S21 (FA_SUM S13 S14 S15))
			(C21 (FA_CARRY S13 S14 S15))
		)
			
	(let
		(
			(S22 (FA_SUM S17 S18 S16))
			(C22 (FA_CARRY S17 S18 S16))
			(S23 (FA_SUM S19 S20 S21))
			(C23 (FA_CARRY S19 S20 S21))
		)
		
	(let
		(
			(Z24 (HA_SUM S22 S23)) ;25th bit
			(C24 (HA_CARRY S22 S23))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM T325 T326 T327))
			(C9 (FA_CARRY T325 T326 T327))
			(S10 (FA_SUM T328 T329 T330))
			(C10 (FA_CARRY T328 T329 T330))
			(S11 (FA_SUM T331 T332 T333))
			(C11 (FA_CARRY T331 T332 T333))
			(S12 (FA_SUM T334 T335 T336))
			(C12 (FA_CARRY T334 T335 T336))
			(S13 (FA_SUM T337 T338 T339))
			(C13 (FA_CARRY T337 T338 T339))
			(S14 (FA_SUM T340 T341 T342))
			(C14 (FA_CARRY T340 T341 T342))
			(S15 (FA_SUM T343 T344 T345))
			(C15 (FA_CARRY T343 T344 T345))
			(S16 (FA_SUM T346 T347 T348))
			(C16 (FA_CARRY T346 T347 T348))
		)
		
	(let
		(
			(S17 (FA_SUM S1 S2 S3))
			(C17 (FA_CARRY S1 S2 S3))
			(S18 (FA_SUM S4 S5 S6))
			(C18 (FA_CARRY S4 S5 S6))
			(S19 (FA_SUM S7 S8 S9))
			(C19 (FA_CARRY S7 S8 S9))
			(S20 (FA_SUM S10 S11 S12))
			(C20 (FA_CARRY S10 S11 S12))
			(S21 (FA_SUM S13 S14 S15))
			(C21 (FA_CARRY S13 S14 S15))
			(S22 (FA_SUM S16 T349 T350))
			(C22 (FA_CARRY S16 T349 T350))
		)
		
	(let
		(
			(S23 (FA_SUM S17 S18 S19))
			(C23 (FA_CARRY S17 S18 S19))
			(S24 (FA_SUM S20 S21 S22))
			(C24 (FA_CARRY S20 S21 S22))
		)
		
	(let
		(
			(Z25 (HA_SUM S23 S24)) ;26th bit
			(C25 (HA_CARRY S23 S24))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 T351 T352))
			(C9 (FA_CARRY C25 T351 T352))
			(S10 (FA_SUM T353 T354 T355))
			(C10 (FA_CARRY T353 T354 T355))
			(S11 (FA_SUM T356 T357 T358))
			(C11 (FA_CARRY T356 T357 T358))
			(S12 (FA_SUM T359 T360 T361))
			(C12 (FA_CARRY T359 T360 T361))
			(S13 (FA_SUM T362 T363 T364))
			(C13 (FA_CARRY T362 T363 T364))
			(S14 (FA_SUM T365 T366 T367))
			(C14 (FA_CARRY T365 T366 T367))
			(S15 (FA_SUM T368 T369 T370))
			(C15 (FA_CARRY T368 T369 T370))
			(S16 (FA_SUM T371 T372 T373))
			(C16 (FA_CARRY T371 T372 T373))
			(S17 (FA_SUM T374 T375 T376))
			(C17 (FA_CARRY T374 T375 T376))
		)
		
	(let
		(
			(S18 (FA_SUM S1 S2 S3))
			(C18 (FA_CARRY S1 S2 S3))
			(S19 (FA_SUM S4 S5 S6))
			(C19 (FA_CARRY S4 S5 S6))
			(S20 (FA_SUM S7 S8 S9))
			(C20 (FA_CARRY S7 S8 S9))
			(S21 (FA_SUM S10 S11 S12))
			(C21 (FA_CARRY S10 S11 S12))
			(S22 (FA_SUM S13 S14 S15))
			(C22 (FA_CARRY S13 S14 S15))
			(S23 (FA_SUM S16 S17 T377))
			(C23 (FA_CARRY S16 S17 T377))
		)
		
	(let
		(
			(S24 (FA_SUM S18 S19 S20))
			(C24 (FA_CARRY S18 S19 S20))
			(S25 (FA_SUM S21 S22 S23))
			(C25 (FA_CARRY S21 S22 S23))
		)
		
	(let
		(
			(Z26 (HA_SUM S24 S25)) ;27th bit
			(C26 (HA_CARRY S24 S25))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 T378))
			(C9 (FA_CARRY C25 C26 T378))
			(S10 (FA_SUM T379 T380 T381))
			(C10 (FA_CARRY T379 T380 T381))
			(S11 (FA_SUM T382 T383 T384))
			(C11 (FA_CARRY T382 T383 T384))
			(S12 (FA_SUM T385 T386 T387))
			(C12 (FA_CARRY T385 T386 T387))
			(S13 (FA_SUM T388 T389 T390))
			(C13 (FA_CARRY T388 T389 T390))
			(S14 (FA_SUM T391 T392 T393))
			(C14 (FA_CARRY T391 T392 T393))
			(S15 (FA_SUM T394 T395 T396))
			(C15 (FA_CARRY T394 T395 T396))
			(S16 (FA_SUM T397 T398 T399))
			(C16 (FA_CARRY T397 T398 T399))
			(S17 (FA_SUM T400 T401 T402))
			(C17 (FA_CARRY T400 T401 T402))
			(S18 (FA_SUM T403 T404 T405))
			(C18 (FA_CARRY T403 T404 T405))
		)
		
	(let
		(
			(S19 (FA_SUM S1 S2 S3))
			(C19 (FA_CARRY S1 S2 S3))
			(S20 (FA_SUM S4 S5 S6))
			(C20 (FA_CARRY S4 S5 S6))
			(S21 (FA_SUM S7 S8 S9))
			(C21 (FA_CARRY S7 S8 S9))
			(S22 (FA_SUM S10 S11 S12))
			(C22 (FA_CARRY S10 S11 S12))
			(S23 (FA_SUM S13 S14 S15))
			(C23 (FA_CARRY S13 S14 S15))
			(S24 (FA_SUM S16 S17 S18))
			(C24 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(S25 (FA_SUM S19 S20 S21))
			(C25 (FA_CARRY S19 S20 S21))
			(S26 (FA_SUM S22 S23 S24))
			(C26 (FA_CARRY S22 S23 S24))
		)
		
	(let
		(
			(Z27 (HA_SUM S25 S26)) ;28th bit
			(C27 (HA_CARRY S25 S26))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM T406 T407 T408))
			(C10 (FA_CARRY T406 T407 T408))
			(S11 (FA_SUM T409 T410 T411))
			(C11 (FA_CARRY T409 T410 T411))
			(S12 (FA_SUM T412 T413 T414))
			(C12 (FA_CARRY T412 T413 T414))
			(S13 (FA_SUM T415 T416 T417))
			(C13 (FA_CARRY T415 T416 T417))
			(S14 (FA_SUM T418 T419 T420))
			(C14 (FA_CARRY T418 T419 T420))
			(S15 (FA_SUM T421 T422 T423))
			(C15 (FA_CARRY T421 T422 T423))
			(S16 (FA_SUM T424 T425 T426))
			(C16 (FA_CARRY T424 T425 T426))
			(S17 (FA_SUM T427 T428 T429))
			(C17 (FA_CARRY T427 T428 T429))
			(S18 (FA_SUM T430 T431 T432))
			(C18 (FA_CARRY T430 T431 T432))
		)
		
	(let
		(
			(S19 (FA_SUM S1 S2 S3))
			(C19 (FA_CARRY S1 S2 S3))
			(S20 (FA_SUM S4 S5 S6))
			(C20 (FA_CARRY S4 S5 S6))
			(S21 (FA_SUM S7 S8 S9))
			(C21 (FA_CARRY S7 S8 S9))
			(S22 (FA_SUM S10 S11 S12))
			(C22 (FA_CARRY S10 S11 S12))
			(S23 (FA_SUM S13 S14 S15))
			(C23 (FA_CARRY S13 S14 S15))
			(S24 (FA_SUM S16 S17 S18))
			(C24 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(S25 (FA_SUM S19 S20 S21))
			(C25 (FA_CARRY S19 S20 S21))
			(S26 (FA_SUM S22 S23 S24))
			(C26 (FA_CARRY S22 S23 S24))
			(S27 (HA_SUM T433 T434))
			(C27 (HA_CARRY T433 T434))
		)
		
	(let
		(
			(Z28 (FA_SUM S25 S26 S27)) ;29th bit
			(C28 (FA_CARRY S25 S26 S27))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM C28 T435 T436))
			(C10 (FA_CARRY C28 T435 T436))
			(S11 (FA_SUM T437 T438 T439))
			(C11 (FA_CARRY T437 T438 T439))
			(S12 (FA_SUM T440 T441 T442))
			(C12 (FA_CARRY T440 T441 T442))
			(S13 (FA_SUM T443 T444 T445))
			(C13 (FA_CARRY T443 T444 T445))
			(S14 (FA_SUM T446 T447 T448))
			(C14 (FA_CARRY T446 T447 T448))
			(S15 (FA_SUM T449 T450 T451))
			(C15 (FA_CARRY T449 T450 T451))
			(S16 (FA_SUM T452 T453 T454))
			(C16 (FA_CARRY T452 T453 T454))
			(S17 (FA_SUM T455 T456 T457))
			(C17 (FA_CARRY T455 T456 T457))
			(S18 (FA_SUM T458 T459 T460))
			(C18 (FA_CARRY T458 T459 T460))
			(S19 (FA_SUM T461 T462 T463))
			(C19 (FA_CARRY T461 T462 T463))
		)
		
	(let
		(
			(S20 (FA_SUM S1 S2 S3))
			(C20 (FA_CARRY S1 S2 S3))
			(S21 (FA_SUM S4 S5 S6))
			(C21 (FA_CARRY S4 S5 S6))
			(S22 (FA_SUM S7 S8 S9))
			(C22 (FA_CARRY S7 S8 S9))
			(S23 (FA_SUM S10 S11 S12))
			(C23 (FA_CARRY S10 S11 S12))
			(S24 (FA_SUM S13 S14 S15))
			(C24 (FA_CARRY S13 S14 S15))
			(S25 (FA_SUM S16 S17 S18))
			(C25 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(S26 (FA_SUM S20 S21 S22))
			(C26 (FA_CARRY S20 S21 S22))
			(S27 (FA_SUM S23 S24 S25))
			(C27 (FA_CARRY S23 S24 S25))
			(S28 (HA_SUM S19 T464))
			(C28 (HA_CARRY S19 T464))
		)
		
	(let
		(
			(Z29 (FA_SUM S26 S27 S28)) ;30th bit
			(C29 (FA_CARRY S26 S27 S28))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM C28 C29 T465))
			(C10 (FA_CARRY C28 C29 T465))
			(S11 (FA_SUM T466 T467 T468))
			(C11 (FA_CARRY T466 T467 T468))
			(S12 (FA_SUM T469 T470 T471))
			(C12 (FA_CARRY T469 T470 T471))
			(S13 (FA_SUM T472 T473 T474))
			(C13 (FA_CARRY T472 T473 T474))
			(S14 (FA_SUM T475 T476 T477))
			(C14 (FA_CARRY T475 T476 T477))
			(S15 (FA_SUM T478 T479 T480))
			(C15 (FA_CARRY T478 T479 T480))
			(S16 (FA_SUM T481 T482 T483))
			(C16 (FA_CARRY T481 T482 T483))
			(S17 (FA_SUM T484 T485 T486))
			(C17 (FA_CARRY T484 T485 T486))
			(S18 (FA_SUM T487 T488 T489))
			(C18 (FA_CARRY T487 T488 T489))
			(S19 (FA_SUM T490 T491 T492))
			(C19 (FA_CARRY T490 T491 T492))
			(S20 (FA_SUM T493 T494 T495))
			(C20 (FA_CARRY T493 T494 T495))
		)
		
	(let
		(
			(S21 (FA_SUM S1 S2 S3))
			(C21 (FA_CARRY S1 S2 S3))
			(S22 (FA_SUM S4 S5 S6))
			(C22 (FA_CARRY S4 S5 S6))
			(S23 (FA_SUM S7 S8 S9))
			(C23 (FA_CARRY S7 S8 S9))
			(S24 (FA_SUM S10 S11 S12))
			(C24 (FA_CARRY S10 S11 S12))
			(S25 (FA_SUM S13 S14 S15))
			(C25 (FA_CARRY S13 S14 S15))
			(S26 (FA_SUM S16 S17 S18))
			(C26 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(S27 (FA_SUM S21 S22 S23))
			(C27 (FA_CARRY S21 S22 S23))
			(S28 (FA_SUM S24 S25 S26))
			(C28 (FA_CARRY S24 S25 S26))
			(S29 (HA_SUM S19 S20))
			(C29 (HA_CARRY S19 S20))
		)
		
	(let
		(
			(Z30 (FA_SUM S27 S28 S29)) ;31st bit
			(C30 (FA_CARRY S27 S28 S29))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM C28 C29 C30))
			(C10 (FA_CARRY C28 C29 C30))
			(S11 (FA_SUM T496 T497 T498))
			(C11 (FA_CARRY T496 T497 T498))
			(S12 (FA_SUM T499 T500 T501))
			(C12 (FA_CARRY T499 T500 T501))
			(S13 (FA_SUM T502 T503 T504))
			(C13 (FA_CARRY T502 T503 T504))
			(S14 (FA_SUM T505 T506 T507))
			(C14 (FA_CARRY T505 T506 T507))
			(S15 (FA_SUM T508 T509 T510))
			(C15 (FA_CARRY T508 T509 T510))
			(S16 (FA_SUM T511 T512 T513))
			(C16 (FA_CARRY T511 T512 T513))
			(S17 (FA_SUM T514 T515 T516))
			(C17 (FA_CARRY T514 T515 T516))
			(S18 (FA_SUM T517 T518 T519))
			(C18 (FA_CARRY T517 T518 T519))
			(S19 (FA_SUM T520 T521 T522))
			(C19 (FA_CARRY T520 T521 T522))
			(S20 (FA_SUM T523 T524 T525))
			(C20 (FA_CARRY T523 T524 T525))
		)
		
	(let
		(
			(S21 (FA_SUM S1 S2 S3))
			(C21 (FA_CARRY S1 S2 S3))
			(S22 (FA_SUM S4 S5 S6))
			(C22 (FA_CARRY S4 S5 S6))
			(S23 (FA_SUM S7 S8 S9))
			(C23 (FA_CARRY S7 S8 S9))
			(S24 (FA_SUM S10 S11 S12))
			(C24 (FA_CARRY S10 S11 S12))
			(S25 (FA_SUM S13 S14 S15))
			(C25 (FA_CARRY S13 S14 S15))
			(S26 (FA_SUM S16 S17 S18))
			(C26 (FA_CARRY S16 S17 S18))
			(S27 (FA_SUM S19 S20 T526))
			(C27 (FA_CARRY S19 S20 T526))
		)
		
	(let
		(
			(S28 (FA_SUM S21 S22 S23))
			(C28 (FA_CARRY S21 S22 S23))
			(S29 (FA_SUM S24 S25 S26))
			(C29 (FA_CARRY S24 S25 S26))
			(S30 (HA_SUM S27 T527))
			(C30 (HA_CARRY S27 T527))
		)
		
	(let
		(
			(Z31 (FA_SUM S28 S29 S30)) ;32nd bit
			(C31 (FA_CARRY S28 S29 S30))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM C28 C29 C30))
			(C10 (FA_CARRY C28 C29 C30))
			(S11 (FA_SUM C31 T528 T529))
			(C11 (FA_CARRY C31 T528 T529))
			(S12 (FA_SUM T530 T531 T532))
			(C12 (FA_CARRY T530 T531 T532))
			(S13 (FA_SUM T533 T534 T535))
			(C13 (FA_CARRY T533 T534 T535))
			(S14 (FA_SUM T536 T537 T538))
			(C14 (FA_CARRY T536 T537 T538))
			(S15 (FA_SUM T539 T540 T541))
			(C15 (FA_CARRY T539 T540 T541))
			(S16 (FA_SUM T542 T543 T544))
			(C16 (FA_CARRY T542 T543 T544))
			(S17 (FA_SUM T545 T546 T547))
			(C17 (FA_CARRY T545 T546 T547))
			(S18 (FA_SUM T548 T549 T550))
			(C18 (FA_CARRY T548 T549 T550))
			(S19 (FA_SUM T551 T552 T553))
			(C19 (FA_CARRY T551 T552 T553))
			(S20 (FA_SUM T554 T555 T556))
			(C20 (FA_CARRY T554 T555 T556))
		)
		
	(let
		(
			(S21 (FA_SUM S1 S2 S3))
			(C21 (FA_CARRY S1 S2 S3))
			(S22 (FA_SUM S4 S5 S6))
			(C22 (FA_CARRY S4 S5 S6))
			(S23 (FA_SUM S7 S8 S9))
			(C23 (FA_CARRY S7 S8 S9))
			(S24 (FA_SUM S10 S11 S12))
			(C24 (FA_CARRY S10 S11 S12))
			(S25 (FA_SUM S13 S14 S15))
			(C25 (FA_CARRY S13 S14 S15))
			(S26 (FA_SUM S16 S17 S18))
			(C26 (FA_CARRY S16 S17 S18))
			(S27 (FA_SUM S19 S20 T557))
			(C27 (FA_CARRY S19 S20 T557))
		)
		
	(let
		(
			(S28 (FA_SUM S21 S22 S23))
			(C28 (FA_CARRY S21 S22 S23))
			(S29 (FA_SUM S24 S25 S26))
			(C29 (FA_CARRY S24 S25 S26))
			(S30 (HA_SUM S27 T558))
			(C30 (HA_CARRY S27 T558))
		)
		
	(let
		(
			(Z32 (FA_SUM S28 S29 S30)) ;33rd bit
			(C31 (FA_CARRY S28 S29 S30))
		)
				
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM C28 C29 C30))
			(C10 (FA_CARRY C28 C29 C30))
			(S11 (FA_SUM C31 T559 T560))
			(C11 (FA_CARRY C31 T559 T560))
			(S12 (FA_SUM T561 T562 T563))
			(C12 (FA_CARRY T561 T562 T563))
			(S13 (FA_SUM T564 T565 T566))
			(C13 (FA_CARRY T564 T565 T566))
			(S14 (FA_SUM T567 T568 T569))
			(C14 (FA_CARRY T567 T568 T569))
			(S15 (FA_SUM T570 T571 T572))
			(C15 (FA_CARRY T570 T571 T572))
			(S16 (FA_SUM T573 T574 T575))
			(C16 (FA_CARRY T573 T574 T575))
			(S17 (FA_SUM T576 T577 T578))
			(C17 (FA_CARRY T576 T577 T578))
			(S18 (FA_SUM T579 T580 T581))
			(C18 (FA_CARRY T579 T580 T581))
			(S19 (FA_SUM T582 T583 T584))
			(C19 (FA_CARRY T582 T583 T584))
			(S20 (FA_SUM T585 T586 T587))
			(C20 (FA_CARRY T585 T586 T587))
		)
			
	(let
		(
			(S21 (FA_SUM S1 S2 S3))
			(C21 (FA_CARRY S1 S2 S3))
			(S22 (FA_SUM S4 S5 S6))
			(C22 (FA_CARRY S4 S5 S6))
			(S23 (FA_SUM S7 S8 S9))
			(C23 (FA_CARRY S7 S8 S9))
			(S24 (FA_SUM S10 S11 S12))
			(C24 (FA_CARRY S10 S11 S12))
			(S25 (FA_SUM S13 S14 S15))
			(C25 (FA_CARRY S13 S14 S15))
			(S26 (FA_SUM S16 S17 S18))
			(C26 (FA_CARRY S16 S17 S18))
			(S27 (FA_SUM S19 S20 T588))
			(C27 (FA_CARRY S19 S20 T588))
		)
		
	(let
		(
			(S28 (FA_SUM S21 S22 S23))
			(C28 (FA_CARRY S21 S22 S23))
			(S29 (FA_SUM S24 S25 S26))
			(C29 (FA_CARRY S24 S25 S26))
		)
		
	(let
		(
			(Z33 (FA_SUM S27 S28 S29)) ;34th bit
			(C30 (FA_CARRY S27 S28 S29))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM C28 C29 C30))
			(C10 (FA_CARRY C28 C29 C30))
			(S11 (FA_SUM T589 T590 T591))
			(C11 (FA_CARRY T589 T590 T591))
			(S12 (FA_SUM T592 T593 T594))
			(C12 (FA_CARRY T592 T593 T594))
			(S13 (FA_SUM T595 T596 T597))
			(C13 (FA_CARRY T595 T596 T597))
			(S14 (FA_SUM T598 T599 T600))
			(C14 (FA_CARRY T598 T599 T600))
			(S15 (FA_SUM T601 T602 T603))
			(C15 (FA_CARRY T601 T602 T603))
			(S16 (FA_SUM T604 T605 T606))
			(C16 (FA_CARRY T604 T605 T606))
			(S17 (FA_SUM T607 T608 T609))
			(C17 (FA_CARRY T607 T608 T609))
			(S18 (FA_SUM T610 T611 T612))
			(C18 (FA_CARRY T610 T611 T612))
			(S19 (FA_SUM T613 T614 T615))
			(C19 (FA_CARRY T613 T614 T615))
		)
		
	(let
		(
			(S20 (FA_SUM S1 S2 S3))
			(C20 (FA_CARRY S1 S2 S3))
			(S21 (FA_SUM S4 S5 S6))
			(C21 (FA_CARRY S4 S5 S6))
			(S22 (FA_SUM S7 S8 S9))
			(C22 (FA_CARRY S7 S8 S9))
			(S23 (FA_SUM S10 S11 S12))
			(C23 (FA_CARRY S10 S11 S12))
			(S24 (FA_SUM S13 S14 S15))
			(C24 (FA_CARRY S13 S14 S15))
			(S25 (FA_SUM S16 S17 S18))
			(C25 (FA_CARRY S16 S17 S18))
			(S26 (FA_SUM S19 T616 T617))
			(C26 (FA_CARRY S19 T616 T617))
		)
		
	(let
		(
			(S27 (FA_SUM S20 S21 S22))
			(C27 (FA_CARRY S20 S21 S22))
			(S28 (FA_SUM S23 S24 S25))
			(C28 (FA_CARRY S23 S24 S25))
		)
		
	(let
		(
			(Z34 (FA_SUM S26 S27 S28)) ;35th bit
			(C29 (FA_CARRY S26 S27 S28))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM C28 C29 T618))
			(C10 (FA_CARRY C28 C29 T618))
			(S11 (FA_SUM T619 T620 T621))
			(C11 (FA_CARRY T619 T620 T621))
			(S12 (FA_SUM T622 T623 T624))
			(C12 (FA_CARRY T622 T623 T624))
			(S13 (FA_SUM T625 T626 T627))
			(C13 (FA_CARRY T625 T626 T627))
			(S14 (FA_SUM T628 T629 T630))
			(C14 (FA_CARRY T628 T629 T630))
			(S15 (FA_SUM T631 T632 T633))
			(C15 (FA_CARRY T631 T632 T633))
			(S16 (FA_SUM T634 T635 T636))
			(C16 (FA_CARRY T634 T635 T636))
			(S17 (FA_SUM T637 T638 T639))
			(C17 (FA_CARRY T637 T638 T639))
			(S18 (FA_SUM T640 T641 T642))
			(C18 (FA_CARRY T640 T641 T642))
			(S19 (FA_SUM T643 T644 T645))
			(C19 (FA_CARRY T643 T644 T645))
		)
	
	(let
		(
			(S20 (FA_SUM S1 S2 S3))
			(C20 (FA_CARRY S1 S2 S3))
			(S21 (FA_SUM S4 S5 S6))
			(C21 (FA_CARRY S4 S5 S6))
			(S22 (FA_SUM S7 S8 S9))
			(C22 (FA_CARRY S7 S8 S9))
			(S23 (FA_SUM S10 S11 S12))
			(C23 (FA_CARRY S10 S11 S12))
			(S24 (FA_SUM S13 S14 S15))
			(C24 (FA_CARRY S13 S14 S15))
			(S25 (FA_SUM S16 S17 S18))
			(C25 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(S26 (FA_SUM S20 S21 S22))
			(C26 (FA_CARRY S20 S21 S22))
			(S27 (FA_SUM S23 S24 S25))
			(C27 (FA_CARRY S23 S24 S25))
		)
		
	(let
		(
			(Z35 (FA_SUM S19 S26 S27)) ;36th bit
			(C28 (FA_CARRY S19 S26 S27))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM C28 T646 T647))
			(C10 (FA_CARRY C28 T646 T647))
			(S11 (FA_SUM T648 T649 T650))
			(C11 (FA_CARRY T648 T649 T650))
			(S12 (FA_SUM T651 T652 T653))
			(C12 (FA_CARRY T651 T652 T653))
			(S13 (FA_SUM T654 T655 T656))
			(C13 (FA_CARRY T654 T655 T656))
			(S14 (FA_SUM T657 T658 T659))
			(C14 (FA_CARRY T657 T658 T659))
			(S15 (FA_SUM T660 T661 T662))
			(C15 (FA_CARRY T660 T661 T662))
			(S16 (FA_SUM T663 T664 T665))
			(C16 (FA_CARRY T663 T664 T665))
			(S17 (FA_SUM T666 T667 T668))
			(C17 (FA_CARRY T666 T667 T668))
			(S18 (FA_SUM T669 T670 T671))
			(C18 (FA_CARRY T669 T670 T671))
		)

	(let
		(
			(S19 (FA_SUM S1 S2 S3))
			(C19 (FA_CARRY S1 S2 S3))
			(S20 (FA_SUM S4 S5 S6))
			(C20 (FA_CARRY S4 S5 S6))
			(S21 (FA_SUM S7 S8 S9))
			(C21 (FA_CARRY S7 S8 S9))
			(S22 (FA_SUM S10 S11 S12))
			(C22 (FA_CARRY S10 S11 S12))
			(S23 (FA_SUM S13 S14 S15))
			(C23 (FA_CARRY S13 S14 S15))
			(S24 (FA_SUM S16 S17 S18))
			(C24 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(S25 (FA_SUM S19 S20 S21))
			(C25 (FA_CARRY S19 S20 S21))
			(S26 (FA_SUM S22 S23 S24))
			(C26 (FA_CARRY S22 S23 S24))
		)
		
	(let
		(
			(Z36 (FA_SUM S25 S26 T672)) ;37th bit
			(C27 (FA_CARRY S25 S26 T672))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 C27))
			(C9 (FA_CARRY C25 C26 C27))
			(S10 (FA_SUM T673 T674 T675))
			(C10 (FA_CARRY T673 T674 T675))
			(S11 (FA_SUM T676 T677 T678))
			(C11 (FA_CARRY T676 T677 T678))
			(S12 (FA_SUM T679 T680 T681))
			(C12 (FA_CARRY T679 T680 T681))
			(S13 (FA_SUM T682 T683 T684))
			(C13 (FA_CARRY T682 T683 T684))
			(S14 (FA_SUM T685 T686 T687))
			(C14 (FA_CARRY T685 T686 T687))
			(S15 (FA_SUM T688 T689 T690))
			(C15 (FA_CARRY T688 T689 T690))
			(S16 (FA_SUM T691 T692 T693))
			(C16 (FA_CARRY T691 T692 T693))
			(S17 (FA_SUM T694 T695 T696))
			(C17 (FA_CARRY T694 T695 T696))
		)
		
	(let
		(
			(S18 (FA_SUM S1 S2 S3))
			(C18 (FA_CARRY S1 S2 S3))
			(S19 (FA_SUM S4 S5 S6))
			(C19 (FA_CARRY S4 S5 S6))
			(S20 (FA_SUM S7 S8 S9))
			(C20 (FA_CARRY S7 S8 S9))
			(S21 (FA_SUM S10 S11 S12))
			(C21 (FA_CARRY S10 S11 S12))
			(S22 (FA_SUM S13 S14 S15))
			(C22 (FA_CARRY S13 S14 S15))
			(S23 (FA_SUM S16 S17 T697))
			(C23 (FA_CARRY S16 S17 T697))
		)
	
	(let
		(
			(S24 (FA_SUM S18 S19 S20))
			(C24 (FA_CARRY S18 S19 S20))
			(S25 (FA_SUM S21 S22 S23))
			(C25 (FA_CARRY S21 S22 S23))
		)
		
	(let
		(
			(Z37 (FA_SUM S24 S25 T698)) ;38th bit
			(C26 (FA_CARRY S24 S25 T698))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 C26 T699))
			(C9 (FA_CARRY C25 C26 T699))
			(S10 (FA_SUM T700 T701 T702))
			(C10 (FA_CARRY T700 T701 T702))
			(S11 (FA_SUM T703 T704 T705))
			(C11 (FA_CARRY T703 T704 T705))
			(S12 (FA_SUM T706 T707 T708))
			(C12 (FA_CARRY T706 T707 T708))
			(S13 (FA_SUM T709 T710 T711))
			(C13 (FA_CARRY T709 T710 T711))
			(S14 (FA_SUM T712 T713 T714))
			(C14 (FA_CARRY T712 T713 T714))
			(S15 (FA_SUM T715 T716 T717))
			(C15 (FA_CARRY T715 T716 T717))
			(S16 (FA_SUM T718 T719 T720))
			(C16 (FA_CARRY T718 T719 T720))
			(S17 (FA_SUM T721 T722 T723))
			(C17 (FA_CARRY T721 T722 T723))
		)
	
	(let
		(
			(S18 (FA_SUM S1 S2 S3))
			(C18 (FA_CARRY S1 S2 S3))
			(S19 (FA_SUM S4 S5 S6))
			(C19 (FA_CARRY S4 S5 S6))
			(S20 (FA_SUM S7 S8 S9))
			(C20 (FA_CARRY S7 S8 S9))
			(S21 (FA_SUM S10 S11 S12))
			(C21 (FA_CARRY S10 S11 S12))
			(S22 (FA_SUM S13 S14 S15))
			(C22 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(S23 (FA_SUM S18 S19 S20))
			(C23 (FA_CARRY S18 S19 S20))
			(S24 (FA_SUM S16 S21 S22))
			(C24 (FA_CARRY S16 S21 S22))
		)
		
	(let
		(
			(Z38 (FA_SUM S17 S23 S24)) ;39th bit
			(C25 (FA_CARRY S17 S23 S24))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM C25 T724 T725))
			(C9 (FA_CARRY C25 T724 T725))
			(S10 (FA_SUM T726 T727 T728))
			(C10 (FA_CARRY T726 T727 T728))
			(S11 (FA_SUM T729 T730 T731))
			(C11 (FA_CARRY T729 T730 T731))
			(S12 (FA_SUM T732 T733 T734))
			(C12 (FA_CARRY T732 T733 T734))
			(S13 (FA_SUM T735 T736 T737))
			(C13 (FA_CARRY T735 T736 T737))
			(S14 (FA_SUM T738 T739 T740))
			(C14 (FA_CARRY T738 T739 T740))
			(S15 (FA_SUM T741 T742 T743))
			(C15 (FA_CARRY T741 T742 T743))
			(S16 (FA_SUM T744 T745 T746))
			(C16 (FA_CARRY T744 T745 T746))
		)
	
	(let
		(
			(S17 (FA_SUM S1 S2 S3))
			(C17 (FA_CARRY S1 S2 S3))
			(S18 (FA_SUM S4 S5 S6))
			(C18 (FA_CARRY S4 S5 S6))
			(S19 (FA_SUM S7 S8 S9))
			(C19 (FA_CARRY S7 S8 S9))
			(S20 (FA_SUM S10 S11 S12))
			(C20 (FA_CARRY S10 S11 S12))
			(S21 (FA_SUM S13 S14 S15))
			(C21 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(S22 (FA_SUM S17 S18 S19))
			(C22 (FA_CARRY S17 S18 S19))
			(S23 (FA_SUM S16 S21 S20))
			(C23 (FA_CARRY S16 S21 S20))
		)
		
	(let
		(
			(Z39 (FA_SUM T747 S22 S23)) ;40th bit
			(C24 (FA_CARRY T747 S22 S23))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 C24))
			(C8 (FA_CARRY C22 C23 C24))
			(S9 (FA_SUM T748 T749 T750))
			(C9 (FA_CARRY T748 T749 T750))
			(S10 (FA_SUM T751 T752 T753))
			(C10 (FA_CARRY T751 T752 T753))
			(S11 (FA_SUM T754 T755 T756))
			(C11 (FA_CARRY T754 T755 T756))
			(S12 (FA_SUM T757 T758 T759))
			(C12 (FA_CARRY T757 T758 T759))
			(S13 (FA_SUM T760 T761 T762))
			(C13 (FA_CARRY T760 T761 T762))
			(S14 (FA_SUM T763 T764 T765))
			(C14 (FA_CARRY T763 T764 T765))
			(S15 (FA_SUM T766 T767 T768))
			(C15 (FA_CARRY T766 T767 T768))
		)
	
	(let
		(
			(S16 (FA_SUM S1 S2 S3))
			(C16 (FA_CARRY S1 S2 S3))
			(S17 (FA_SUM S4 S5 S6))
			(C17 (FA_CARRY S4 S5 S6))
			(S18 (FA_SUM S7 S8 S9))
			(C18 (FA_CARRY S7 S8 S9))
			(S19 (FA_SUM S10 S11 S12))
			(C19 (FA_CARRY S10 S11 S12))
			(S20 (FA_SUM S13 S14 S15))
			(C20 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(S21 (FA_SUM S16 S17 S18))
			(C21 (FA_CARRY S16 S17 S18))
			(S22 (FA_SUM S19 S20 T769))
			(C22 (FA_CARRY S19 S20 T769))
		)
		
	(let
		(
			(Z40 (FA_SUM S21 S22 T770)) ;41st bit
			(C23 (FA_CARRY S21 S22 T770))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 C23 T771))
			(C8 (FA_CARRY C22 C23 T771))
			(S9 (FA_SUM T772 T773 T774))
			(C9 (FA_CARRY T772 T773 T774))
			(S10 (FA_SUM T775 T776 T777))
			(C10 (FA_CARRY T775 T776 T777))
			(S11 (FA_SUM T778 T779 T780))
			(C11 (FA_CARRY T778 T779 T780))
			(S12 (FA_SUM T781 T782 T783))
			(C12 (FA_CARRY T781 T782 T783))
			(S13 (FA_SUM T784 T785 T786))
			(C13 (FA_CARRY T784 T785 T786))
			(S14 (FA_SUM T787 T788 T789))
			(C14 (FA_CARRY T787 T788 T789))
			(S15 (FA_SUM T790 T791 T792))
			(C15 (FA_CARRY T790 T791 T792))
		)
		
	(let
		(
			(S16 (FA_SUM S1 S2 S3))
			(C16 (FA_CARRY S1 S2 S3))
			(S17 (FA_SUM S4 S5 S6))
			(C17 (FA_CARRY S4 S5 S6))
			(S18 (FA_SUM S7 S8 S9))
			(C18 (FA_CARRY S7 S8 S9))
			(S19 (FA_SUM S10 S11 S12))
			(C19 (FA_CARRY S10 S11 S12))
			(S20 (FA_SUM S13 S14 S15))
			(C20 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(S21 (FA_SUM S16 S17 S18))
			(C21 (FA_CARRY S16 S17 S18))
		)
		
	(let
		(
			(Z41 (FA_SUM S19 S20 S21)) ;42nd bit
			(C22 (FA_CARRY S19 S20 S21))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM C22 T793 T794))
			(C8 (FA_CARRY C22 T793 T794))
			(S9 (FA_SUM T795 T796 T797))
			(C9 (FA_CARRY T795 T796 T797))
			(S10 (FA_SUM T798 T799 T800))
			(C10 (FA_CARRY T798 T799 T800))
			(S11 (FA_SUM T801 T802 T803))
			(C11 (FA_CARRY T801 T802 T803))
			(S12 (FA_SUM T804 T805 T806))
			(C12 (FA_CARRY T804 T805 T806))
			(S13 (FA_SUM T807 T808 T809))
			(C13 (FA_CARRY T807 T808 T809))
			(S14 (FA_SUM T810 T811 T812))
			(C14 (FA_CARRY T810 T811 T812))
		)
		
	(let
		(
			(S15 (FA_SUM S1 S2 S3))
			(C15 (FA_CARRY S1 S2 S3))
			(S16 (FA_SUM S4 S5 S6))
			(C16 (FA_CARRY S4 S5 S6))
			(S17 (FA_SUM S7 S8 S9))
			(C17 (FA_CARRY S7 S8 S9))
			(S18 (FA_SUM S10 S11 S12))
			(C18 (FA_CARRY S10 S11 S12))
			(S19 (FA_SUM S13 S14 T813))
			(C19 (FA_CARRY S13 S14 T813))
		)
		
	(let
		(
			(S20 (FA_SUM S15 S16 S17))
			(C20 (FA_CARRY S15 S16 S17))
		)
		
	(let
		(
			(Z42 (FA_SUM S18 S19 S20)) ;43rd bit
			(C21 (FA_CARRY S18 S19 S20))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 C21))
			(C7 (FA_CARRY C19 C20 C21))
			(S8 (FA_SUM T814 T815 T816))
			(C8 (FA_CARRY T814 T815 T816))
			(S9 (FA_SUM T817 T818 T819))
			(C9 (FA_CARRY T817 T818 T819))
			(S10 (FA_SUM T820 T821 T822))
			(C10 (FA_CARRY T820 T821 T822))
			(S11 (FA_SUM T823 T824 T825))
			(C11 (FA_CARRY T823 T824 T825))
			(S12 (FA_SUM T826 T827 T828))
			(C12 (FA_CARRY T826 T827 T828))
			(S13 (FA_SUM T829 T830 T831))
			(C13 (FA_CARRY T829 T830 T831))
		)
		
	(let
		(
			(S14 (FA_SUM S1 S2 S3))
			(C14 (FA_CARRY S1 S2 S3))
			(S15 (FA_SUM S4 S5 S6))
			(C15 (FA_CARRY S4 S5 S6))
			(S16 (FA_SUM S7 S8 S9))
			(C16 (FA_CARRY S7 S8 S9))
			(S17 (FA_SUM S10 S11 S12))
			(C17 (FA_CARRY S10 S11 S12))
			(S18 (FA_SUM S13 T832 T833))
			(C18 (FA_CARRY S13 T832 T833))
		)
		
	(let
		(
			(S19 (FA_SUM S14 S15 S16))
			(C19 (FA_CARRY S14 S15 S16))
		)
		
	(let
		(
			(Z43 (FA_SUM S17 S18 S19)) ;44th bit
			(C20 (FA_CARRY S17 S18 S19))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 C20 T834))
			(C7 (FA_CARRY C19 C20 T834))
			(S8 (FA_SUM T835 T836 T837))
			(C8 (FA_CARRY T835 T836 T837))
			(S9 (FA_SUM T838 T839 T840))
			(C9 (FA_CARRY T838 T839 T840))
			(S10 (FA_SUM T841 T842 T843))
			(C10 (FA_CARRY T841 T842 T843))
			(S11 (FA_SUM T844 T845 T846))
			(C11 (FA_CARRY T844 T845 T846))
			(S12 (FA_SUM T847 T848 T849))
			(C12 (FA_CARRY T847 T848 T849))
			(S13 (FA_SUM T850 T851 T852))
			(C13 (FA_CARRY T850 T851 T852))
		)
		
	(let
		(
			(S14 (FA_SUM S1 S2 S3))
			(C14 (FA_CARRY S1 S2 S3))
			(S15 (FA_SUM S4 S5 S6))
			(C15 (FA_CARRY S4 S5 S6))
			(S16 (FA_SUM S7 S8 S9))
			(C16 (FA_CARRY S7 S8 S9))
			(S17 (FA_SUM S10 S11 S12))
			(C17 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S18 (FA_SUM S14 S15 S16))
			(C18 (FA_CARRY S14 S15 S16))
		)
		
	(let
		(
			(Z44 (FA_SUM S13 S17 S18)) ;45th bit
			(C19 (FA_CARRY S13 S17 S18))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM C19 T853 T854))
			(C7 (FA_CARRY C19 T853 T854))
			(S8 (FA_SUM T855 T856 T857))
			(C8 (FA_CARRY T855 T856 T857))
			(S9 (FA_SUM T858 T859 T860))
			(C9 (FA_CARRY T858 T859 T860))
			(S10 (FA_SUM T861 T862 T863))
			(C10 (FA_CARRY T861 T862 T863))
			(S11 (FA_SUM T864 T865 T866))
			(C11 (FA_CARRY T864 T865 T866))
			(S12 (FA_SUM T867 T868 T869))
			(C12 (FA_CARRY T867 T868 T869))
		)
		
	(let
		(
			(S13 (FA_SUM S1 S2 S3))
			(C13 (FA_CARRY S1 S2 S3))
			(S14 (FA_SUM S4 S5 S6))
			(C14 (FA_CARRY S4 S5 S6))
			(S15 (FA_SUM S7 S8 S9))
			(C15 (FA_CARRY S7 S8 S9))
			(S16 (FA_SUM S10 S11 S12))
			(C16 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S17 (FA_SUM S13 S14 S15))
			(C17 (FA_CARRY S13 S14 S15))
		)
		
	(let
		(
			(Z45 (FA_SUM S16 S17 T870)) ;46th bit
			(C18 (FA_CARRY S16 S17 T870))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 C18))
			(C6 (FA_CARRY C16 C17 C18))
			(S7 (FA_SUM T871 T872 T873))
			(C7 (FA_CARRY T871 T872 T873))
			(S8 (FA_SUM T874 T875 T876))
			(C8 (FA_CARRY T874 T875 T876))
			(S9 (FA_SUM T877 T878 T879))
			(C9 (FA_CARRY T877 T878 T879))
			(S10 (FA_SUM T880 T881 T882))
			(C10 (FA_CARRY T880 T881 T882))
			(S11 (FA_SUM T883 T884 T885))
			(C11 (FA_CARRY T883 T884 T885))
		)
		
	(let
		(
			(S12 (FA_SUM S1 S2 S3))
			(C12 (FA_CARRY S1 S2 S3))
			(S13 (FA_SUM S4 S5 S6))
			(C13 (FA_CARRY S4 S5 S6))
			(S14 (FA_SUM S7 S8 S9))
			(C14 (FA_CARRY S7 S8 S9))
			(S15 (FA_SUM S10 S11 T886))
			(C15 (FA_CARRY S10 S11 T886))
		)
		
	(let
		(
			(S16 (FA_SUM S12 S13 S14))
			(C16 (FA_CARRY S12 S13 S14))
		)
		
	(let
		(
			(Z46 (FA_SUM S15 S16 T887)) ;47th bit
			(C17 (FA_CARRY S15 S16 T887))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 C17 T888))
			(C6 (FA_CARRY C16 C17 T888))
			(S7 (FA_SUM T889 T890 T891))
			(C7 (FA_CARRY T889 T890 T891))
			(S8 (FA_SUM T892 T893 T894))
			(C8 (FA_CARRY T892 T893 T894))
			(S9 (FA_SUM T895 T896 T897))
			(C9 (FA_CARRY T895 T896 T897))
			(S10 (FA_SUM T898 T899 T900))
			(C10 (FA_CARRY T898 T899 T900))
			(S11 (FA_SUM T901 T902 T903))
			(C11 (FA_CARRY T901 T902 T903))
		)
		
	(let
		(
			(S12 (FA_SUM S1 S2 S3))
			(C12 (FA_CARRY S1 S2 S3))
			(S13 (FA_SUM S4 S5 S6))
			(C13 (FA_CARRY S4 S5 S6))
			(S14 (FA_SUM S7 S8 S9))
			(C14 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S15 (FA_SUM S12 S13 S14))
			(C15 (FA_CARRY S12 S13 S14))
		)
		
	(let
		(
			(Z47 (FA_SUM S10 S11 S15)) ;48th bit
			(C16 (FA_CARRY S10 S11 S15))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM C16 T904 T905))
			(C6 (FA_CARRY C16 T904 T905))
			(S7 (FA_SUM T906 T907 T908))
			(C7 (FA_CARRY T906 T907 T908))
			(S8 (FA_SUM T909 T910 T911))
			(C8 (FA_CARRY T909 T910 T911))
			(S9 (FA_SUM T912 T913 T914))
			(C9 (FA_CARRY T912 T913 T914))
			(S10 (FA_SUM T915 T916 T917))
			(C10 (FA_CARRY T915 T916 T917))
		)
		
	(let
		(
			(S11 (FA_SUM S1 S2 S3))
			(C11 (FA_CARRY S1 S2 S3))
			(S12 (FA_SUM S4 S5 S6))
			(C12 (FA_CARRY S4 S5 S6))
			(S13 (FA_SUM S7 S8 S9))
			(C13 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S14 (FA_SUM S11 S12 S13))
			(C14 (FA_CARRY S11 S12 S13))
		)
		
	(let
		(
			(Z48 (FA_SUM S10 S14 T918)) ; 49th bit
			(C15 (FA_CARRY S10 S14 T918))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 C15))
			(C5 (FA_CARRY C13 C14 C15))
			(S6 (FA_SUM T919 T920 T921))
			(C6 (FA_CARRY T919 T920 T921))
			(S7 (FA_SUM T922 T923 T924))
			(C7 (FA_CARRY T922 T923 T924))
			(S8 (FA_SUM T925 T926 T927))
			(C8 (FA_CARRY T925 T926 T927))
			(S9 (FA_SUM T928 T929 T930))
			(C9 (FA_CARRY T928 T929 T930))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S13 (FA_SUM S10 S11 S12))
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(Z49 (FA_SUM S13 T931 T932)) ; 50th bit
			(C14 (FA_CARRY S13 T931 T932))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 C14 T933))
			(C5 (FA_CARRY C13 C14 T933))
			(S6 (FA_SUM T934 T935 T936))
			(C6 (FA_CARRY T934 T935 T936))
			(S7 (FA_SUM T937 T938 T939))
			(C7 (FA_CARRY T937 T938 T939))
			(S8 (FA_SUM T940 T941 T942))
			(C8 (FA_CARRY T940 T941 T942))
			(S9 (FA_SUM T943 T944 T945))
			(C9 (FA_CARRY T943 T944 T945))
		)
		
	(let
		(
			(S10 (FA_SUM S1 S2 S3))
			(C10 (FA_CARRY S1 S2 S3))
			(S11 (FA_SUM S4 S5 S6))
			(C11 (FA_CARRY S4 S5 S6))
			(S12 (FA_SUM S7 S8 S9))
			(C12 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(Z50 (FA_SUM S10 S11 S12)) ;51st bit
			(C13 (FA_CARRY S10 S11 S12))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM C13 T946 T947))
			(C5 (FA_CARRY C13 T946 T947))
			(S6 (FA_SUM T948 T949 T950))
			(C6 (FA_CARRY T948 T949 T950))
			(S7 (FA_SUM T951 T952 T953))
			(C7 (FA_CARRY T951 T952 T953))
			(S8 (FA_SUM T954 T955 T956))
			(C8 (FA_CARRY T954 T955 T956))
		)
		
	(let
		(
			(S9 (FA_SUM S1 S2 S3))
			(C9 (FA_CARRY S1 S2 S3))
			(S10 (FA_SUM S4 S5 S6))
			(C10 (FA_CARRY S4 S5 S6))
			(S11 (FA_SUM S7 S8 T957))
			(C11 (FA_CARRY S7 S8 T957))
		)
		
	(let
		(
			(Z51 (FA_SUM S9 S10 S11)) ;52nd bit
			(C12 (FA_CARRY S9 S10 S11))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 C12))
			(C4 (FA_CARRY C10 C11 C12))
			(S5 (FA_SUM T958 T959 T960))
			(C5 (FA_CARRY T958 T959 T960))
			(S6 (FA_SUM T961 T962 T963))
			(C6 (FA_CARRY T961 T962 T963))
			(S7 (FA_SUM T964 T965 T966))
			(C7 (FA_CARRY T964 T965 T966))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
			(S10 (FA_SUM S7 T967 T968))
			(C10 (FA_CARRY S7 T967 T968))
		)
		
	(let
		(
			(Z52 (FA_SUM S8 S9 S10)) ;53rd bit
			(C11 (FA_CARRY S8 S9 S10))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 C11 T969))
			(C4 (FA_CARRY C10 C11 T969))
			(S5 (FA_SUM T970 T971 T972))
			(C5 (FA_CARRY T970 T971 T972))
			(S6 (FA_SUM T973 T974 T975))
			(C6 (FA_CARRY T973 T974 T975))
			(S7 (FA_SUM T976 T977 T978))
			(C7 (FA_CARRY T976 T977 T978))
		)
		
	(let
		(
			(S8 (FA_SUM S1 S2 S3))
			(C8 (FA_CARRY S1 S2 S3))
			(S9 (FA_SUM S4 S5 S6))
			(C9 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z53 (FA_SUM S7 S8 S9)) ;54th bit
			(C10 (FA_CARRY S7 S8 S9))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM C10 T979 T980))
			(C4 (FA_CARRY C10 T979 T980))
			(S5 (FA_SUM T981 T982 T983))
			(C5 (FA_CARRY T981 T982 T983))
			(S6 (FA_SUM T984 T985 T986))
			(C6 (FA_CARRY T984 T985 T986))
		)
		
	(let
		(
			(S7 (FA_SUM S1 S2 S3))
			(C7 (FA_CARRY S1 S2 S3))
			(S8 (FA_SUM S4 S5 S6))
			(C8 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(Z54 (FA_SUM S7 S8 T987)) ;55th bit
			(C9 (FA_CARRY S7 S8 T987))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 C9))
			(C3 (FA_CARRY C7 C8 C9))
			(S4 (FA_SUM T988 T989 T990))
			(C4 (FA_CARRY T988 T989 T990))
			(S5 (FA_SUM T991 T992 T993))
			(C5 (FA_CARRY T991 T992 T993))
		)
		
	(let
		(
			(S6 (FA_SUM S1 S2 S3))
			(C6 (FA_CARRY S1 S2 S3))
			(S7 (FA_SUM S4 S5 T994))
			(C7 (FA_CARRY S4 S5 T994))
		)
		
	(let
		(
			(Z55 (FA_SUM S6 S7 T995)) ;56th bit
			(C8 (FA_CARRY S6 S7 T995))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 C8 T996))
			(C3 (FA_CARRY C7 C8 T996))
			(S4 (FA_SUM T997 T998 T999))
			(C4 (FA_CARRY T997 T998 T999))
			(S5 (FA_SUM T1000 T1001 T1002))
			(C5 (FA_CARRY T1000 T1001 T1002))
		)
		
	(let
		(
			(S6 (FA_SUM S1 S2 S3))
			(C6 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z56 (FA_SUM S4 S5 S6)) ;57th bit
			(C7 (FA_CARRY S4 S5 S6))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 T1003 T1004))
			(C3 (FA_CARRY C7 T1003 T1004))
			(S4 (FA_SUM T1005 T1006 T1007))
			(C4 (FA_CARRY T1005 T1006 T1007))
		)
		
	(let
		(
			(S5 (FA_SUM S1 S2 S3))
			(C5 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z57 (FA_SUM S4 S5 T1008)) ;58th bit
			(C6 (FA_CARRY S4 S5 T1008))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T1009 T1010 T1011))
			(C3 (FA_CARRY T1009 T1010 T1011))
		)
		
	(let
		(
			(S4 (FA_SUM S1 S2 S3))
			(C4 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z58 (FA_SUM S4 T1012 T1013)) ;59th bit
			(C5 (FA_CARRY S4 T1012 T1013))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 T1014))
			(C2 (FA_CARRY C4 C5 T1014))
			(S3 (FA_SUM T1015 T1016 T1017))
			(C3 (FA_CARRY T1015 T1016 T1017))
		)
		
	(let
		(
			(Z59 (FA_SUM S1 S2 S3)) ;60th bit
			(C4 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 T1018 T1019))
			(C2 (FA_CARRY C4 T1018 T1019))
		)
		
	(let
		(
			(Z60 (FA_SUM S1 S2 T1020)) ;61st bit
			(C3 (FA_CARRY S1 S2 T1020))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
		)
		
	(let
		(
			(Z61 (FA_SUM S1 T1021 T1022)) ;62nd bit
			(C2 (FA_CARRY S1 T1021 T1022))
		)
		
	(let
		(
			(Z62 (FA_SUM T1023 C1 C2)) ;63rd bit
			(Z63 (FA_CARRY T1023 C1 C2)) ;64th bit
		)
	
	(let
		(
			(Z (concat Z63 Z62 Z61 Z60 Z59 Z58 Z57 Z56 Z55 Z54 Z53 Z52 Z51 Z50 Z49 Z48 Z47 Z46 Z45 Z44 Z43 Z42 Z41 Z40 Z39 Z38 Z37 Z36 Z35 Z34 Z33 Z32 Z31 Z30 Z29 Z28 Z27 Z26 Z25 Z24 Z23 Z22 Z21 Z20 Z19 Z18 Z17 Z16 Z15 Z14 Z13 Z12 Z11 Z10 Z9 Z8 Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
			(select (bvand Z63 Z62 Z61 Z60 Z59 Z58 Z57 Z56 Z55 Z54 Z53 Z52 Z51 Z50 Z49 Z48 Z47 Z46 Z45 Z44 Z43 Z42 Z41 Z40 Z39 Z38 Z37 Z36 Z35 Z34 Z33 Z32 Z31 Z30 Z29 Z28 Z27 Z26 Z25 Z24 Z23 Z22 Z21 Z20 Z19 Z18 Z17 Z16 Z15 Z14 Z13 Z12 Z11 Z10 Z9 Z8 Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
		)
		
	(let
		(
			(muxout (mux2to1 Z trojanout select))
		)

	(let
		(
			(F (concat trojanout muxout))
		)		
	))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))))
)	

(assert
		(not
			(let
				(
					(ye0 ((_ extract 0 0) Ymul))
					(ye1 ((_ extract 1 1) Ymul))
					(ye2 ((_ extract 2 2) Ymul))
					(ye3 ((_ extract 3 3) Ymul))
					(ye4 ((_ extract 4 4) Ymul))
					(ye5 ((_ extract 5 5) Ymul))
					(ye6 ((_ extract 6 6) Ymul))
					(ye7 ((_ extract 7 7) Ymul))
					(ye8 ((_ extract 8 8) Ymul))
					(ye9 ((_ extract 9 9) Ymul))
					(ye10 ((_ extract 10 10) Ymul))
					(ye11 ((_ extract 11 11) Ymul))
					(ye12 ((_ extract 12 12) Ymul))
					(ye13 ((_ extract 13 13) Ymul))
					(ye14 ((_ extract 14 14) Ymul))
					(ye15 ((_ extract 15 15) Ymul))
					(ye16 ((_ extract 16 16) Ymul))
					(ye17 ((_ extract 17 17) Ymul))
					(ye18 ((_ extract 18 18) Ymul))
					(ye19 ((_ extract 19 19) Ymul))
					(ye20 ((_ extract 20 20) Ymul))
					(ye21 ((_ extract 21 21) Ymul))
					(ye22 ((_ extract 22 22) Ymul))
					(ye23 ((_ extract 23 23) Ymul))
					(ye24 ((_ extract 24 24) Ymul))
					(ye25 ((_ extract 25 25) Ymul))
					(ye26 ((_ extract 26 26) Ymul))
					(ye27 ((_ extract 27 27) Ymul))
					(ye28 ((_ extract 28 28) Ymul))
					(ye29 ((_ extract 29 29) Ymul))
					(ye30 ((_ extract 30 30) Ymul))
					(trojanew (concat trojan trojan trojan trojan trojan trojan trojan trojan trojan trojan trojan trojan trojan trojan trojan trojan))
				)
			(let
				(
					(Ymul1 (concat ye30 ye29 ye28 ye27 ye26 ye25 ye24 ye23 ye22 ye21 ye20 ye19 ye18 ye17 ye16 ye15 ye14 ye13 ye12 ye11 ye10 ye9 ye8 ye7 ye6 ye5 ye4 ye3 ye2 ye1 ye0 t0)) ;first bit in shift register is 0
					(Ymul2 (concat ye30 ye29 ye28 ye27 ye26 ye25 ye24 ye23 ye22 ye21 ye20 ye19 ye18 ye17 ye16 ye15 ye14 ye13 ye12 ye11 ye10 ye9 ye8 ye7 ye6 ye5 ye4 ye3 ye2 ye1 ye0 t1)) ;first bit in shift register is 1
				)
			(let
				(
					(state0 (ARRAY_MUL Xmul Ymul1 trojanew))
					(state1 (ARRAY_MUL Xmul Ymul2 trojanew))
				) 
			(let
				(
					(LHS ((_ extract 63 0) state0))
					(RHS ((_ extract 63 0) state1))
				)
			(=>
				(and (= a0 (_ bv0 1)) (= a1 (_ bv1 1)))
				(= LHS RHS)
			))))))
)
			
(check-sat)
(get-model)