(set-logic QF_BV)
(declare-fun Xmul () (_ BitVec 4))
(declare-fun Ymul () (_ BitVec 4))
(declare-fun t0 () (_ BitVec 1))
(declare-fun t1 () (_ BitVec 1))
(declare-fun trojan () (_ BitVec 3))

(define-fun mux2to1 ( (s0(_ BitVec 8)) (s1(_ BitVec 8)) (s(_ BitVec 1))) (_ BitVec 8)
	(ite (= s (_ bv0 1)) s0 s1)
)

(define-fun HA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor x y)
)

(define-fun HA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvand x y)
)

(define-fun FA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor (bvxor x y) cin)
)

(define-fun FA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvor (bvand x y) (bvor (bvand x cin) (bvand y cin)) )
)

(define-fun ARRAY_MUL ( (X(_ BitVec 4)) (Y(_ BitVec 4)) (trojan(_ BitVec 8))) (_ BitVec 16) 
	(let
		(
			(X3 ((_ extract 3 3) X))
			(X2 ((_ extract 2 2) X))
			(X1 ((_ extract 1 1) X))
			(X0 ((_ extract 0 0) X))
			(Y3 ((_ extract 3 3) Y))
			(Y2 ((_ extract 2 2) Y))
			(Y1 ((_ extract 1 1) Y))
			(Y0 ((_ extract 0 0) Y))
		)
	(let
		(
			(Z0 (bvand X0 Y0))
			(T1 (bvand X1 Y0))
			(T2 (bvand X0 Y1))
			(T3 (bvand X2 Y0))
			(T4 (bvand X1 Y1))
			(T5 (bvand X0 Y2))
			(T6 (bvand X3 Y0))
			(T7 (bvand X2 Y1))
			(T8 (bvand X1 Y2))
			(T9 (bvand X0 Y3))
			(T10 (bvand X3 Y1))
			(T11 (bvand X2 Y2))
			(T12 (bvand X1 Y3))
			(T13 (bvand X3 Y2))
			(T14 (bvand X2 Y3))
			(T15 (bvand X3 Y3))
		)
	(let
		(
			(Z1 (HA_SUM T1 T2))
			(C0 (HA_CARRY T1 T2))
			(trojanout (bvshl trojan (_ bv1 8)))
		)
			
	(let
		(
			(S1 (FA_SUM T3 T4 C0))
			(C1 (FA_CARRY T3 T4 C0))
		)
			
	(let
		(
			(Z2 (HA_SUM S1 T5))
			(C3 (HA_CARRY S1 T5))
		)
			
	(let
		(
			(S2 (FA_SUM T6 T7 C1))
			(C2 (FA_CARRY T6 T7 C1))
		)
			
	(let
		(
			(S3 (FA_SUM S2 T8 C3))
			(C4 (FA_CARRY S2 T8 C3))
		)
		
	(let
		(
			(Z3 (HA_SUM S3 T9))
			(C6 (HA_CARRY S3 T9))
		)
			
	(let
		(
			(S4 (FA_SUM C2 T10 C4))
			(C5 (FA_CARRY C2 T10 C4))
		)
		
	(let
		(
			(S5 (FA_SUM S4 T11 C6))
			(C7 (FA_CARRY S4 T11 C6))
		)
			
	(let
		(
			(Z4 (HA_SUM S5 T12))
			(C9 (HA_CARRY S5 T12))
		)
	
	(let
		(
			(S6 (FA_SUM C5 T13 C7))
			(C8 (FA_CARRY C5 T13 C7))
		)
			
	(let
		(
			(Z5 (FA_SUM S6 T14 C9))
			(C10 (FA_CARRY S6 T14 C9))
		)
		
	(let
		(
			(Z6 (FA_SUM C8 T15 C10))
			(Z7 (FA_CARRY C8 T15 C10))
		)
		
	(let
		(
			(Z (concat Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
			(select (bvand Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
		)
		
	(let
		(
			(muxout (mux2to1 Z trojanout select))
		)

	(let
		(
			(F (concat trojanout muxout))
		)		
	)))))))))))))))))
)	

(assert
		(not
			(let
				(
					(te2 ((_ extract 2 2) trojan))
					(te1 ((_ extract 1 1) trojan))
					(te0 ((_ extract 0 0) trojan))
				)
			(let
				(
					(trojan1 (concat te2 te1 te0 t0))
					(trojan2 (concat te2 te1 te0 t1))
				)
			(let
				(
					(trojanew1 (concat trojan1 trojan1))
					(trojanew2 (concat trojan2 trojan2))
				)
			(let
				(
					(state0 (ARRAY_MUL Xmul Ymul trojanew1))
					(state1 (ARRAY_MUL Xmul Ymul trojanew2))
				) 
			(let
				(
					(LHS ((_ extract 7 0) state0))
					(RHS ((_ extract 7 0) state1))
				)
			(=>
				(and (= t0 (_ bv0 1)) (= t1 (_ bv1 1)))
				(= LHS RHS)
			)))))))
)
			
(check-sat)
(get-model)