(set-logic QF_BV)
(declare-fun Xmul () (_ BitVec 8))
(declare-fun Ymul () (_ BitVec 7))
(declare-fun a0 () (_ BitVec 1))
(declare-fun a1 () (_ BitVec 1))
(declare-fun trojan () (_ BitVec 4))

(define-fun mux2to1 ( (s0(_ BitVec 16)) (s1(_ BitVec 16)) (s(_ BitVec 1))) (_ BitVec 16)
	(ite (= s (_ bv0 1)) s0 s1)
)

(define-fun HA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor x y)
)

(define-fun HA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) ) (_ BitVec 1)
	(bvand x y)
)

(define-fun FA_SUM ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvxor (bvxor x y) cin)
)

(define-fun FA_CARRY ( (x(_ BitVec 1)) (y(_ BitVec 1)) (cin(_ BitVec 1)) ) (_ BitVec 1)
	(bvor (bvand x y) (bvor (bvand x cin) (bvand y cin)) )
)

(define-fun ARRAY_MUL ( (X(_ BitVec 8)) (Y(_ BitVec 8)) (trojan(_ BitVec 16))) (_ BitVec 32) 
	(let
		(
			(X7((_ extract 7 7) X))
			(X6((_ extract 6 6) X))
			(X5((_ extract 5 5) X))
			(X4((_ extract 4 4) X))
			(X3((_ extract 3 3) X))
			(X2((_ extract 2 2) X))
			(X1((_ extract 1 1) X))
			(X0((_ extract 0 0) X))
			(Y7((_ extract 7 7) Y))
			(Y6((_ extract 6 6) Y))
			(Y5((_ extract 5 5) Y))
			(Y4((_ extract 4 4) Y))
			(Y3((_ extract 3 3) Y))
			(Y2((_ extract 2 2) Y))
			(Y1((_ extract 1 1) Y))
			(Y0((_ extract 0 0) Y))
		)
	(let
		(
			(Z0 (bvand X0 Y0)) ; first bit
			(T1 (bvand X1 Y0))
			(T2 (bvand X2 Y0))
			(T3 (bvand X3 Y0))
			(T4 (bvand X4 Y0))
			(T5 (bvand X5 Y0))
			(T6 (bvand X6 Y0))
			(T7 (bvand X7 Y0))
			(T8 (bvand X0 Y1))
			(T9 (bvand X1 Y1))
			(T10 (bvand X2 Y1))
			(T11 (bvand X3 Y1))
			(T12 (bvand X4 Y1))
			(T13 (bvand X5 Y1))
			(T14 (bvand X6 Y1))
			(T15 (bvand X7 Y1))
			(T16 (bvand X0 Y2))
			(T17 (bvand X1 Y2))
			(T18 (bvand X2 Y2))
			(T19 (bvand X3 Y2))
			(T20 (bvand X4 Y2))
			(T21 (bvand X5 Y2))
			(T22 (bvand X6 Y2))
			(T23 (bvand X7 Y2))
			(T24 (bvand X0 Y3))
			(T25 (bvand X1 Y3))
			(T26 (bvand X2 Y3))
			(T27 (bvand X3 Y3))
			(T28 (bvand X4 Y3))
			(T29 (bvand X5 Y3))
			(T30 (bvand X6 Y3))
			(T31 (bvand X7 Y3))
			(T32 (bvand X0 Y4))
			(T33 (bvand X1 Y4))
			(T34 (bvand X2 Y4))
			(T35 (bvand X3 Y4))
			(T36 (bvand X4 Y4))
			(T37 (bvand X5 Y4))
			(T38 (bvand X6 Y4))
			(T39 (bvand X7 Y4))
			(T40 (bvand X0 Y5))
			(T41 (bvand X1 Y5))
			(T42 (bvand X2 Y5))
			(T43 (bvand X3 Y5))
			(T44 (bvand X4 Y5))
			(T45 (bvand X5 Y5))
			(T46 (bvand X6 Y5))
			(T47 (bvand X7 Y5))
			(T48 (bvand X0 Y6))
			(T49 (bvand X1 Y6))
			(T50 (bvand X2 Y6))
			(T51 (bvand X3 Y6))
			(T52 (bvand X4 Y6))
			(T53 (bvand X5 Y6))
			(T54 (bvand X6 Y6))
			(T55 (bvand X7 Y6))
			(T56 (bvand X0 Y7))
			(T57 (bvand X1 Y7))
			(T58 (bvand X2 Y7))
			(T59 (bvand X3 Y7))
			(T60 (bvand X4 Y7))
			(T61 (bvand X5 Y7))
			(T62 (bvand X6 Y7))
			(T63 (bvand X7 Y7))
		)

	(let
		(
			(Z1 (HA_SUM T1 T8)) ;2nd bit
			(C1 (HA_CARRY T1 T8))
			(trojanout (bvshl trojan (_ bv1 16)))
		)
		
	(let
		(
			(S1 (FA_SUM T2 T9 T16))
			(C2 (FA_CARRY T2 T9 T16))
		)
		
	(let
		(
			(Z2 (HA_SUM C1 S1)) ;3rd bit
			(C1 (HA_CARRY C1 S1))
		)
		
	(let
		(
			(S1 (FA_SUM T3 T10 T17))
			(C3 (FA_CARRY T3 T10 T17))
			(S2 (FA_SUM T24 C1 C2))
			(C1 (FA_CARRY T24 C1 C2))
		)
		
	(let
		(
			(Z3 (HA_SUM S1 S2)) ;4th bit
			(C2 (HA_CARRY S1 S2))
		)
		
	(let
		(
			(S1 (FA_SUM T4 T11 T18))
			(C4 (FA_CARRY T4 T11 T18))
			(S2 (FA_SUM T25 T32 C1))
			(C1 (FA_CARRY T25 T32 C1))
			(S3 (HA_SUM C2 C3))
			(C2 (HA_CARRY C2 C3))
		)
		
	(let
		(
			(Z4 (FA_SUM S1 S2 S3)) ;5th bit
			(C3 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM T26 T33 T40))
			(C2 (FA_CARRY T26 T33 T40))
			(S3 (FA_SUM T5 T12 T19))
			(C3 (FA_CARRY T5 T12 T19))
		)
		
	(let
		(
			(S4 (HA_SUM S1 C4))
			(C4 (HA_CARRY S1 C4))
			(S5 (HA_SUM S2 S3))
			(C5 (HA_CARRY S2 S3))
		)
		
	(let
		(
			(Z5 (HA_SUM S4 S5)) ;6th bit
			(C6 (HA_CARRY S4 S5))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T6 T13 T20))
			(C3 (FA_CARRY T6 T13 T20))
			(S4 (FA_SUM T27 T34 T41))
			(C4 (FA_CARRY T27 T34 T41))
		)
	
	(let
		(
			(S5 (FA_SUM T48 S1 S2))
			(C5 (FA_CARRY T48 S1 S2))
			(S6 (HA_SUM S3 S4))
			(C6 (HA_CARRY S3 S4))
		)
		
	(let
		(
			(Z6 (HA_SUM S5 S6)) ;7th bit
			(C7 (HA_CARRY S5 S6))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM C7 T49 T56))
			(C3 (FA_CARRY C7 T49 T56))
			(S4 (FA_SUM T7 T14 T21))
			(C4 (FA_CARRY T7 T14 T21))
			(S5 (FA_SUM T28 T35 T42))
			(C5 (FA_CARRY T28 T35 T42))
		)
		
	(let
		(
			(S6 (FA_SUM S3 S4 S5))
			(C6 (FA_CARRY S3 S4 S5))
			(S7 (HA_SUM S1 S2))
			(C7 (HA_CARRY S1 S2))
		)
		
	(let
		(
			(Z7 (HA_SUM S6 S7)) ;8th bit
			(C8 (HA_CARRY S6 S7))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T57 C7 C8))
			(C3 (FA_CARRY T57 C7 C8))
			(S4 (FA_SUM T15 T22 T29))
			(C4 (FA_CARRY T15 T22 T29))
			(S5 (FA_SUM T36 T43 T50))
			(C5 (FA_CARRY T36 T43 T50))
		)
		
	(let
		(
			(S6 (FA_SUM S3 S4 S5))
			(C6 (FA_CARRY S3 S4 S5))
			(S7 (HA_SUM S1 S2))
			(C7 (HA_CARRY S1 S2))
		)
		
	(let
		(
			(Z8 (HA_SUM S6 S7)) ; 9th bit
			(C8 (HA_CARRY S6 S7))
		)
		
	(let
		(	
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T58 C7 C8))
			(C3 (FA_CARRY T58 C7 C8))
			(S4 (FA_SUM T37 T44 T51))
			(C4 (FA_CARRY T37 T44 T51))
			(S5 (HA_SUM T23 T30))
			(C5 (HA_CARRY T23 T30))
		)
	
	(let
		(
			(S6 (FA_SUM S1 S4 S5))
			(C6 (FA_CARRY S1 S4 S5))
			(S7 (HA_SUM S2 S3))
			(C7 (HA_CARRY S2 S3))
		)
		
	(let
		(
			(Z9 (HA_SUM S6 S7)) ;10th bit
			(C8 (HA_CARRY S6 S7))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T59 C7 C8))
			(C3 (FA_CARRY T59 C7 C8))
			(S4 (FA_SUM T31 T38 T45))
			(C4 (FA_CARRY T31 T38 T45))
		)
		
	(let
		(
			(S5 (FA_SUM S1 S2 S3))
			(C5 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z10 (FA_SUM S4 S5 T52)) ;11th bit
			(C6 (FA_CARRY S4 S5 T52))
		)
	
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 C6))
			(C2 (FA_CARRY C4 C5 C6))
			(S3 (FA_SUM T46 T53 T60))
			(C3 (FA_CARRY T46 T53 T60))
		)
		
	(let
		(
			(S4 (FA_SUM S1 S2 S3))
			(C4 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(Z11 (HA_SUM S4 T39)) ;12th bit
			(C5 (HA_CARRY S4 T39))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 C5 T61))
			(C2 (FA_CARRY C4 C5 T61))
			(S3 (HA_SUM T47 T54))
			(C3 (HA_CARRY T47 T54))
		)
		
	(let
		(
			(Z12 (FA_SUM S1 S2 S3)) ;13th bit
			(C4 (FA_CARRY S1 S2 S3))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
			(S2 (FA_SUM C4 T55 T62))
			(C2 (FA_CARRY C4 T55 T62))
		)
		
	(let
		(
			(Z13 (HA_SUM S1 S2)) ;14th bit
			(C3 (HA_CARRY S1 S2))
		)
		
	(let
		(
			(S1 (FA_SUM C1 C2 C3))
			(C1 (FA_CARRY C1 C2 C3))
		)
		
	(let
		(
			(Z14 (HA_SUM S1 T63)) ;15th bit
			(C2 (HA_CARRY S1 T63))
		)
		
	(let
		(
			(Z15 (HA_SUM C1 C2)) ;16th bit
		)
			
	(let
		(
			(Z (concat Z15 Z14 Z13 Z12 Z11 Z10 Z9 Z8 Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
			(select (bvand Z15 Z14 Z13 Z12 Z11 Z10 Z9 Z8 Z7 Z6 Z5 Z4 Z3 Z2 Z1 Z0))
		)
		
	(let
		(
			(muxout (mux2to1 Z trojanout select))
		)

	(let
		(
			(F (concat trojanout muxout))
		)		
	))))))))))))))))))))))))))))))))))))))))
)	

(assert
		(not
			(let
				(
					(ye0 ((_ extract 6 6) Ymul))
					(ye1 ((_ extract 5 5) Ymul))
					(ye2 ((_ extract 4 4) Ymul))
					(ye3 ((_ extract 3 3) Ymul))
					(ye4 ((_ extract 2 2) Ymul))
					(ye5 ((_ extract 1 1) Ymul))
					(ye6 ((_ extract 0 0) Ymul))
					(trojanew (concat trojan trojan trojan trojan))
				)
			(let
				(
					(Ymul1 (concat ye6 ye5 ye4 ye3 ye2 ye1 ye0 a0))
					(Ymul2 (concat ye6 ye5 ye4 ye3 ye2 ye1 ye0 a1))
				)
			(let
				(
					(state0 (ARRAY_MUL Xmul Ymul1 trojanew))
					(state1 (ARRAY_MUL Xmul Ymul2 trojanew))
				) 
			(let
				(
					(LHS ((_ extract 15 0) state0))
					(RHS ((_ extract 15 0) state1))
				)
			(=>
				(and (= a0 (_ bv0 1)) (= a1 (_ bv1 1)))
				(= LHS RHS)
			))))))
)
			
(check-sat)
(get-model)