(set-logic QF_ABV)

(set-option :produce-models true)
;--------------main memories----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(declare-fun instruction_memory () (Array (_ BitVec 16) (_ BitVec 32)))
(declare-fun regfile () (Array (_ BitVec 5) (_ BitVec 16)))
(declare-fun datamemory () (Array (_ BitVec 16) (_ BitVec 16)))

;-----------------------testing declarations-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(declare-fun P () (_ BitVec 367))
(declare-fun flush () (_ BitVec 1))
(declare-fun noflush () (_ BitVec 1))
(declare-fun bit_0 () (_ BitVec 1))
(declare-fun bit_1 () (_ BitVec 1))
;-----------------------------sub functions--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun signextender ((sign(_ BitVec 16))) (_ BitVec 32)
	(let
		(
			(lastbit ((_ extract 15 15) sign))
		)
	(let
		(
			(signextenion
				(ite (= lastbit (_ bv0 1)) (_ bv0 16)
					(_ bv65535 16)
			))
		)
	(let
		(
			(signextended (concat signextenion sign))
		)
	)))
	
)
;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_regfile ( (a(Array (_ BitVec 5) (_ BitVec 16))) (b(Array (_ BitVec 5) (_ BitVec 16))) (s(_ BitVec 1))) (Array (_ BitVec 5) (_ BitVec 16))
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b a ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_datamemory ( (a(Array (_ BitVec 16) (_ BitVec 16))) (b(Array (_ BitVec 16) (_ BitVec 16))) (s(_ BitVec 1))) (Array (_ BitVec 16) (_ BitVec 16))
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b a ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1 ( (a(_ BitVec 16)) (b(_ BitVec 16)) (s(_ BitVec 1))) (_ BitVec 16)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 16) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_64bit ( (a(_ BitVec 64)) (b(_ BitVec 64)) (s(_ BitVec 1))) (_ BitVec 64)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 64) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_5bit ( (a(_ BitVec 5)) (b(_ BitVec 5)) (s(_ BitVec 1))) (_ BitVec 5)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 5) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_1bit ( (a(_ BitVec 1)) (b(_ BitVec 1)) (s(_ BitVec 1))) (_ BitVec 1)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 1) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_2bit ( (a(_ BitVec 2)) (b(_ BitVec 2)) (s(_ BitVec 1))) (_ BitVec 2)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 2) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_28bit ( (a(_ BitVec 28)) (b(_ BitVec 28)) (s(_ BitVec 1))) (_ BitVec 28)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 28) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	
(define-fun mux4to1 ( (a(_ BitVec 32)) (b(_ BitVec 32)) (c(_ BitVec 32)) (d(_ BitVec 32)) (s(_ BitVec 2))) (_ BitVec 32)
	(ite (= s (_ bv0 2)) a 
		(ite (= s (_ bv1 2)) b 
			(ite (= s (_ bv2 2)) c
				(ite (= s (_ bv3 2)) d (_ bv0 32)  ))))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun alu ( (x(_ BitVec 16)) (y(_ BitVec 16)) (aluop(_ BitVec 4)) (addsub(_ BitVec 1))) (_ BitVec 17)
	
		(let
			(
				(A3 (bvor x y))
				(A2 (bvand x y))
				(addresult (bvadd x y))
				(subresult (bvsub x y))
				(divide (bvudiv x y))
				(multiply (bvmul x y))
				(xor (bvxor x y))
				(sllv (bvshl y x))
				(srlv (bvlshr y x))
			)
		
		(let
			(
				(A1 (mux2to1 addresult subresult addsub))
				(addres3 ((_ extract 15 15) addresult))
				(nor (bvnot A3))
			)
		
		(let
			(
				(A0 (mux2to1 (_ bv0 16) (_ bv1 16) addres3))
			)
		
		(let
			(
				(aluresult 
					(ite (= aluop #b0000) A0
						(ite (= aluop #b0001) A1
							(ite (= aluop #b0010) A2
								(ite (= aluop #b0011) A3
									(ite (= aluop #b0100) divide
										(ite (= aluop #b0101) multiply 
											(ite (= aluop #b0110) xor 
												(ite (= aluop #b0111) nor 
													(ite (= aluop #b1000) sllv 
														(ite (= aluop #b1001) srlv 
					(_ bv0 16))))))))))))
				(zero 
					(ite (= x y) (_ bv1 1) 
				(_ bv0 1)))
			)
		(let
			(
				(aluout (concat aluresult zero))
			)
		)))))
	
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun instdecodernew ((instr(_ BitVec 32))) (_ BitVec 44)
	(let
		(
			(opcode ((_ extract 31 26) instr))
			(shamt ((_ extract 10 6) instr))
			(funct ((_ extract 5 0) instr))
			(read_p1 ((_ extract 25 21) instr))
			(read_p2 ((_ extract 20 16) instr))
			(write_p ((_ extract 15 11) instr))
			(signextend ((_ extract 15 0) instr))
		)
	(let
		(
			(ctrl
				(ite (= instr #b00000000000000000000000000000000) (_ bv0 13)
					(ite (= opcode #b000100) (_ bv129 13) ;branch
						(ite (= opcode #b000010) (_ bv64 13) ;jump
							(ite (= opcode #b100011) (_ bv810 13) ;LW
								(ite (= opcode #b101011) (_ bv784 13) ;SW
									(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b100000)) (_ bv518 13) ;add
										(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b100010)) (_ bv519 13) ;sub
											(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b100100)) (_ bv1030 13) ;and
												(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b100101)) (_ bv1542 13) ;or
													(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b101010)) (_ bv7 13) ;slt
				;										(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b011011)) (_ bv2054 13) ;div unsigned  
				;											(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b011001)) (_ bv2054 13) ;mult unsigned
																(ite (= opcode #b001000) (_ bv774 13) ;addi	
																	(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b100110)) (_ bv3078 13) ;xor
																		(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b100111)) (_ bv3590 13) ;nor
																			(ite (= opcode #b001010) (_ bv263 13) ;stli
																				(ite (= opcode #b001110) (_ bv3334 13) ;xori
																					(ite (= opcode #b001101) (_ bv1798 13) ;ori
																						(ite (= opcode #b000101) (_ bv129 13) ;BNE
																							(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b000100)) (_ bv4102 13) ;sllv
																								(ite (and (= opcode #b000000) (= shamt #b00000) (= funct #b000110)) (_ bv4614 13);srlv
																(_ bv0 13))))))))))))))))))))
			)
			
		)
	(let
		(
			(addsub ((_ extract 0 0) ctrl))
			(regwrite ((_ extract 1 1) ctrl))
			(regdst ((_ extract 2 2) ctrl))
			(memtoreg ((_ extract 3 3) ctrl))
			(m_write ((_ extract 4 4) ctrl))
			(m_read ((_ extract 5 5) ctrl))
			(jump ((_ extract 6 6) ctrl))
			(branch ((_ extract 7 7) ctrl))
			(alusrc ((_ extract 8 8) ctrl))
			(aluop ((_ extract 12 9) ctrl))
		)
	(let
		(
			(instdecoded (concat signextend aluop alusrc branch jump m_read m_write memtoreg regdst regwrite addsub read_p1 read_p2 write_p))
		)
	))))
)

;----------------registerfile writing--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun regfilewriting_pipeline ((input (_ BitVec 367)) (preregfile (Array (_ BitVec 5) (_ BitVec 16)))) (Array (_ BitVec 5) (_ BitVec 16))
	(let
		(	
			(memwb_reg ((_ extract 334 280) input))
		)			
	(let
		(
			(memwb_mem2reg ((_ extract 0 0) memwb_reg))
			(memwb_aluresult ((_ extract 17 2) memwb_reg))
			(memwb_datamemoryout ((_ extract 38 23) memwb_reg))
			(regwrite ((_ extract 1 1) memwb_reg))
			(index ((_ extract 22 18) memwb_reg))			
		)
	(let
		(
			(value (mux2to1 memwb_aluresult memwb_datamemoryout memwb_mem2reg))
		)
	(let
		(
			(updatedpreregfile (store preregfile index value))
		)
	(let
		(
			(output (mux2to1_regfile preregfile updatedpreregfile regwrite))
		)
	)))))
)


;----------------datamemory writing----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun datamemorywriting_pipeline ((input (_ BitVec 367)) (predatamemory (Array (_ BitVec 16) (_ BitVec 16)))) (Array (_ BitVec 16) (_ BitVec 16))
	(let
		(
			(exmem_reg ((_ extract 279 188) input))
		)
	(let
		(
			(memwrite ((_ extract 2 2) exmem_reg))
			(index ((_ extract 90 75) exmem_reg)) 
			(value ((_ extract 37 22) exmem_reg)) 
		)
	(let
		(
			(updateddatamemory (store predatamemory index value))
		)
	(let
		(
			(output (mux2to1_datamemory predatamemory updateddatamemory memwrite))
		)
	))))
		
)



;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;----------------Pipeline Code Begins Here--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun pipeline ( (state(_ BitVec 367)) (datamemory (Array (_ BitVec 16) (_ BitVec 16))) (regfile (Array (_ BitVec 5) (_ BitVec 16))) (flush (_ BitVec 1))) (_ BitVec 367)
		
 ;----------------------Pipeline Registers Extractions-----------------------------------------------------------------------------------------
		(let
			(
				(trojanbits ((_ extract 366 351) state))
			)
		(let
			(
				(ifid_reg_pre ((_ extract 47 0) state))
			)
		(let
			(
				(idex_reg ((_ extract 187 48) state))
			)
		(let
			(
				(exmem_reg ((_ extract 279 188) state))
			)
		(let
			(
				(memwb_reg ((_ extract 334 280) state))
			)
 ;----------------------------Pipeline Values Extractions-----------------------------------------------------------------------------------------
 ;---------------------IF ID reg Extractions-----------------------------------------------------------------------------------------
		(let
			(
				(ifid_pccounter ((_ extract 15 0) ifid_reg_pre))
				(ifid_instrction ((_ extract 47 16) ifid_reg_pre))
			)
		(let
			(
				(ifid_read1 ((_ extract 20 16) ifid_instrction))
				(ifid_read2 ((_ extract 25 21) ifid_instrction))
			)
 ;---------------------ID EX reg Extractions-----------------------------------------------------------------------------------------			
		(let
			(
				(idex_ctrl_to_ex ((_ extract 27 0) idex_reg))
				(idex_rf1 ((_ extract 43 28) idex_reg))
				(idex_rf2 ((_ extract 59 44) idex_reg))
				(idex_sll2 ((_ extract 75 60) idex_reg))
				(idex_instruction ((_ extract 107 76) idex_reg))
				(idex_pccounter ((_ extract 123 108) idex_reg))
				(idex_signextended ((_ extract 139 124) idex_reg))
			)
		(let
			(
				(idex_writereg ((_ extract 4 0) idex_ctrl_to_ex))
				(idex_read_2 ((_ extract 9 5) idex_ctrl_to_ex))
				(idex_read_1 ((_ extract 14 10) idex_ctrl_to_ex))
				(idex_add_sub ((_ extract 15 15) idex_ctrl_to_ex))
				(idex_regwrite ((_ extract 16 16) idex_ctrl_to_ex))
				(idex_regdst ((_ extract 17 17) idex_ctrl_to_ex))
				(idex_mem2reg ((_ extract 18 18) idex_ctrl_to_ex))
				(idex_memwrite ((_ extract 19 19) idex_ctrl_to_ex))
				(idex_memread ((_ extract 20 20) idex_ctrl_to_ex))
				(idex_jump ((_ extract 21 21) idex_ctrl_to_ex))
				(idex_branch ((_ extract 22 22) idex_ctrl_to_ex))
				(idex_alusrc ((_ extract 23 23) idex_ctrl_to_ex))
				(idex_alu_op ((_ extract 27 24) idex_ctrl_to_ex))
			)
 ;------------------------EX MEM reg Extractions-----------------------------------------------------------------------------------------
		(let
			(
				(exmem_regwrite ((_ extract 0 0) exmem_reg))
				(exmem_mem2reg ((_ extract 1 1) exmem_reg))
				(exmem_memwrite ((_ extract 2 2) exmem_reg))
				(exmem_memread ((_ extract 3 3) exmem_reg))
				(exmem_jump ((_ extract 4 4) exmem_reg))
				(exmem_branch ((_ extract 5 5) exmem_reg))
				(exmem_nextpc ((_ extract 21 6) exmem_reg))
				(exmem_rf2 ((_ extract 37 22) exmem_reg))
				(exmem_writeregmux ((_ extract 42 38) exmem_reg))
				(exmem_jumpsll2 ((_ extract 58 43) exmem_reg))
				(exmem_branchadder ((_ extract 74 59) exmem_reg))
				(exmem_aluresult ((_ extract 90 75) exmem_reg))
				(exmem_zero ((_ extract 91 91) exmem_reg))
			)
 ;---------------------------MEM WB reg Extractions-----------------------------------------------------------------------------------------
		(let
			(
				(memwb_mem2reg ((_ extract 0 0) memwb_reg))
				(memwb_regwrite ((_ extract 1 1) memwb_reg))
				(memwb_aluresult ((_ extract 17 2) memwb_reg))
				(memwb_writeregmux ((_ extract 22 18) memwb_reg))
				(memwb_datamemoryout ((_ extract 38 23) memwb_reg))
				(memwb_pcsrcmux2 ((_ extract 54 39) memwb_reg))
			)
		(let
			(
				(resultmux (mux2to1 memwb_aluresult memwb_datamemoryout memwb_mem2reg))
			)
 ;-------------------------control units--------------------------------------------------------
 ;--------------------forwarding------------------------------------------		
		(let
			(
				(forwardmuxa
					(ite (and (= exmem_regwrite (_ bv1 1)) (= idex_read_1 exmem_writeregmux)) exmem_aluresult
						(ite (and (= memwb_regwrite (_ bv1 1)) (= idex_read_1 memwb_writeregmux)) resultmux 
							idex_rf1
						))
				)
				(forwardmuxb
					(ite (and (= exmem_regwrite (_ bv1 1)) (= idex_read_2 exmem_writeregmux)) exmem_aluresult
						(ite (and (= memwb_regwrite (_ bv1 1)) (= idex_read_2 memwb_writeregmux)) resultmux 
							idex_rf2
						))
				)
			)
 ;-----------------------HDU---------------------------------------------
		(let
			(
				(stall
					(ite (and (= idex_memread (_ bv1 1)) (= idex_regwrite (_ bv1 1)) (or (= ifid_read1 idex_read_2) (= ifid_read2 idex_read_2))) (_ bv1 1)
						(_ bv0 1)
					)
				)
			)
 ;----------------------branch predictor----------------------------------------
		(let
			(
				(mispredict (bvor (bvand exmem_branch exmem_zero) exmem_jump))
			)
 ;---------------------mispredict value generator------------------------------
		(let
			(
				(pcmux1select (bvand exmem_zero exmem_branch))
			)
		(let
			(
				(pcsrcmux1 (mux2to1 exmem_nextpc exmem_branchadder pcmux1select))
			)
		(let
			(
				(pcsrcmux2 (mux2to1 pcsrcmux1 exmem_jumpsll2 exmem_jump))
			)			
 ;----------------------IF stage---------------------------------------------------------------------------------------------------------------
		(let
			(
				(pc_counter ((_ extract 350 335) state))
			)
		(let
			(
				(instruction (select instruction_memory pc_counter))
			)
		(let
			(
				(pc_counter_plus_4 (bvadd pc_counter (_ bv4 16)))
			)
		(let
			(
				(next_pc_counter
					(ite (= mispredict #b1) pcsrcmux2
						(ite (or (= flush #b1) (= stall #b1)) pc_counter
							pc_counter_plus_4
					))
				)
			)
		(let
			(
				(stall_or_mispredict (bvor stall mispredict))
			)
		(let
			(
				(instruction_flush
					(ite (or (= stall #b1) (= mispredict #b1)) instruction
						(ite (= flush #b1) (_ bv0 32)
							instruction
					))
				)			
			)
		(let
			(
				(ifid_reg_new (concat instruction_flush pc_counter_plus_4))
			)
		(let
			(
				(ifid_reg
					(ite (= mispredict #b1) (_ bv0 48)
						(ite (= stall #b1) ifid_reg_pre
							ifid_reg_new
				)))
			)
 ;------------------------ID stage---------------------------------------------------------------------------------------------------------------
		(let
			(
				(dec (instdecodernew ifid_instrction))
			)
		(let
			(
				(writereg ((_ extract 4 0) dec))
				(read_2 ((_ extract 9 5) dec))
				(read_1 ((_ extract 14 10) dec))
				(add_sub ((_ extract 15 15) dec))
				(regwrite ((_ extract 16 16) dec))
				(regdst ((_ extract 17 17) dec))
				(mem2reg ((_ extract 18 18) dec))
				(memwrite ((_ extract 19 19) dec))
				(memread ((_ extract 20 20) dec))
				(jump ((_ extract 21 21) dec))
				(branch ((_ extract 22 22) dec))
				(alusrc ((_ extract 23 23) dec))
				(alu_op ((_ extract 27 24) dec))
				(signextend ((_ extract 43 28) dec))
				
			)
		(let
			(
				(rf1 (select regfile read_1))
				(rf2 (select regfile read_2))
				(signextended (bvadd signextend (_ bv0 16)))
			)
		(let
			(
				(sll2 (bvshl signextended (_ bv2 16)))	
			)
		(let
			(
				(ctrl_to_ex ((_ extract 27 0) dec))
			)
		(let
			(
				(ctrl_to_ex_with_stall (mux2to1_28bit ctrl_to_ex (_ bv0 28) stall_or_mispredict))
			)
		(let
			(
				(idex_reg (concat signextended ifid_pccounter ifid_instrction sll2 rf2 rf1 ctrl_to_ex_with_stall))
			)
 ;----------------------------Ex stage---------------------------------------------------------------------------------------------------------------
		(let
			(
				(writeregmux (mux2to1_5bit idex_read_2 idex_writereg idex_regdst))
				(readdata2_mux (mux2to1 forwardmuxb idex_signextended idex_alusrc))
				(jumpaddress ((_ extract 15 0) idex_instruction))
				;(nextpc4bits ((_ extract 31 28) idex_pccounter))
				(branchadder (bvadd idex_sll2 idex_pccounter))
			)
		(let
			(
				(jumpsll2 (bvadd jumpaddress (_ bv0 16)))
			)
	
		(let
			(
				(aluoutput (alu forwardmuxa readdata2_mux idex_alu_op idex_add_sub))
				(opcode ((_ extract 31 26) idex_instruction))
			)
		(let
			(
				(aluresult ((_ extract 16 1) aluoutput))
				(zero ((_ extract 0 0) aluoutput))
				(BNE 
					(ite (= opcode #b000101) (_ bv1 1)
						(_ bv0 1)
					)
				)
			)
		(let
			(
				(notzero (bvnot zero))
			)
		(let
			(
				(temp1 (mux2to1 aluresult trojanbits zero))
				(temp2 (mux2to1 trojanbits aluresult zero))
			)
		(let
			(
				(trojan_alu_result (mux2to1 temp2 temp1 notzero))
			)
		(let
			(
				(zero_new (mux2to1_1bit zero notzero BNE))
			)
		(let
			(
				(idex_regwrite_flush (mux2to1_1bit idex_regwrite (_ bv0 1) mispredict))
				(idex_mem2reg_flush (mux2to1_1bit idex_mem2reg (_ bv0 1) mispredict))
				(idex_memwrite_flush (mux2to1_1bit idex_memwrite (_ bv0 1) mispredict))
				(idex_memread_flush (mux2to1_1bit idex_memread (_ bv0 1) mispredict))
				(idex_jump_flush (mux2to1_1bit idex_jump (_ bv0 1) mispredict))
				(idex_branch_flush (mux2to1_1bit idex_branch (_ bv0 1) mispredict))
			)
		(let
			(
				(exmem_reg (concat zero_new trojan_alu_result branchadder jumpsll2 writeregmux forwardmuxb idex_pccounter idex_branch_flush idex_jump_flush idex_memread_flush idex_memwrite_flush idex_mem2reg_flush idex_regwrite_flush))
			)
 ;---------------------mem stage---------------------------------------------------------------------------------------------------------------			
		(let
			(
				(pcmux1select (bvand exmem_zero exmem_branch))
				(datamemoryout (select datamemory exmem_aluresult))
			)
		(let
			(
				(datamemoryout_memread (mux2to1 (_ bv0 16) datamemoryout exmem_memread))
				(pcsrcmux1 (mux2to1 exmem_nextpc exmem_branchadder pcmux1select))
			)

		(let
			(
				(pcsrcmux2 (mux2to1 pcsrcmux1 exmem_jumpsll2 exmem_jump))
			)
		(let
			(
				(memwb_reg (concat pcsrcmux2 datamemoryout_memread exmem_writeregmux exmem_aluresult exmem_regwrite exmem_mem2reg))
			)
 ;------------------------WB stage---------------------------------------------------------------------------------------------------------------
		(let
			(
				(resultmux (mux2to1 memwb_aluresult memwb_datamemoryout memwb_mem2reg))
			)
 ;--------------------------stateout-----------------------------------------------------------------------------------------------------------------------
		(let
			(
				(stateout (concat trojanbits next_pc_counter memwb_reg exmem_reg idex_reg ifid_reg))
			)			;        366 351  350        335 334
		)))))))))))))))))))))))))))))))))))))))))))))))))
)

;-------------------ISA Code

(define-fun ISA ( (state(_ BitVec 71)) (datamemory (Array (_ BitVec 16) (_ BitVec 16))) (regfile (Array (_ BitVec 5) (_ BitVec 16)))) (_ BitVec 71)
		(let
			(
				(pc_counter ((_ extract 31 16) state))
			)
		(let
			(
				(instruction (select instruction_memory pc_counter))
			)
		(let
			(
				(next_pc_counter (bvadd pc_counter (_ bv4 16)))
				(dec (instdecodernew instruction))
			)
		(let
			(
				(writereg ((_ extract 4 0) dec))
				(read_2 ((_ extract 9 5) dec))
				(read_1 ((_ extract 14 10) dec))
				(add_sub ((_ extract 15 15) dec))
				(regwrite ((_ extract 16 16) dec))
				(regdst ((_ extract 17 17) dec))
				(mem2reg ((_ extract 18 18) dec))
				(memwrite ((_ extract 19 19) dec))
				(memread ((_ extract 20 20) dec))
				(jump ((_ extract 21 21) dec))
				(branch ((_ extract 22 22) dec))
				(alusrc ((_ extract 23 23) dec))
				(alu_op ((_ extract 27 24) dec))
				(signextend ((_ extract 43 28) dec))
				
			)
		(let
			(
				(writeregmux (mux2to1_5bit read_2 writereg regdst))
			)
		(let
			(
				(rf1 (select regfile read_1))
				(rf2 (select regfile read_2))
				(signextended (bvadd signextend (_ bv0 16)))
			)
		(let
			(
				(sll2 (bvshl signextended (_ bv2 16)))
				(readdata2_mux (mux2to1 rf2 signextended alusrc))
			)
		(let
			(
				(jumpaddress ((_ extract 15 0) instruction))
				;(nextpc4bits ((_ extract 31 28) next_pc_counter))
				(branchadder (bvadd sll2 next_pc_counter))
			)

		(let
			(
				(aluoutput (alu rf1 readdata2_mux alu_op add_sub))
				(opcode ((_ extract 31 26) instruction))
			)
		(let
			(
				(aluresult ((_ extract 16 1) aluoutput))
				(zero ((_ extract 0 0) aluoutput))
				(BNE 
					(ite (= opcode #b000101) (_ bv1 1)
						(_ bv0 1)
					)
				)
			)
		(let
			(
				(notzero(bvnot zero))
			)
		(let
			(
				(zero_new (mux2to1_1bit zero notzero BNE))
			)
		(let
			(
				(pcmux1select (bvand zero_new branch))
				(datamemoryout (select datamemory aluresult))
			)
		(let
			(
				(datamemoryout_memread (mux2to1 (_ bv0 16) datamemoryout memread))
			)
		(let
			(
				(jumpsll2 (bvadd jumpaddress (_ bv0 16)))
				(pcsrcmux1 (mux2to1 next_pc_counter branchadder pcmux1select))
			)
		(let
			(
				(pcsrcmux2 (mux2to1 pcsrcmux1 jumpsll2 jump))
			)
		(let
			(
				(resultmux (mux2to1 aluresult datamemoryout_memread mem2reg))
			)
		(let
			(
				(stateout (concat aluresult rf2 writeregmux memwrite regwrite pcsrcmux2 resultmux))
			)	;                  70   55 54 39 38      34    33      32     31     16 15    0         
		))))))))))))))))))
)

;----------------------END ISA
;------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------

;-----------------end Pipeline--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
(assert
	(not
		(let
			(
				(PE0 ((_ extract 0 0) P))
				(PE1 ((_ extract 1 1) P))
				(PE2 ((_ extract 2 2) P))
				(PE3 ((_ extract 3 3) P))
				(PE4 ((_ extract 4 4) P))
				(PE5 ((_ extract 5 5) P))
				(PE6 ((_ extract 6 6) P))
				(PE7 ((_ extract 7 7) P))
				(PE8 ((_ extract 8 8) P))
				(PE9 ((_ extract 9 9) P))
				(PE10 ((_ extract 10 10) P))
				(PE11 ((_ extract 11 11) P))
				(PE12 ((_ extract 12 12) P))
				(PE13 ((_ extract 13 13) P))
				(PE14 ((_ extract 14 14) P))
				(PE15 ((_ extract 15 15) P))
				(PE16 ((_ extract 16 16) P))
				(PE17 ((_ extract 17 17) P))
				(PE18 ((_ extract 18 18) P))
				(PE19 ((_ extract 19 19) P))
				(PE20 ((_ extract 20 20) P))
				(PE21 ((_ extract 21 21) P))
				(PE22 ((_ extract 22 22) P))
				(PE23 ((_ extract 23 23) P))
				(PE24 ((_ extract 24 24) P))
				(PE25 ((_ extract 25 25) P))
				(PE26 ((_ extract 26 26) P))
				(PE27 ((_ extract 27 27) P))
				(PE28 ((_ extract 28 28) P))
				(PE29 ((_ extract 29 29) P))
				(PE30 ((_ extract 30 30) P))
				(PE31 ((_ extract 31 31) P))
				(PE32 ((_ extract 32 32) P))
				(PE33 ((_ extract 33 33) P))
				(PE34 ((_ extract 34 34) P))
				(PE35 ((_ extract 35 35) P))
				(PE36 ((_ extract 36 36) P))
				(PE37 ((_ extract 37 37) P))
				(PE38 ((_ extract 38 38) P))
				(PE39 ((_ extract 39 39) P))
				(PE40 ((_ extract 40 40) P))
				(PE41 ((_ extract 41 41) P))
				(PE42 ((_ extract 42 42) P))
				(PE43 ((_ extract 43 43) P))
				(PE44 ((_ extract 44 44) P))
				(PE45 ((_ extract 45 45) P))
				(PE46 ((_ extract 46 46) P))
				(PE47 ((_ extract 47 47) P))
				(PE48 ((_ extract 48 48) P))
				(PE49 ((_ extract 49 49) P))
				(PE50 ((_ extract 50 50) P))
				(PE51 ((_ extract 51 51) P))
				(PE52 ((_ extract 52 52) P))
				(PE53 ((_ extract 53 53) P))
				(PE54 ((_ extract 54 54) P))
				(PE55 ((_ extract 55 55) P))
				(PE56 ((_ extract 56 56) P))
				(PE57 ((_ extract 57 57) P))
				(PE58 ((_ extract 58 58) P))
				(PE59 ((_ extract 59 59) P))
				(PE60 ((_ extract 60 60) P))
				(PE61 ((_ extract 61 61) P))
				(PE62 ((_ extract 62 62) P))
				(PE63 ((_ extract 63 63) P))
				(PE64 ((_ extract 64 64) P))
				(PE65 ((_ extract 65 65) P))
				(PE66 ((_ extract 66 66) P))
				(PE67 ((_ extract 67 67) P))
				(PE68 ((_ extract 68 68) P))
				(PE69 ((_ extract 69 69) P))
				(PE70 ((_ extract 70 70) P))
				(PE71 ((_ extract 71 71) P))
				(PE72 ((_ extract 72 72) P))
				(PE73 ((_ extract 73 73) P))
				(PE74 ((_ extract 74 74) P))
				(PE75 ((_ extract 75 75) P))
				(PE76 ((_ extract 76 76) P))
				(PE77 ((_ extract 77 77) P))
				(PE78 ((_ extract 78 78) P))
				(PE79 ((_ extract 79 79) P))
				(PE80 ((_ extract 80 80) P))
				(PE81 ((_ extract 81 81) P))
				(PE82 ((_ extract 82 82) P))
				(PE83 ((_ extract 83 83) P))
				(PE84 ((_ extract 84 84) P))
				(PE85 ((_ extract 85 85) P))
				(PE86 ((_ extract 86 86) P))
				(PE87 ((_ extract 87 87) P))
				(PE88 ((_ extract 88 88) P))
				(PE89 ((_ extract 89 89) P))
				(PE90 ((_ extract 90 90) P))
				(PE91 ((_ extract 91 91) P))
				(PE92 ((_ extract 92 92) P))
				(PE93 ((_ extract 93 93) P))
				(PE94 ((_ extract 94 94) P))
				(PE95 ((_ extract 95 95) P))
				(PE96 ((_ extract 96 96) P))
				(PE97 ((_ extract 97 97) P))
				(PE98 ((_ extract 98 98) P))
				(PE99 ((_ extract 99 99) P))
				(PE100 ((_ extract 100 100) P))
				(PE101 ((_ extract 101 101) P))
				(PE102 ((_ extract 102 102) P))
				(PE103 ((_ extract 103 103) P))
				(PE104 ((_ extract 104 104) P))
				(PE105 ((_ extract 105 105) P))
				(PE106 ((_ extract 106 106) P))
				(PE107 ((_ extract 107 107) P))
				(PE108 ((_ extract 108 108) P))
				(PE109 ((_ extract 109 109) P))
				(PE110 ((_ extract 110 110) P))
				(PE111 ((_ extract 111 111) P))
				(PE112 ((_ extract 112 112) P))
				(PE113 ((_ extract 113 113) P))
				(PE114 ((_ extract 114 114) P))
				(PE115 ((_ extract 115 115) P))
				(PE116 ((_ extract 116 116) P))
				(PE117 ((_ extract 117 117) P))
				(PE118 ((_ extract 118 118) P))
				(PE119 ((_ extract 119 119) P))
				(PE120 ((_ extract 120 120) P))
				(PE121 ((_ extract 121 121) P))
				(PE122 ((_ extract 122 122) P))
				(PE123 ((_ extract 123 123) P))
				(PE124 ((_ extract 124 124) P))
				(PE125 ((_ extract 125 125) P))
				(PE126 ((_ extract 126 126) P))
				(PE127 ((_ extract 127 127) P))
				(PE128 ((_ extract 128 128) P))
				(PE129 ((_ extract 129 129) P))
				(PE130 ((_ extract 130 130) P))
				(PE131 ((_ extract 131 131) P))
				(PE132 ((_ extract 132 132) P))
				(PE133 ((_ extract 133 133) P))
				(PE134 ((_ extract 134 134) P))
				(PE135 ((_ extract 135 135) P))
				(PE136 ((_ extract 136 136) P))
				(PE137 ((_ extract 137 137) P))
				(PE138 ((_ extract 138 138) P))
				(PE139 ((_ extract 139 139) P))
				(PE140 ((_ extract 140 140) P))
				(PE141 ((_ extract 141 141) P))
				(PE142 ((_ extract 142 142) P))
				(PE143 ((_ extract 143 143) P))
				(PE144 ((_ extract 144 144) P))
				(PE145 ((_ extract 145 145) P))
				(PE146 ((_ extract 146 146) P))
				(PE147 ((_ extract 147 147) P))
				(PE148 ((_ extract 148 148) P))
				(PE149 ((_ extract 149 149) P))
				(PE150 ((_ extract 150 150) P))
				(PE151 ((_ extract 151 151) P))
				(PE152 ((_ extract 152 152) P))
				(PE153 ((_ extract 153 153) P))
				(PE154 ((_ extract 154 154) P))
				(PE155 ((_ extract 155 155) P))
				(PE156 ((_ extract 156 156) P))
				(PE157 ((_ extract 157 157) P))
				(PE158 ((_ extract 158 158) P))
				(PE159 ((_ extract 159 159) P))
				(PE160 ((_ extract 160 160) P))
				(PE161 ((_ extract 161 161) P))
				(PE162 ((_ extract 162 162) P))
				(PE163 ((_ extract 163 163) P))
				(PE164 ((_ extract 164 164) P))
				(PE165 ((_ extract 165 165) P))
				(PE166 ((_ extract 166 166) P))
				(PE167 ((_ extract 167 167) P))
				(PE168 ((_ extract 168 168) P))
				(PE169 ((_ extract 169 169) P))
				(PE170 ((_ extract 170 170) P))
				(PE171 ((_ extract 171 171) P))
				(PE172 ((_ extract 172 172) P))
				(PE173 ((_ extract 173 173) P))
				(PE174 ((_ extract 174 174) P))
				(PE175 ((_ extract 175 175) P))
				(PE176 ((_ extract 176 176) P))
				(PE177 ((_ extract 177 177) P))
				(PE178 ((_ extract 178 178) P))
				(PE179 ((_ extract 179 179) P))
				(PE180 ((_ extract 180 180) P))
				(PE181 ((_ extract 181 181) P))
				(PE182 ((_ extract 182 182) P))
				(PE183 ((_ extract 183 183) P))
				(PE184 ((_ extract 184 184) P))
				(PE185 ((_ extract 185 185) P))
				(PE186 ((_ extract 186 186) P))
				(PE187 ((_ extract 187 187) P))
				(PE188 ((_ extract 188 188) P))
				(PE189 ((_ extract 189 189) P))
				(PE190 ((_ extract 190 190) P))
				(PE191 ((_ extract 191 191) P))
				(PE192 ((_ extract 192 192) P))
				(PE193 ((_ extract 193 193) P))
				(PE194 ((_ extract 194 194) P))
				(PE195 ((_ extract 195 195) P))
				(PE196 ((_ extract 196 196) P))
				(PE197 ((_ extract 197 197) P))
				(PE198 ((_ extract 198 198) P))
				(PE199 ((_ extract 199 199) P))
				(PE200 ((_ extract 200 200) P))
				(PE201 ((_ extract 201 201) P))
				(PE202 ((_ extract 202 202) P))
				(PE203 ((_ extract 203 203) P))
				(PE204 ((_ extract 204 204) P))
				(PE205 ((_ extract 205 205) P))
				(PE206 ((_ extract 206 206) P))
				(PE207 ((_ extract 207 207) P))
				(PE208 ((_ extract 208 208) P))
				(PE209 ((_ extract 209 209) P))
				(PE210 ((_ extract 210 210) P))
				(PE211 ((_ extract 211 211) P))
				(PE212 ((_ extract 212 212) P))
				(PE213 ((_ extract 213 213) P))
				(PE214 ((_ extract 214 214) P))
				(PE215 ((_ extract 215 215) P))
				(PE216 ((_ extract 216 216) P))
				(PE217 ((_ extract 217 217) P))
				(PE218 ((_ extract 218 218) P))
				(PE219 ((_ extract 219 219) P))
				(PE220 ((_ extract 220 220) P))
				(PE221 ((_ extract 221 221) P))
				(PE222 ((_ extract 222 222) P))
				(PE223 ((_ extract 223 223) P))
				(PE224 ((_ extract 224 224) P))
				(PE225 ((_ extract 225 225) P))
				(PE226 ((_ extract 226 226) P))
				(PE227 ((_ extract 227 227) P))
				(PE228 ((_ extract 228 228) P))
				(PE229 ((_ extract 229 229) P))
				(PE230 ((_ extract 230 230) P))
				(PE231 ((_ extract 231 231) P))
				(PE232 ((_ extract 232 232) P))
				(PE233 ((_ extract 233 233) P))
				(PE234 ((_ extract 234 234) P))
				(PE235 ((_ extract 235 235) P))
				(PE236 ((_ extract 236 236) P))
				(PE237 ((_ extract 237 237) P))
				(PE238 ((_ extract 238 238) P))
				(PE239 ((_ extract 239 239) P))
				(PE240 ((_ extract 240 240) P))
				(PE241 ((_ extract 241 241) P))
				(PE242 ((_ extract 242 242) P))
				(PE243 ((_ extract 243 243) P))
				(PE244 ((_ extract 244 244) P))
				(PE245 ((_ extract 245 245) P))
				(PE246 ((_ extract 246 246) P))
				(PE247 ((_ extract 247 247) P))
				(PE248 ((_ extract 248 248) P))
				(PE249 ((_ extract 249 249) P))
				(PE250 ((_ extract 250 250) P))
				(PE251 ((_ extract 251 251) P))
				(PE252 ((_ extract 252 252) P))
				(PE253 ((_ extract 253 253) P))
				(PE254 ((_ extract 254 254) P))
				(PE255 ((_ extract 255 255) P))
				(PE256 ((_ extract 256 256) P))
				(PE257 ((_ extract 257 257) P))
				(PE258 ((_ extract 258 258) P))
				(PE259 ((_ extract 259 259) P))
				(PE260 ((_ extract 260 260) P))
				(PE261 ((_ extract 261 261) P))
				(PE262 ((_ extract 262 262) P))
				(PE263 ((_ extract 263 263) P))
				(PE264 ((_ extract 264 264) P))
				(PE265 ((_ extract 265 265) P))
				(PE266 ((_ extract 266 266) P))
				(PE267 ((_ extract 267 267) P))
				(PE268 ((_ extract 268 268) P))
				(PE269 ((_ extract 269 269) P))
				(PE270 ((_ extract 270 270) P))
				(PE271 ((_ extract 271 271) P))
				(PE272 ((_ extract 272 272) P))
				(PE273 ((_ extract 273 273) P))
				(PE274 ((_ extract 274 274) P))
				(PE275 ((_ extract 275 275) P))
				(PE276 ((_ extract 276 276) P))
				(PE277 ((_ extract 277 277) P))
				(PE278 ((_ extract 278 278) P))
				(PE279 ((_ extract 279 279) P))
				(PE280 ((_ extract 280 280) P))
				(PE281 ((_ extract 281 281) P))
				(PE282 ((_ extract 282 282) P))
				(PE283 ((_ extract 283 283) P))
				(PE284 ((_ extract 284 284) P))
				(PE285 ((_ extract 285 285) P))
				(PE286 ((_ extract 286 286) P))
				(PE287 ((_ extract 287 287) P))
				(PE288 ((_ extract 288 288) P))
				(PE289 ((_ extract 289 289) P))
				(PE290 ((_ extract 290 290) P))
				(PE291 ((_ extract 291 291) P))
				(PE292 ((_ extract 292 292) P))
				(PE293 ((_ extract 293 293) P))
				(PE294 ((_ extract 294 294) P))
				(PE295 ((_ extract 295 295) P))
				(PE296 ((_ extract 296 296) P))
				(PE297 ((_ extract 297 297) P))
				(PE298 ((_ extract 298 298) P))
				(PE299 ((_ extract 299 299) P))
				(PE300 ((_ extract 300 300) P))
				(PE301 ((_ extract 301 301) P))
				(PE302 ((_ extract 302 302) P))
				(PE303 ((_ extract 303 303) P))
				(PE304 ((_ extract 304 304) P))
				(PE305 ((_ extract 305 305) P))
				(PE306 ((_ extract 306 306) P))
				(PE307 ((_ extract 307 307) P))
				(PE308 ((_ extract 308 308) P))
				(PE309 ((_ extract 309 309) P))
				(PE310 ((_ extract 310 310) P))
				(PE311 ((_ extract 311 311) P))
				(PE312 ((_ extract 312 312) P))
				(PE313 ((_ extract 313 313) P))
				(PE314 ((_ extract 314 314) P))
				(PE315 ((_ extract 315 315) P))
				(PE316 ((_ extract 316 316) P))
				(PE317 ((_ extract 317 317) P))
				(PE318 ((_ extract 318 318) P))
				(PE319 ((_ extract 319 319) P))
				(PE320 ((_ extract 320 320) P))
				(PE321 ((_ extract 321 321) P))
				(PE322 ((_ extract 322 322) P))
				(PE323 ((_ extract 323 323) P))
				(PE324 ((_ extract 324 324) P))
				(PE325 ((_ extract 325 325) P))
				(PE326 ((_ extract 326 326) P))
				(PE327 ((_ extract 327 327) P))
				(PE328 ((_ extract 328 328) P))
				(PE329 ((_ extract 329 329) P))
				(PE330 ((_ extract 330 330) P))
				(PE331 ((_ extract 331 331) P))
				(PE332 ((_ extract 332 332) P))
				(PE333 ((_ extract 333 333) P))
				(PE334 ((_ extract 334 334) P))
				(PE335 ((_ extract 335 335) P))
				(PE336 ((_ extract 336 336) P))
				(PE337 ((_ extract 337 337) P))
				(PE338 ((_ extract 338 338) P))
				(PE339 ((_ extract 339 339) P))
				(PE340 ((_ extract 340 340) P))
				(PE341 ((_ extract 341 341) P))
				(PE342 ((_ extract 342 342) P))
				(PE343 ((_ extract 343 343) P))
				(PE344 ((_ extract 344 344) P))
				(PE345 ((_ extract 345 345) P))
				(PE346 ((_ extract 346 346) P))
				(PE347 ((_ extract 347 347) P))
				(PE348 ((_ extract 348 348) P))
				(PE349 ((_ extract 349 349) P))
				(PE350 ((_ extract 350 350) P))
				(PE351 ((_ extract 351 351) P))
				(PE352 ((_ extract 352 352) P))
				(PE353 ((_ extract 353 353) P))
				(PE354 ((_ extract 354 354) P))
				(PE355 ((_ extract 355 355) P))
				(PE356 ((_ extract 356 356) P))
				(PE357 ((_ extract 357 357) P))
				(PE358 ((_ extract 358 358) P))
				(PE359 ((_ extract 359 359) P))
				(PE360 ((_ extract 360 360) P))
				(PE361 ((_ extract 361 361) P))
				(PE362 ((_ extract 362 362) P))
				(PE363 ((_ extract 363 363) P))
				(PE364 ((_ extract 364 364) P))
				(PE365 ((_ extract 365 365) P))
			)
		(let
			(
				(P0 (concat PE365 PE364 PE363 PE362 PE361 PE360 PE359 PE358 PE357 PE356 PE355 PE354 PE353 PE352 PE351 PE350 PE349 PE348 PE347 PE346 PE345 PE344 PE343 PE342 PE341 PE340 PE339 PE338 PE337 PE336 PE335 PE334 PE333 PE332 PE331 PE330 PE329 PE328 PE327 PE326 PE325 PE324 PE323 PE322 PE321 PE320 PE319 PE318 PE317 PE316 PE315 PE314 PE313 PE312 PE311 PE310 PE309 PE308 PE307 PE306 PE305 PE304 PE303 PE302 PE301 PE300 PE299 PE298 PE297 PE296 PE295 PE294 PE293 PE292 PE291 PE290 PE289 PE288 PE287 PE286 PE285 PE284 PE283 PE282 PE281 PE280 PE279 PE278 PE277 PE276 PE275 PE274 PE273 PE272 PE271 PE270 PE269 PE268 PE267 PE266 PE265 PE264 PE263 PE262 PE261 PE260 PE259 PE258 PE257 PE256 PE255 PE254 PE253 PE252 PE251 PE250 PE249 PE248 PE247 PE246 PE245 PE244 PE243 PE242 PE241 PE240 PE239 PE238 PE237 PE236 PE235 PE234 PE233 PE232 PE231 PE230 PE229 PE228 PE227 PE226 PE225 PE224 PE223 PE222 PE221 PE220 PE219 PE218 PE217 PE216 PE215 PE214 PE213 PE212 PE211 PE210 PE209 PE208 PE207 PE206 PE205 PE204 PE203 PE202 PE201 PE200 PE199 PE198 PE197 PE196 PE195 PE194 PE193 PE192 PE191 PE190 PE189 PE188 PE187 PE186 PE185 PE184 PE183 PE182 PE181 PE180 PE179 PE178 PE177 PE176 PE175 PE174 PE173 PE172 PE171 PE170 PE169 PE168 PE167 PE166 PE165 PE164 PE163 PE162 PE161 PE160 PE159 PE158 PE157 PE156 PE155 PE154 PE153 PE152 PE151 PE150 PE149 PE148 PE147 PE146 PE145 PE144 PE143 PE142 PE141 PE140 PE139 PE138 PE137 PE136 PE135 PE134 PE133 PE132 PE131 PE130 PE129 PE128 PE127 PE126 PE125 PE124 PE123 PE122 PE121 PE120 PE119 PE118 PE117 PE116 PE115 PE114 PE113 PE112 PE111 PE110 PE109 PE108 PE107 PE106 PE105 PE104 PE103 PE102 PE101 PE100 PE99 PE98 PE97 PE96 PE95 PE94 PE93 PE92 PE91 PE90 PE89 PE88 PE87 PE86 PE85 PE84 PE83 PE82 PE81 PE80 PE79 PE78 PE77 PE76 PE75 PE74 PE73 PE72 PE71 PE70 PE69 PE68 PE67 PE66 PE65 PE64 PE63 PE62 PE61 PE60 PE59 PE58 PE57 PE56 PE55 PE54 PE53 PE52 PE51 PE50 PE49 PE48 PE47 PE46 PE45 PE44 PE43 PE42 PE41 PE40 PE39 PE38 PE37 PE36 PE35 PE34 PE33 PE32 PE31 PE30 PE29 PE28 PE27 PE26 PE25 PE24 PE23 PE22 PE21 PE20 PE19 PE18 PE17 PE16 PE15 PE14 PE13 PE12 PE11 PE10 PE9 PE8 PE7 PE6 PE5 PE4 PE3 PE2 PE1 PE0 bit_0))
				(P1_1 (concat PE365 PE364 PE363 PE362 PE361 PE360 PE359 PE358 PE357 PE356 PE355 PE354 PE353 PE352 PE351 PE350 PE349 PE348 PE347 PE346 PE345 PE344 PE343 PE342 PE341 PE340 PE339 PE338 PE337 PE336 PE335 PE334 PE333 PE332 PE331 PE330 PE329 PE328 PE327 PE326 PE325 PE324 PE323 PE322 PE321 PE320 PE319 PE318 PE317 PE316 PE315 PE314 PE313 PE312 PE311 PE310 PE309 PE308 PE307 PE306 PE305 PE304 PE303 PE302 PE301 PE300 PE299 PE298 PE297 PE296 PE295 PE294 PE293 PE292 PE291 PE290 PE289 PE288 PE287 PE286 PE285 PE284 PE283 PE282 PE281 PE280 PE279 PE278 PE277 PE276 PE275 PE274 PE273 PE272 PE271 PE270 PE269 PE268 PE267 PE266 PE265 PE264 PE263 PE262 PE261 PE260 PE259 PE258 PE257 PE256 PE255 PE254 PE253 PE252 PE251 PE250 PE249 PE248 PE247 PE246 PE245 PE244 PE243 PE242 PE241 PE240 PE239 PE238 PE237 PE236 PE235 PE234 PE233 PE232 PE231 PE230 PE229 PE228 PE227 PE226 PE225 PE224 PE223 PE222 PE221 PE220 PE219 PE218 PE217 PE216 PE215 PE214 PE213 PE212 PE211 PE210 PE209 PE208 PE207 PE206 PE205 PE204 PE203 PE202 PE201 PE200 PE199 PE198 PE197 PE196 PE195 PE194 PE193 PE192 PE191 PE190 PE189 PE188 PE187 PE186 PE185 PE184 PE183 PE182 PE181 PE180 PE179 PE178 PE177 PE176 PE175 PE174 PE173 PE172 PE171 PE170 PE169 PE168 PE167 PE166 PE165 PE164 PE163 PE162 PE161 PE160 PE159 PE158 PE157 PE156 PE155 PE154 PE153 PE152 PE151 PE150 PE149 PE148 PE147 PE146 PE145 PE144 PE143 PE142 PE141 PE140 PE139 PE138 PE137 PE136 PE135 PE134 PE133 PE132 PE131 PE130 PE129 PE128 PE127 PE126 PE125 PE124 PE123 PE122 PE121 PE120 PE119 PE118 PE117 PE116 PE115 PE114 PE113 PE112 PE111 PE110 PE109 PE108 PE107 PE106 PE105 PE104 PE103 PE102 PE101 PE100 PE99 PE98 PE97 PE96 PE95 PE94 PE93 PE92 PE91 PE90 PE89 PE88 PE87 PE86 PE85 PE84 PE83 PE82 PE81 PE80 PE79 PE78 PE77 PE76 PE75 PE74 PE73 PE72 PE71 PE70 PE69 PE68 PE67 PE66 PE65 PE64 PE63 PE62 PE61 PE60 PE59 PE58 PE57 PE56 PE55 PE54 PE53 PE52 PE51 PE50 PE49 PE48 PE47 PE46 PE45 PE44 PE43 PE42 PE41 PE40 PE39 PE38 PE37 PE36 PE35 PE34 PE33 PE32 PE31 PE30 PE29 PE28 PE27 PE26 PE25 PE24 PE23 PE22 PE21 PE20 PE19 PE18 PE17 PE16 PE15 PE14 PE13 PE12 PE11 PE10 PE9 PE8 PE7 PE6 PE5 PE4 PE3 PE2 PE1 PE0 bit_1))				
			)
;----------------P to Q pipeline step------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q (pipeline P0 datamemory regfile noflush))
			)
		(let
			(
				(Q_r (regfilewriting_pipeline Q regfile))
				(Q_d (datamemorywriting_pipeline Q datamemory))
			)
	
;----------------Q to Qf flush 1-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q1 (pipeline Q Q_d Q_r flush))
			)
		(let
			(
				(Q1_r (regfilewriting_pipeline Q1 Q_r))
				(Q1_d (datamemorywriting_pipeline Q1 Q_d))
			)
		
;----------------Q to Qf flush 2-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q2 (pipeline Q1 Q1_d Q1_r flush))
			)
		(let
			(
				(Q2_r (regfilewriting_pipeline Q2 Q1_r))
				(Q2_d (datamemorywriting_pipeline Q2 Q1_d))
			)

;----------------Q to Qf flush 3-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q3 (pipeline Q2 Q2_d Q2_r flush))
			)
		(let
			(
				(Q3_r (regfilewriting_pipeline Q3 Q2_r))
				(Q3_d (datamemorywriting_pipeline Q3 Q2_d))
			)
			
;----------------Q to Qf flush 4-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q4 (pipeline Q3 Q3_d Q3_r flush))
			)
		(let
			(
				(Q4_r (regfilewriting_pipeline Q4 Q3_r))
				(Q4_d (datamemorywriting_pipeline Q4 Q3_d))
			)
			
;----------------Q to Qf flush 5-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q5 (pipeline Q4 Q4_d Q4_r flush))
			)
		(let
			(
				(Q5_r (regfilewriting_pipeline Q5 Q4_r))
				(Q5_d (datamemorywriting_pipeline Q5 Q4_d))
			)
			
;----------------Q to Qf flush 6-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q6 (pipeline Q5 Q5_d Q5_r flush))
			)
		(let
			(
				(Q6_r (regfilewriting_pipeline Q6 Q5_r))
				(Q6_d (datamemorywriting_pipeline Q6 Q5_d))
			)
			
;--------------Q to Qf flush 7----------------
         (let
			(
				(Qf (pipeline Q6 Q6_d Q6_r flush))
			)
		(let
			(
				(Qf_r (regfilewriting_pipeline Qf Q6_r))
				(Qf_d (datamemorywriting_pipeline Qf Q6_d))
			)
			
;-----------------------------------------------------
						

;----------------P to I flush 1------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P1 (pipeline P1_1 datamemory regfile noflush))
			)
		(let
			(
				(P1_r (regfilewriting_pipeline P1 regfile))
				(P1_d (datamemorywriting_pipeline P1 datamemory))
			)
;----------------P to I flush 2------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P2 (pipeline P1 P1_d P1_r flush))
			)
		(let
			(
				(P2_r (regfilewriting_pipeline P2 P1_r))
				(P2_d (datamemorywriting_pipeline P2 P1_d))
			)

;----------------P to I flush 3------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P3 (pipeline P2 P2_d P2_r flush))
			)
		(let
			(
				(P3_r (regfilewriting_pipeline P3 P2_r))
				(P3_d (datamemorywriting_pipeline P3 P2_d))
			)
;----------------P to I flush 4------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P4 (pipeline P3 P3_d P3_r flush))
			)
		(let
			(
				(P4_r (regfilewriting_pipeline P4 P3_r))
				(P4_d (datamemorywriting_pipeline P4 P3_d))
			)
;----------------P to I flush 5------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P5 (pipeline P4 P4_d P4_r flush))
			)
		(let
			(
				(P5_r (regfilewriting_pipeline P5 P4_r))
				(P5_d (datamemorywriting_pipeline P5 P4_d))
			)
;----------------P to I flush 6------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P6 (pipeline P5 P5_d P5_r flush))
			)
		(let
			(
				(P6_r (regfilewriting_pipeline P6 P5_r))
				(P6_d (datamemorywriting_pipeline P6 P5_d))
			)

;----------------P to I flush 7------------------------------------------------------------------------------------------------------------------
		(let
			(
				(I (pipeline P6 P6_d P6_r flush))
			)
		(let
			(
				(I_r (regfilewriting_pipeline I P6_r))
				(I_d (datamemorywriting_pipeline I P6_d))
			)
			
;----------------------------------------------------------------------------------------------------------------------------
;------------------property------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------			

		(let
			(	
				(Qf_pccounter ((_ extract 350 335) Qf))
				(I_pccounter ((_ extract 350 335) I))
			)	
		(=>
			(and (= flush (_ bv1 1)) (= noflush (_ bv0 1)) (= bit_0 (_ bv0 1)) (= bit_1 (_ bv1 1)))
			
			(and (= Qf_d I_d) 
				 (= I_pccounter Qf_pccounter)
				 (= Qf_r I_r)) 		
			)
		)))))))))))))))))))))))))))))))))
	)
)

(check-sat)
(get-model)
