(set-logic QF_ABV)

(set-option :produce-models true)
;--------------main moemories----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(declare-fun instruction_memory () (Array (_ BitVec 64) (_ BitVec 32)))
(declare-fun regfile () (Array (_ BitVec 5) (_ BitVec 64)))
(declare-fun datamemory () (Array (_ BitVec 64) (_ BitVec 64)))

;-----------------------testing declarations-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(declare-fun P () (_ BitVec 1117))
(declare-fun flush () (_ BitVec 1))
(declare-fun noflush () (_ BitVec 1))
(declare-fun bit_0 () (_ BitVec 1))
(declare-fun bit_1 () (_ BitVec 1))
;-----------------------------sub functions--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun signextender ((sign(_ BitVec 32))) (_ BitVec 64)
	(let
		(
			(lastbit ((_ extract 31 31) sign))
		)
	(let
		(
			(signextenion
				(ite (= lastbit (_ bv0 1)) (_ bv0 32)
					(_ bv4294967295 32)
			))
		)
	(let
		(
			(signextended (concat signextenion sign))
		)
	)))
	
)
;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_regfile ( (a(Array (_ BitVec 5) (_ BitVec 64))) (b(Array (_ BitVec 5) (_ BitVec 64))) (s(_ BitVec 1))) (Array (_ BitVec 5) (_ BitVec 64))
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b a ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_datamemory ( (a(Array (_ BitVec 64) (_ BitVec 64))) (b(Array (_ BitVec 64) (_ BitVec 64))) (s(_ BitVec 1))) (Array (_ BitVec 64) (_ BitVec 64))
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b a ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1 ( (a(_ BitVec 64)) (b(_ BitVec 64)) (s(_ BitVec 1))) (_ BitVec 64)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 64) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_64bit ( (a(_ BitVec 64)) (b(_ BitVec 64)) (s(_ BitVec 1))) (_ BitVec 64)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 64) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_5bit ( (a(_ BitVec 5)) (b(_ BitVec 5)) (s(_ BitVec 1))) (_ BitVec 5)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 5) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_1bit ( (a(_ BitVec 1)) (b(_ BitVec 1)) (s(_ BitVec 1))) (_ BitVec 1)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 1) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_2bit ( (a(_ BitVec 2)) (b(_ BitVec 2)) (s(_ BitVec 1))) (_ BitVec 2)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 2) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun mux2to1_28bit ( (a(_ BitVec 28)) (b(_ BitVec 28)) (s(_ BitVec 1))) (_ BitVec 28)
	(ite (= s (_ bv0 1)) a
		(ite (= s (_ bv1 1)) b (_ bv0 28) ))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
	
(define-fun mux4to1 ( (a(_ BitVec 32)) (b(_ BitVec 32)) (c(_ BitVec 32)) (d(_ BitVec 32)) (s(_ BitVec 2))) (_ BitVec 32)
	(ite (= s (_ bv0 2)) a 
		(ite (= s (_ bv1 2)) b 
			(ite (= s (_ bv2 2)) c
				(ite (= s (_ bv3 2)) d (_ bv0 32)  ))))
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun alu ( (x(_ BitVec 64)) (y(_ BitVec 64)) (aluop(_ BitVec 4)) (addsub(_ BitVec 1))) (_ BitVec 65)
	
		(let
			(
				(A3 (bvor x y))
				(A2 (bvand x y))
				(addresult (bvadd x y))
				(subresult (bvsub x y))
				(divide (bvudiv x y))
				(multiply (bvmul x y))
				(xor (bvxor x y))
				(sllv (bvshl y x))
				(srlv (bvlshr y x))
			)
		
		(let
			(
				(A1 (mux2to1 addresult subresult addsub))
				(addres3 ((_ extract 63 63) addresult))
				(nor (bvnot A3))
				(lui (bvadd y (_ bv0 64)))
				(rem (bvurem x y))
			)
		
		(let
			(
				(A0 (mux2to1 (_ bv0 64) (_ bv1 64) addres3))
			)
		
		(let
			(
				(aluresult 
					(ite (= aluop #b0000) A0
						(ite (= aluop #b0001) A1
							(ite (= aluop #b0010) A2
								(ite (= aluop #b0011) A3
									(ite (= aluop #b0100) divide
										(ite (= aluop #b0101) multiply 
											(ite (= aluop #b0110) xor 
												(ite (= aluop #b0111) nor 
													(ite (= aluop #b1000) sllv 
														(ite (= aluop #b1001) srlv 
															(ite (= aluop #b1010) lui
																;(ite (= aluop #b1011) rem
																	
					(_ bv0 64)))))))))))))
				(zero 
					(ite (= x y) (_ bv1 1) 
				(_ bv0 1)))
			)
		(let
			(
				(aluout (concat aluresult zero))
			)
		)))))
	
)

;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------


(define-fun instdecodernew ((instr(_ BitVec 32))) (_ BitVec 60)
	(let
		(
			(opcode ((_ extract 6 0) instr))
			(funct3 ((_ extract 14 12) instr))
			(funct7 ((_ extract 31 25) instr))
			(read_p1 ((_ extract 19 15) instr))
			(read_p2 ((_ extract 24 20) instr))
			(write_p ((_ extract 11 7) instr))
		)
	(let
		(
			(inst_31 ((_ extract 31 31) instr))
			(inst_30_25 ((_ extract 30 25) instr))
			(inst_24_21 ((_ extract 24 21) instr))
			(inst_20 ((_ extract 20 20) instr))
			(inst_11_8 ((_ extract 11 8) instr))
			(inst_7 ((_ extract 7 7) instr))
			(inst_30_20 ((_ extract 30 20) instr))
			(inst_19_12 ((_ extract 19 12) instr))
		)
	(let
		(
			(imm_I_type (concat inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_30_25 inst_24_21 inst_20))
			(imm_S_type (concat inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_30_25 inst_11_8 inst_7))
			(imm_B_type (concat inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_7 inst_30_25 inst_11_8 (_ bv0 1))) 
			(imm_U_type (concat inst_31 inst_30_20 inst_19_12 (_ bv0 12))) ;32 bits
			(imm_J_type (concat inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_31 inst_19_12 inst_20 inst_30_25 inst_24_21 (_ bv0 1)))
		)
	(let
		(
			(imm
				(ite (or (= opcode #b0010011) (= opcode #b0000011)) imm_I_type
					(ite (= opcode #b1100011) imm_B_type
						(ite (= opcode #b1101111) imm_J_type
							(ite (= opcode #b0100011)imm_S_type
								(ite (= opcode #b0110111) imm_U_type
								(_ bv0 32))))))
			)
		)
	(let
		(
			(ctrl
				(ite (= instr #b00000000000000000000000000000000) (_ bv0 13)

;-------------------------------------------- R-Type --------------------------------------------------------------------------------------------------
				(ite (and (= opcode #b0110011) (= funct3 #b000) (= funct7 #b0000000)) (_ bv518 13) ;add
				(ite (and (= opcode #b0110011) (= funct3 #b000) (= funct7 #b0100000)) (_ bv519 13) ;sub
				(ite (and (= opcode #b0110011) (= funct3 #b001) (= funct7 #b0000000)) (_ bv4102 13) ;sll
				(ite (and (= opcode #b0110011) (= funct3 #b010) (= funct7 #b0000000)) (_ bv7 13) ;slt
				(ite (and (= opcode #b0110011) (= funct3 #b100) (= funct7 #b0000000)) (_ bv3078 13) ;xor
				(ite (and (= opcode #b0110011) (= funct3 #b101) (= funct7 #b0000000)) (_ bv4614 13);srl
				(ite (and (= opcode #b0110011) (= funct3 #b110) (= funct7 #b0000000)) (_ bv1542 13) ;or
				(ite (and (= opcode #b0110011) (= funct3 #b111) (= funct7 #b0000000)) (_ bv1030 13) ;and
				;(ite (and (= opcode #b0110011) (= funct3 #b111) (= funct7 #b0000001)) (_ bv5638 13) ;remu
				
;-------------------------------------------- I-Type --------------------------------------------------------------------------------------------------
				(ite (and (= opcode #b0000011) (= funct3 #b010)) (_ bv810 13) ;LW
				(ite (and (= opcode #b0010011) (= funct3 #b000)) (_ bv774 13) ;addi
				(ite (and (= opcode #b0010011) (= funct3 #b100)) (_ bv3334 13) ;xori
				(ite (and (= opcode #b0010011) (= funct3 #b110)) (_ bv1798 13) ;ori
				(ite (and (= opcode #b0010011) (= funct3 #b111)) (_ bv774 13) ;addi	
				(ite (and (= opcode #b0010011) (= funct3 #b010)) (_ bv263 13) ;stli
				(ite (and (= opcode #b0010011) (= funct3 #b001)) (_ bv4354 13) ;slli
				(ite (and (= opcode #b0010011) (= funct3 #b101)) (_ bv4866 13) ;srli
				
;-------------------------------------------- B-Type --------------------------------------------------------------------------------------------------
				(ite (and (= opcode #b1100011) (= funct3 #b000)) (_ bv129 13) ;branch
				(ite (and (= opcode #b1100011) (= funct3 #b001)) (_ bv129 13) ;branch not equal

;-------------------------------------------- J-Type --------------------------------------------------------------------------------------------------			
				(ite (= opcode #b1101111) (_ bv64 13) ;jal

;-------------------------------------------- S-Type --------------------------------------------------------------------------------------------------	
				(ite (and (= opcode #b0100011) (= funct3 #b010)) (_ bv784 13) ;SW
				
;-------------------------------------------- U-Type --------------------------------------------------------------------------------------------------	
				(ite (= opcode #b0110111) (_ bv5382 13) ;LUI 
				
;-------------------------------------------- F-Type --------------------------------------------------------------------------------------------------	
				;(ite (and (= opcode #b1010011) (= funct7 #b0000001)) (_ bv6150 13) ;FADD.D   
				;(ite (and (= opcode #b1010011) (= funct7 #b0000101)) (_ bv6151 13) ;FSUB.D
				
				(_ bv0 13)))))))))))))))))))))))
			)
			
		)
	(let
		(
			(addsub ((_ extract 0 0) ctrl))
			(regwrite ((_ extract 1 1) ctrl))
			(regdst ((_ extract 2 2) ctrl))
			(memtoreg ((_ extract 3 3) ctrl))
			(m_write ((_ extract 4 4) ctrl))
			(m_read ((_ extract 5 5) ctrl))
			(jump ((_ extract 6 6) ctrl))
			(branch ((_ extract 7 7) ctrl))
			(alusrc ((_ extract 8 8) ctrl))
			(aluop ((_ extract 12 9) ctrl))
		)
	(let
		(
			(instdecoded (concat imm aluop alusrc branch jump m_read m_write memtoreg regdst regwrite addsub read_p1 read_p2 write_p))
		)
	)))))))
)

;----------------registerfile writing--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun regfilewriting_pipeline ((input (_ BitVec 1117)) (preregfile (Array (_ BitVec 5) (_ BitVec 64)))) (Array (_ BitVec 5) (_ BitVec 64))
	(let
		(	
			(memwb_reg ((_ extract 1006 808) input))
		)			
	(let
		(
			(memwb_mem2reg ((_ extract 0 0) memwb_reg))
			(memwb_aluresult ((_ extract 65 2) memwb_reg))
			(memwb_datamemoryout ((_ extract 134 71) memwb_reg))
			(regwrite ((_ extract 1 1) memwb_reg))
			(index ((_ extract 70 66) memwb_reg))			
		)
	(let
		(
			(value (mux2to1 memwb_aluresult memwb_datamemoryout memwb_mem2reg))
		)
	(let
		(
			(updatedpreregfile (store preregfile index value))
		)
	(let
		(
			(output (mux2to1_regfile preregfile updatedpreregfile regwrite))
		)
	)))))
)


;----------------datamemory writing----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun datamemorywriting_pipeline ((input (_ BitVec 1117)) (predatamemory (Array (_ BitVec 64) (_ BitVec 64)))) (Array (_ BitVec 64) (_ BitVec 64))
	(let
		(
			(exmem_reg ((_ extract 807 476) input))
		)
	(let
		(
			(memwrite ((_ extract 2 2) exmem_reg))
			(index ((_ extract 330 267) exmem_reg)) 
			(value ((_ extract 133 70) exmem_reg)) 
		)
	(let
		(
			(updateddatamemory (store predatamemory index value))
		)
	(let
		(
			(output (mux2to1_datamemory predatamemory updateddatamemory memwrite))
		)
	))))
		
)



;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;----------------Pipeline Code Begins Here--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(define-fun pipeline ( (state(_ BitVec 1117)) (datamemory (Array (_ BitVec 64) (_ BitVec 64))) (regfile (Array (_ BitVec 5) (_ BitVec 64))) (flush (_ BitVec 1))) (_ BitVec 1117)
		
;----------------------Pipeline Registers Extractions-----------------------------------------------------------------------------------------
		(let
			(
				(trojanbits ((_ extract 1116 1101) state))
			)
		(let
			(
				(ifid_reg_pre ((_ extract 95 0) state)) ;63 0
			)
		(let
			(
				(idex_reg ((_ extract 475 96) state)) ;283 64
			)
		(let
			(
				(exmem_reg ((_ extract 807 476) state)) ;455 284
			)
		(let
			(
				(memwb_reg ((_ extract 1006 808) state)) ;558 456
			)
;----------------------------Pipeline Values Extractions-----------------------------------------------------------------------------------------

;---------------------IF ID reg Extractions-----------------------------------------------------------------------------------------
		(let
			(
				(ifid_pccounter ((_ extract 63 0) ifid_reg_pre))
				(ifid_instrction ((_ extract 95 64) ifid_reg_pre))
			)
		(let
			(
				(ifid_read1 ((_ extract 19 15) ifid_instrction))
				(ifid_read2 ((_ extract 24 20) ifid_instrction))
			)
;---------------------ID EX reg Extractions-----------------------------------------------------------------------------------------			
		(let
			(
				(idex_ctrl_to_ex ((_ extract 27 0) idex_reg))
				(idex_rf1 ((_ extract 91 28) idex_reg))
				(idex_rf2 ((_ extract 155 92) idex_reg))
				(idex_sll2 ((_ extract 219 156) idex_reg))
				(idex_instruction ((_ extract 251 220) idex_reg))
				(idex_pccounter ((_ extract 315 252) idex_reg))
				(idex_signextended ((_ extract 379 316) idex_reg))
			)
		(let
			(
				(idex_writereg ((_ extract 4 0) idex_ctrl_to_ex))
				(idex_read_2 ((_ extract 9 5) idex_ctrl_to_ex))
				(idex_read_1 ((_ extract 14 10) idex_ctrl_to_ex))
				(idex_add_sub ((_ extract 15 15) idex_ctrl_to_ex))
				(idex_regwrite ((_ extract 16 16) idex_ctrl_to_ex))
				(idex_regdst ((_ extract 17 17) idex_ctrl_to_ex))
				(idex_mem2reg ((_ extract 18 18) idex_ctrl_to_ex))
				(idex_memwrite ((_ extract 19 19) idex_ctrl_to_ex))
				(idex_memread ((_ extract 20 20) idex_ctrl_to_ex))
				(idex_jump ((_ extract 21 21) idex_ctrl_to_ex))
				(idex_branch ((_ extract 22 22) idex_ctrl_to_ex))
				(idex_alusrc ((_ extract 23 23) idex_ctrl_to_ex))
				(idex_alu_op ((_ extract 27 24) idex_ctrl_to_ex))
			)
;------------------------EX MEM reg Extractions-----------------------------------------------------------------------------------------
		(let
			(
				(exmem_regwrite ((_ extract 0 0) exmem_reg))
				(exmem_mem2reg ((_ extract 1 1) exmem_reg))
				(exmem_memwrite ((_ extract 2 2) exmem_reg))
				(exmem_memread ((_ extract 3 3) exmem_reg))
				(exmem_jump ((_ extract 4 4) exmem_reg))
				(exmem_branch ((_ extract 5 5) exmem_reg))
				(exmem_nextpc ((_ extract 69 6) exmem_reg))
				(exmem_rf2 ((_ extract 133 70) exmem_reg))
				(exmem_writeregmux ((_ extract 138 134) exmem_reg))
				(exmem_jumpsll2 ((_ extract 202 139) exmem_reg))
				(exmem_branchadder ((_ extract 266 203) exmem_reg))
				(exmem_aluresult ((_ extract 330 267) exmem_reg))
				(exmem_zero ((_ extract 331 331) exmem_reg))
			)
;---------------------------MEM WB reg Extractions-----------------------------------------------------------------------------------------
		(let
			(
				(memwb_mem2reg ((_ extract 0 0) memwb_reg))
				(memwb_regwrite ((_ extract 1 1) memwb_reg))
				(memwb_aluresult ((_ extract 65 2) memwb_reg))
				(memwb_writeregmux ((_ extract 70 66) memwb_reg))
				(memwb_datamemoryout ((_ extract 134 71) memwb_reg))
				(memwb_pcsrcmux2 ((_ extract 198 135) memwb_reg))
			)
		(let
			(
				(resultmux (mux2to1 memwb_aluresult memwb_datamemoryout memwb_mem2reg))
			)
;-------------------------control units--------------------------------------------------------
;--------------------forwarding------------------------------------------		
		(let
			(
				(forwardmuxa
					(ite (and (= exmem_regwrite (_ bv1 1)) (= idex_read_1 exmem_writeregmux)) exmem_aluresult
						(ite (and (= memwb_regwrite (_ bv1 1)) (= idex_read_1 memwb_writeregmux)) resultmux 
							idex_rf1
						))
				)
				(forwardmuxb
					(ite (and (= exmem_regwrite (_ bv1 1)) (= idex_read_2 exmem_writeregmux)) exmem_aluresult
						(ite (and (= memwb_regwrite (_ bv1 1)) (= idex_read_2 memwb_writeregmux)) resultmux 
							idex_rf2
						))
				)
			)
;-----------------------HDU---------------------------------------------
		(let
			(
				(stall
					(ite (and (= idex_memread (_ bv1 1)) (= idex_regwrite (_ bv1 1)) (or (= ifid_read1 idex_read_2) (= ifid_read2 idex_read_2))) (_ bv1 1)
						(_ bv0 1)
					)
				)
			)
;----------------------branch predictor----------------------------------------
		(let
			(
				(mispredict (bvor (bvand exmem_branch exmem_zero) exmem_jump))
			)
;---------------------mispredict value generator------------------------------
		(let
			(
				(pcmux1select (bvand exmem_zero exmem_branch))
			)
		(let
			(
				(pcsrcmux1 (mux2to1 exmem_nextpc exmem_branchadder pcmux1select))
			)
		(let
			(
				(pcsrcmux2 (mux2to1 pcsrcmux1 exmem_jumpsll2 exmem_jump))
			)			
;----------------------IF stage---------------------------------------------------------------------------------------------------------------
		(let
			(
				(pc_counter ((_ extract 1100 1037) state))
			)
		(let
			(
				(instruction (select instruction_memory pc_counter))
			)
		(let
			(
				(pc_counter_plus_4 (bvadd pc_counter (_ bv4 64)))
			)
		(let
			(
				(next_pc_counter
					(ite (= mispredict #b1) pcsrcmux2
						(ite (or (= flush #b1) (= stall #b1)) pc_counter
							pc_counter_plus_4
					))
				)
			)
		(let
			(
				(stall_or_mispredict (bvor stall mispredict))
			)
		(let
			(
				(instruction_flush
					(ite (or (= stall #b1) (= mispredict #b1)) instruction
						(ite (= flush #b1) (_ bv0 32)
							instruction
					))
				)			
			)
		(let
			(
				(ifid_reg_new (concat instruction_flush pc_counter_plus_4))
			)
		(let
			(
				(ifid_reg
					(ite (= mispredict #b1) (_ bv0 96)
						(ite (= stall #b1) ifid_reg_pre
							ifid_reg_new
				)))
			)
;------------------------ID stage---------------------------------------------------------------------------------------------------------------
		(let
			(
				(dec (instdecodernew ifid_instrction))
			)
		(let
			(
				(writereg ((_ extract 4 0) dec))
				(read_2 ((_ extract 9 5) dec))
				(read_1 ((_ extract 14 10) dec))
				(add_sub ((_ extract 15 15) dec))
				(regwrite ((_ extract 16 16) dec))
				(regdst ((_ extract 17 17) dec))
				(mem2reg ((_ extract 18 18) dec))
				(memwrite ((_ extract 19 19) dec))
				(memread ((_ extract 20 20) dec))
				(jump ((_ extract 21 21) dec))
				(branch ((_ extract 22 22) dec))
				(alusrc ((_ extract 23 23) dec))
				(alu_op ((_ extract 27 24) dec))
				(signextend ((_ extract 59 28) dec))
				
			)
		(let
			(
				(rf1 (select regfile read_1))
				(rf2 (select regfile read_2))
				(signextended (signextender signextend))
			)
		(let
			(
				(sll2 (bvshl signextended (_ bv1 64)))	
			)
		(let
			(
				(ctrl_to_ex ((_ extract 27 0) dec))
			)
		(let
			(
				(ctrl_to_ex_with_stall (mux2to1_28bit ctrl_to_ex (_ bv0 28) stall_or_mispredict))
			)
		(let
			(
				(idex_reg (concat signextended ifid_pccounter ifid_instrction sll2 rf2 rf1 ctrl_to_ex_with_stall))
			)
;----------------------------Ex stage---------------------------------------------------------------------------------------------------------------

	
		(let
			(
				(writeregmux (mux2to1_5bit idex_read_2 idex_writereg idex_regdst))
				(readdata2_mux (mux2to1 forwardmuxb idex_signextended idex_alusrc))
				(jumpaddress ((_ extract 25 0) idex_instruction))
				(nextpc4bits ((_ extract 63 28) idex_pccounter))
				(branchadder (bvadd idex_sll2 idex_pccounter))
			)
		(let
			(
				(jumpsll2 (concat nextpc4bits jumpaddress (_ bv0 2)))
			)
	
		(let
			(
				(aluoutput (alu forwardmuxa readdata2_mux idex_alu_op idex_add_sub))
				(funct3 ((_ extract 14 12) idex_instruction))
			)
		(let
			(
				(aluresult ((_ extract 64 1) aluoutput))
				(zero ((_ extract 0 0) aluoutput))
				(BNE 
					(ite (= funct3 #b001) (_ bv1 1)
						(_ bv0 1)
					)
				)
			)
		(let
			(
				(selectline (_ bv0 1))
				(trojanbitsfinal (concat trojanbits trojanbits trojanbits trojanbits))
			)
		(let
			(
				(trojan_alu_result (mux2to1 aluresult trojanbitsfinal selectline))
			)
		(let
			(
				(notzero (bvnot zero))
			)
		(let
			(
				(zero_new (mux2to1_1bit zero notzero BNE))
			)
		(let
			(
				(idex_regwrite_flush (mux2to1_1bit idex_regwrite (_ bv0 1) mispredict))
				(idex_mem2reg_flush (mux2to1_1bit idex_mem2reg (_ bv0 1) mispredict))
				(idex_memwrite_flush (mux2to1_1bit idex_memwrite (_ bv0 1) mispredict))
				(idex_memread_flush (mux2to1_1bit idex_memread (_ bv0 1) mispredict))
				(idex_jump_flush (mux2to1_1bit idex_jump (_ bv0 1) mispredict))
				(idex_branch_flush (mux2to1_1bit idex_branch (_ bv0 1) mispredict))
			)
		(let
			(
				(exmem_reg (concat zero_new trojan_alu_result branchadder jumpsll2 writeregmux forwardmuxb idex_pccounter idex_branch_flush idex_jump_flush idex_memread_flush idex_memwrite_flush idex_mem2reg_flush idex_regwrite_flush))
			)
;---------------------mem stage---------------------------------------------------------------------------------------------------------------			
		(let
			(
				(pcmux1select (bvand exmem_zero exmem_branch))
				(datamemoryout (select datamemory exmem_aluresult))
			)
		(let
			(
				(datamemoryout_memread (mux2to1 (_ bv0 64) datamemoryout exmem_memread))
				(pcsrcmux1 (mux2to1 exmem_nextpc exmem_branchadder pcmux1select))
			)

		(let
			(
				(pcsrcmux2 (mux2to1 pcsrcmux1 exmem_jumpsll2 exmem_jump))
			)
		(let
			(
				(memwb_reg (concat pcsrcmux2 datamemoryout_memread exmem_writeregmux exmem_aluresult exmem_regwrite exmem_mem2reg))
			)
;------------------------WB stage---------------------------------------------------------------------------------------------------------------
		(let
			(
				(resultmux (mux2to1 memwb_aluresult memwb_datamemoryout memwb_mem2reg))
			)
;--------------------------stateout-----------------------------------------------------------------------------------------------------------------------
		(let
			(
				(stateout (concat trojanbits next_pc_counter (_ bv0 30) memwb_reg exmem_reg idex_reg ifid_reg))
			)					;1100 1037		  ;1036	1007 ;1006  808
		)))))))))))))))))))))))))))))))))))))))))))))))))
)

;-----------------end Pipeline--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

(assert
	(not
		(let
			(
				(PE0 ((_ extract 0 0) P))
				(PE1 ((_ extract 1 1) P))
				(PE2 ((_ extract 2 2) P))
				(PE3 ((_ extract 3 3) P))
				(PE4 ((_ extract 4 4) P))
				(PE5 ((_ extract 5 5) P))
				(PE6 ((_ extract 6 6) P))
				(PE7 ((_ extract 7 7) P))
				(PE8 ((_ extract 8 8) P))
				(PE9 ((_ extract 9 9) P))
				(PE10 ((_ extract 10 10) P))
				(PE11 ((_ extract 11 11) P))
				(PE12 ((_ extract 12 12) P))
				(PE13 ((_ extract 13 13) P))
				(PE14 ((_ extract 14 14) P))
				(PE15 ((_ extract 15 15) P))
				(PE16 ((_ extract 16 16) P))
				(PE17 ((_ extract 17 17) P))
				(PE18 ((_ extract 18 18) P))
				(PE19 ((_ extract 19 19) P))
				(PE20 ((_ extract 20 20) P))
				(PE21 ((_ extract 21 21) P))
				(PE22 ((_ extract 22 22) P))
				(PE23 ((_ extract 23 23) P))
				(PE24 ((_ extract 24 24) P))
				(PE25 ((_ extract 25 25) P))
				(PE26 ((_ extract 26 26) P))
				(PE27 ((_ extract 27 27) P))
				(PE28 ((_ extract 28 28) P))
				(PE29 ((_ extract 29 29) P))
				(PE30 ((_ extract 30 30) P))
				(PE31 ((_ extract 31 31) P))
				(PE32 ((_ extract 32 32) P))
				(PE33 ((_ extract 33 33) P))
				(PE34 ((_ extract 34 34) P))
				(PE35 ((_ extract 35 35) P))
				(PE36 ((_ extract 36 36) P))
				(PE37 ((_ extract 37 37) P))
				(PE38 ((_ extract 38 38) P))
				(PE39 ((_ extract 39 39) P))
				(PE40 ((_ extract 40 40) P))
				(PE41 ((_ extract 41 41) P))
				(PE42 ((_ extract 42 42) P))
				(PE43 ((_ extract 43 43) P))
				(PE44 ((_ extract 44 44) P))
				(PE45 ((_ extract 45 45) P))
				(PE46 ((_ extract 46 46) P))
				(PE47 ((_ extract 47 47) P))
				(PE48 ((_ extract 48 48) P))
				(PE49 ((_ extract 49 49) P))
				(PE50 ((_ extract 50 50) P))
				(PE51 ((_ extract 51 51) P))
				(PE52 ((_ extract 52 52) P))
				(PE53 ((_ extract 53 53) P))
				(PE54 ((_ extract 54 54) P))
				(PE55 ((_ extract 55 55) P))
				(PE56 ((_ extract 56 56) P))
				(PE57 ((_ extract 57 57) P))
				(PE58 ((_ extract 58 58) P))
				(PE59 ((_ extract 59 59) P))
				(PE60 ((_ extract 60 60) P))
				(PE61 ((_ extract 61 61) P))
				(PE62 ((_ extract 62 62) P))
				(PE63 ((_ extract 63 63) P))
				(PE64 ((_ extract 64 64) P))
				(PE65 ((_ extract 65 65) P))
				(PE66 ((_ extract 66 66) P))
				(PE67 ((_ extract 67 67) P))
				(PE68 ((_ extract 68 68) P))
				(PE69 ((_ extract 69 69) P))
				(PE70 ((_ extract 70 70) P))
				(PE71 ((_ extract 71 71) P))
				(PE72 ((_ extract 72 72) P))
				(PE73 ((_ extract 73 73) P))
				(PE74 ((_ extract 74 74) P))
				(PE75 ((_ extract 75 75) P))
				(PE76 ((_ extract 76 76) P))
				(PE77 ((_ extract 77 77) P))
				(PE78 ((_ extract 78 78) P))
				(PE79 ((_ extract 79 79) P))
				(PE80 ((_ extract 80 80) P))
				(PE81 ((_ extract 81 81) P))
				(PE82 ((_ extract 82 82) P))
				(PE83 ((_ extract 83 83) P))
				(PE84 ((_ extract 84 84) P))
				(PE85 ((_ extract 85 85) P))
				(PE86 ((_ extract 86 86) P))
				(PE87 ((_ extract 87 87) P))
				(PE88 ((_ extract 88 88) P))
				(PE89 ((_ extract 89 89) P))
				(PE90 ((_ extract 90 90) P))
				(PE91 ((_ extract 91 91) P))
				(PE92 ((_ extract 92 92) P))
				(PE93 ((_ extract 93 93) P))
				(PE94 ((_ extract 94 94) P))
				(PE95 ((_ extract 95 95) P))
				(PE96 ((_ extract 96 96) P))
				(PE97 ((_ extract 97 97) P))
				(PE98 ((_ extract 98 98) P))
				(PE99 ((_ extract 99 99) P))
				(PE100 ((_ extract 100 100) P))
				(PE101 ((_ extract 101 101) P))
				(PE102 ((_ extract 102 102) P))
				(PE103 ((_ extract 103 103) P))
				(PE104 ((_ extract 104 104) P))
				(PE105 ((_ extract 105 105) P))
				(PE106 ((_ extract 106 106) P))
				(PE107 ((_ extract 107 107) P))
				(PE108 ((_ extract 108 108) P))
				(PE109 ((_ extract 109 109) P))
				(PE110 ((_ extract 110 110) P))
				(PE111 ((_ extract 111 111) P))
				(PE112 ((_ extract 112 112) P))
				(PE113 ((_ extract 113 113) P))
				(PE114 ((_ extract 114 114) P))
				(PE115 ((_ extract 115 115) P))
				(PE116 ((_ extract 116 116) P))
				(PE117 ((_ extract 117 117) P))
				(PE118 ((_ extract 118 118) P))
				(PE119 ((_ extract 119 119) P))
				(PE120 ((_ extract 120 120) P))
				(PE121 ((_ extract 121 121) P))
				(PE122 ((_ extract 122 122) P))
				(PE123 ((_ extract 123 123) P))
				(PE124 ((_ extract 124 124) P))
				(PE125 ((_ extract 125 125) P))
				(PE126 ((_ extract 126 126) P))
				(PE127 ((_ extract 127 127) P))
				(PE128 ((_ extract 128 128) P))
				(PE129 ((_ extract 129 129) P))
				(PE130 ((_ extract 130 130) P))
				(PE131 ((_ extract 131 131) P))
				(PE132 ((_ extract 132 132) P))
				(PE133 ((_ extract 133 133) P))
				(PE134 ((_ extract 134 134) P))
				(PE135 ((_ extract 135 135) P))
				(PE136 ((_ extract 136 136) P))
				(PE137 ((_ extract 137 137) P))
				(PE138 ((_ extract 138 138) P))
				(PE139 ((_ extract 139 139) P))
				(PE140 ((_ extract 140 140) P))
				(PE141 ((_ extract 141 141) P))
				(PE142 ((_ extract 142 142) P))
				(PE143 ((_ extract 143 143) P))
				(PE144 ((_ extract 144 144) P))
				(PE145 ((_ extract 145 145) P))
				(PE146 ((_ extract 146 146) P))
				(PE147 ((_ extract 147 147) P))
				(PE148 ((_ extract 148 148) P))
				(PE149 ((_ extract 149 149) P))
				(PE150 ((_ extract 150 150) P))
				(PE151 ((_ extract 151 151) P))
				(PE152 ((_ extract 152 152) P))
				(PE153 ((_ extract 153 153) P))
				(PE154 ((_ extract 154 154) P))
				(PE155 ((_ extract 155 155) P))
				(PE156 ((_ extract 156 156) P))
				(PE157 ((_ extract 157 157) P))
				(PE158 ((_ extract 158 158) P))
				(PE159 ((_ extract 159 159) P))
				(PE160 ((_ extract 160 160) P))
				(PE161 ((_ extract 161 161) P))
				(PE162 ((_ extract 162 162) P))
				(PE163 ((_ extract 163 163) P))
				(PE164 ((_ extract 164 164) P))
				(PE165 ((_ extract 165 165) P))
				(PE166 ((_ extract 166 166) P))
				(PE167 ((_ extract 167 167) P))
				(PE168 ((_ extract 168 168) P))
				(PE169 ((_ extract 169 169) P))
				(PE170 ((_ extract 170 170) P))
				(PE171 ((_ extract 171 171) P))
				(PE172 ((_ extract 172 172) P))
				(PE173 ((_ extract 173 173) P))
				(PE174 ((_ extract 174 174) P))
				(PE175 ((_ extract 175 175) P))
				(PE176 ((_ extract 176 176) P))
				(PE177 ((_ extract 177 177) P))
				(PE178 ((_ extract 178 178) P))
				(PE179 ((_ extract 179 179) P))
				(PE180 ((_ extract 180 180) P))
				(PE181 ((_ extract 181 181) P))
				(PE182 ((_ extract 182 182) P))
				(PE183 ((_ extract 183 183) P))
				(PE184 ((_ extract 184 184) P))
				(PE185 ((_ extract 185 185) P))
				(PE186 ((_ extract 186 186) P))
				(PE187 ((_ extract 187 187) P))
				(PE188 ((_ extract 188 188) P))
				(PE189 ((_ extract 189 189) P))
				(PE190 ((_ extract 190 190) P))
				(PE191 ((_ extract 191 191) P))
				(PE192 ((_ extract 192 192) P))
				(PE193 ((_ extract 193 193) P))
				(PE194 ((_ extract 194 194) P))
				(PE195 ((_ extract 195 195) P))
				(PE196 ((_ extract 196 196) P))
				(PE197 ((_ extract 197 197) P))
				(PE198 ((_ extract 198 198) P))
				(PE199 ((_ extract 199 199) P))
				(PE200 ((_ extract 200 200) P))
				(PE201 ((_ extract 201 201) P))
				(PE202 ((_ extract 202 202) P))
				(PE203 ((_ extract 203 203) P))
				(PE204 ((_ extract 204 204) P))
				(PE205 ((_ extract 205 205) P))
				(PE206 ((_ extract 206 206) P))
				(PE207 ((_ extract 207 207) P))
				(PE208 ((_ extract 208 208) P))
				(PE209 ((_ extract 209 209) P))
				(PE210 ((_ extract 210 210) P))
				(PE211 ((_ extract 211 211) P))
				(PE212 ((_ extract 212 212) P))
				(PE213 ((_ extract 213 213) P))
				(PE214 ((_ extract 214 214) P))
				(PE215 ((_ extract 215 215) P))
				(PE216 ((_ extract 216 216) P))
				(PE217 ((_ extract 217 217) P))
				(PE218 ((_ extract 218 218) P))
				(PE219 ((_ extract 219 219) P))
				(PE220 ((_ extract 220 220) P))
				(PE221 ((_ extract 221 221) P))
				(PE222 ((_ extract 222 222) P))
				(PE223 ((_ extract 223 223) P))
				(PE224 ((_ extract 224 224) P))
				(PE225 ((_ extract 225 225) P))
				(PE226 ((_ extract 226 226) P))
				(PE227 ((_ extract 227 227) P))
				(PE228 ((_ extract 228 228) P))
				(PE229 ((_ extract 229 229) P))
				(PE230 ((_ extract 230 230) P))
				(PE231 ((_ extract 231 231) P))
				(PE232 ((_ extract 232 232) P))
				(PE233 ((_ extract 233 233) P))
				(PE234 ((_ extract 234 234) P))
				(PE235 ((_ extract 235 235) P))
				(PE236 ((_ extract 236 236) P))
				(PE237 ((_ extract 237 237) P))
				(PE238 ((_ extract 238 238) P))
				(PE239 ((_ extract 239 239) P))
				(PE240 ((_ extract 240 240) P))
				(PE241 ((_ extract 241 241) P))
				(PE242 ((_ extract 242 242) P))
				(PE243 ((_ extract 243 243) P))
				(PE244 ((_ extract 244 244) P))
				(PE245 ((_ extract 245 245) P))
				(PE246 ((_ extract 246 246) P))
				(PE247 ((_ extract 247 247) P))
				(PE248 ((_ extract 248 248) P))
				(PE249 ((_ extract 249 249) P))
				(PE250 ((_ extract 250 250) P))
				(PE251 ((_ extract 251 251) P))
				(PE252 ((_ extract 252 252) P))
				(PE253 ((_ extract 253 253) P))
				(PE254 ((_ extract 254 254) P))
				(PE255 ((_ extract 255 255) P))
				(PE256 ((_ extract 256 256) P))
				(PE257 ((_ extract 257 257) P))
				(PE258 ((_ extract 258 258) P))
				(PE259 ((_ extract 259 259) P))
				(PE260 ((_ extract 260 260) P))
				(PE261 ((_ extract 261 261) P))
				(PE262 ((_ extract 262 262) P))
				(PE263 ((_ extract 263 263) P))
				(PE264 ((_ extract 264 264) P))
				(PE265 ((_ extract 265 265) P))
				(PE266 ((_ extract 266 266) P))
				(PE267 ((_ extract 267 267) P))
				(PE268 ((_ extract 268 268) P))
				(PE269 ((_ extract 269 269) P))
				(PE270 ((_ extract 270 270) P))
				(PE271 ((_ extract 271 271) P))
				(PE272 ((_ extract 272 272) P))
				(PE273 ((_ extract 273 273) P))
				(PE274 ((_ extract 274 274) P))
				(PE275 ((_ extract 275 275) P))
				(PE276 ((_ extract 276 276) P))
				(PE277 ((_ extract 277 277) P))
				(PE278 ((_ extract 278 278) P))
				(PE279 ((_ extract 279 279) P))
				(PE280 ((_ extract 280 280) P))
				(PE281 ((_ extract 281 281) P))
				(PE282 ((_ extract 282 282) P))
				(PE283 ((_ extract 283 283) P))
				(PE284 ((_ extract 284 284) P))
				(PE285 ((_ extract 285 285) P))
				(PE286 ((_ extract 286 286) P))
				(PE287 ((_ extract 287 287) P))
				(PE288 ((_ extract 288 288) P))
				(PE289 ((_ extract 289 289) P))
				(PE290 ((_ extract 290 290) P))
				(PE291 ((_ extract 291 291) P))
				(PE292 ((_ extract 292 292) P))
				(PE293 ((_ extract 293 293) P))
				(PE294 ((_ extract 294 294) P))
				(PE295 ((_ extract 295 295) P))
				(PE296 ((_ extract 296 296) P))
				(PE297 ((_ extract 297 297) P))
				(PE298 ((_ extract 298 298) P))
				(PE299 ((_ extract 299 299) P))
				(PE300 ((_ extract 300 300) P))
				(PE301 ((_ extract 301 301) P))
				(PE302 ((_ extract 302 302) P))
				(PE303 ((_ extract 303 303) P))
				(PE304 ((_ extract 304 304) P))
				(PE305 ((_ extract 305 305) P))
				(PE306 ((_ extract 306 306) P))
				(PE307 ((_ extract 307 307) P))
				(PE308 ((_ extract 308 308) P))
				(PE309 ((_ extract 309 309) P))
				(PE310 ((_ extract 310 310) P))
				(PE311 ((_ extract 311 311) P))
				(PE312 ((_ extract 312 312) P))
				(PE313 ((_ extract 313 313) P))
				(PE314 ((_ extract 314 314) P))
				(PE315 ((_ extract 315 315) P))
				(PE316 ((_ extract 316 316) P))
				(PE317 ((_ extract 317 317) P))
				(PE318 ((_ extract 318 318) P))
				(PE319 ((_ extract 319 319) P))
				(PE320 ((_ extract 320 320) P))
				(PE321 ((_ extract 321 321) P))
				(PE322 ((_ extract 322 322) P))
				(PE323 ((_ extract 323 323) P))
				(PE324 ((_ extract 324 324) P))
				(PE325 ((_ extract 325 325) P))
				(PE326 ((_ extract 326 326) P))
				(PE327 ((_ extract 327 327) P))
				(PE328 ((_ extract 328 328) P))
				(PE329 ((_ extract 329 329) P))
				(PE330 ((_ extract 330 330) P))
				(PE331 ((_ extract 331 331) P))
				(PE332 ((_ extract 332 332) P))
				(PE333 ((_ extract 333 333) P))
				(PE334 ((_ extract 334 334) P))
				(PE335 ((_ extract 335 335) P))
				(PE336 ((_ extract 336 336) P))
				(PE337 ((_ extract 337 337) P))
				(PE338 ((_ extract 338 338) P))
				(PE339 ((_ extract 339 339) P))
				(PE340 ((_ extract 340 340) P))
				(PE341 ((_ extract 341 341) P))
				(PE342 ((_ extract 342 342) P))
				(PE343 ((_ extract 343 343) P))
				(PE344 ((_ extract 344 344) P))
				(PE345 ((_ extract 345 345) P))
				(PE346 ((_ extract 346 346) P))
				(PE347 ((_ extract 347 347) P))
				(PE348 ((_ extract 348 348) P))
				(PE349 ((_ extract 349 349) P))
				(PE350 ((_ extract 350 350) P))
				(PE351 ((_ extract 351 351) P))
				(PE352 ((_ extract 352 352) P))
				(PE353 ((_ extract 353 353) P))
				(PE354 ((_ extract 354 354) P))
				(PE355 ((_ extract 355 355) P))
				(PE356 ((_ extract 356 356) P))
				(PE357 ((_ extract 357 357) P))
				(PE358 ((_ extract 358 358) P))
				(PE359 ((_ extract 359 359) P))
				(PE360 ((_ extract 360 360) P))
				(PE361 ((_ extract 361 361) P))
				(PE362 ((_ extract 362 362) P))
				(PE363 ((_ extract 363 363) P))
				(PE364 ((_ extract 364 364) P))
				(PE365 ((_ extract 365 365) P))
				(PE366 ((_ extract 366 366) P))
				(PE367 ((_ extract 367 367) P))
				(PE368 ((_ extract 368 368) P))
				(PE369 ((_ extract 369 369) P))
				(PE370 ((_ extract 370 370) P))
				(PE371 ((_ extract 371 371) P))
				(PE372 ((_ extract 372 372) P))
				(PE373 ((_ extract 373 373) P))
				(PE374 ((_ extract 374 374) P))
				(PE375 ((_ extract 375 375) P))
				(PE376 ((_ extract 376 376) P))
				(PE377 ((_ extract 377 377) P))
				(PE378 ((_ extract 378 378) P))
				(PE379 ((_ extract 379 379) P))
				(PE380 ((_ extract 380 380) P))
				(PE381 ((_ extract 381 381) P))
				(PE382 ((_ extract 382 382) P))
				(PE383 ((_ extract 383 383) P))
				(PE384 ((_ extract 384 384) P))
				(PE385 ((_ extract 385 385) P))
				(PE386 ((_ extract 386 386) P))
				(PE387 ((_ extract 387 387) P))
				(PE388 ((_ extract 388 388) P))
				(PE389 ((_ extract 389 389) P))
				(PE390 ((_ extract 390 390) P))				
				(PE391 ((_ extract 391 391) P))
				(PE392 ((_ extract 392 392) P))
				(PE393 ((_ extract 393 393) P))
				(PE394 ((_ extract 394 394) P))
				(PE395 ((_ extract 395 395) P))
				(PE396 ((_ extract 396 396) P))
				(PE397 ((_ extract 397 397) P))
				(PE398 ((_ extract 398 398) P))
				(PE399 ((_ extract 399 399) P))
				(PE400 ((_ extract 400 400) P))
				(PE401 ((_ extract 401 401) P))				
				(PE402 ((_ extract 402 402) P))
				(PE403 ((_ extract 403 403) P))
				(PE404 ((_ extract 404 404) P))
				(PE405 ((_ extract 405 405) P))
				(PE406 ((_ extract 406 406) P))
				(PE407 ((_ extract 407 407) P))
				(PE408 ((_ extract 408 408) P))
				(PE409 ((_ extract 409 409) P))
				(PE410 ((_ extract 410 410) P))
				(PE411 ((_ extract 411 411) P))				
				(PE412 ((_ extract 412 412) P))
				(PE413 ((_ extract 413 413) P))
				(PE414 ((_ extract 414 414) P))
				(PE415 ((_ extract 415 415) P))
				(PE416 ((_ extract 416 416) P))
				(PE417 ((_ extract 417 417) P))
				(PE418 ((_ extract 418 418) P))
				(PE419 ((_ extract 419 419) P))
				(PE420 ((_ extract 420 420) P))
				(PE421 ((_ extract 421 421) P))
				(PE422 ((_ extract 422 422) P))
				(PE423 ((_ extract 423 423) P))
				(PE424 ((_ extract 424 424) P))
				(PE425 ((_ extract 425 425) P))
				(PE426 ((_ extract 426 426) P))
				(PE427 ((_ extract 427 427) P))
				(PE428 ((_ extract 428 428) P))
				(PE429 ((_ extract 429 429) P))
				(PE430 ((_ extract 430 430) P))				
				(PE431 ((_ extract 431 431) P))
				(PE432 ((_ extract 432 432) P))
				(PE433 ((_ extract 433 433) P))
				(PE434 ((_ extract 434 434) P))
				(PE435 ((_ extract 435 435) P))
				(PE436 ((_ extract 436 436) P))
				(PE437 ((_ extract 437 437) P))
				(PE438 ((_ extract 438 438) P))
				(PE439 ((_ extract 439 439) P))
				(PE440 ((_ extract 440 440) P))				
				(PE441 ((_ extract 441 441) P))
				(PE442 ((_ extract 442 442) P))
				(PE443 ((_ extract 443 443) P))
				(PE444 ((_ extract 444 444) P))
				(PE445 ((_ extract 445 445) P))
				(PE446 ((_ extract 446 446) P))
				(PE447 ((_ extract 447 447) P))
				(PE448 ((_ extract 448 448) P))
				(PE449 ((_ extract 449 449) P))
				(PE450 ((_ extract 450 450) P))				
				(PE451 ((_ extract 451 451) P))
				(PE452 ((_ extract 452 452) P))
				(PE453 ((_ extract 453 453) P))
				(PE454 ((_ extract 454 454) P))
				(PE455 ((_ extract 455 455) P))
				(PE456 ((_ extract 456 456) P))
				(PE457 ((_ extract 457 457) P))
				(PE458 ((_ extract 458 458) P))
				(PE459 ((_ extract 459 459) P))
				(PE460 ((_ extract 460 460) P))				
				(PE461 ((_ extract 461 461) P))
				(PE462 ((_ extract 462 462) P))
				(PE463 ((_ extract 463 463) P))
				(PE464 ((_ extract 464 464) P))
				(PE465 ((_ extract 465 465) P))
				(PE466 ((_ extract 466 466) P))
				(PE467 ((_ extract 467 467) P))
				(PE468 ((_ extract 468 468) P))
				(PE469 ((_ extract 469 469) P))
				(PE470 ((_ extract 470 470) P))				
				(PE471 ((_ extract 471 471) P))
				(PE472 ((_ extract 472 472) P))
				(PE473 ((_ extract 473 473) P))
				(PE474 ((_ extract 474 474) P))
				(PE475 ((_ extract 475 475) P))
				(PE476 ((_ extract 476 476) P))
				(PE477 ((_ extract 477 477) P))
				(PE478 ((_ extract 478 478) P))
				(PE479 ((_ extract 479 479) P))
				(PE480 ((_ extract 480 480) P))				
				(PE481 ((_ extract 481 481) P))
				(PE482 ((_ extract 482 482) P))
				(PE483 ((_ extract 483 483) P))
				(PE484 ((_ extract 484 484) P))
				(PE485 ((_ extract 485 485) P))
				(PE486 ((_ extract 486 486) P))
				(PE487 ((_ extract 487 487) P))
				(PE488 ((_ extract 488 488) P))
				(PE489 ((_ extract 489 489) P))
				(PE490 ((_ extract 490 490) P))				
				(PE491 ((_ extract 491 491) P))
				(PE492 ((_ extract 492 492) P))
				(PE493 ((_ extract 493 493) P))
				(PE494 ((_ extract 494 494) P))
				(PE495 ((_ extract 495 495) P))
				(PE496 ((_ extract 496 496) P))
				(PE497 ((_ extract 497 497) P))
				(PE498 ((_ extract 498 498) P))
				(PE499 ((_ extract 499 499) P))
				(PE500 ((_ extract 500 500) P))				
				(PE501 ((_ extract 501 501) P))
				(PE502 ((_ extract 502 502) P))
				(PE503 ((_ extract 503 503) P))
				(PE504 ((_ extract 504 504) P))
				(PE505 ((_ extract 505 505) P))
				(PE506 ((_ extract 506 506) P))
				(PE507 ((_ extract 507 507) P))
				(PE508 ((_ extract 508 508) P))
				(PE509 ((_ extract 509 509) P))
				(PE510 ((_ extract 510 510) P))				
				(PE511 ((_ extract 511 511) P))
				(PE512 ((_ extract 512 512) P))
				(PE513 ((_ extract 513 513) P))
				(PE514 ((_ extract 514 514) P))
				(PE515 ((_ extract 515 515) P))
				(PE516 ((_ extract 516 516) P))
				(PE517 ((_ extract 517 517) P))
				(PE518 ((_ extract 518 518) P))
				(PE519 ((_ extract 519 519) P))
				(PE520 ((_ extract 520 520) P))				
				(PE521 ((_ extract 521 521) P))
				(PE522 ((_ extract 522 522) P))
				(PE523 ((_ extract 523 523) P))
				(PE524 ((_ extract 524 524) P))
				(PE525 ((_ extract 525 525) P))
				(PE526 ((_ extract 526 526) P))
				(PE527 ((_ extract 527 527) P))
				(PE528 ((_ extract 528 528) P))
				(PE529 ((_ extract 529 529) P))
				(PE530 ((_ extract 530 530) P))				
				(PE531 ((_ extract 531 531) P))
				(PE532 ((_ extract 532 532) P))
				(PE533 ((_ extract 533 533) P))
				(PE534 ((_ extract 534 534) P))
				(PE535 ((_ extract 535 535) P))
				(PE536 ((_ extract 536 536) P))
				(PE537 ((_ extract 537 537) P))
				(PE538 ((_ extract 538 538) P))
				(PE539 ((_ extract 539 539) P))
				(PE540 ((_ extract 540 540) P))				
				(PE541 ((_ extract 541 541) P))
				(PE542 ((_ extract 542 542) P))
				(PE543 ((_ extract 543 543) P))
				(PE544 ((_ extract 544 544) P))
				(PE545 ((_ extract 545 545) P))
				(PE546 ((_ extract 546 546) P))
				(PE547 ((_ extract 547 547) P))
				(PE548 ((_ extract 548 548) P))
				(PE549 ((_ extract 549 549) P))
				(PE550 ((_ extract 550 550) P))				
				(PE551 ((_ extract 551 551) P))
				(PE552 ((_ extract 552 552) P))
				(PE553 ((_ extract 553 553) P))
				(PE554 ((_ extract 554 554) P))
				(PE555 ((_ extract 555 555) P))
				(PE556 ((_ extract 556 556) P))
				(PE557 ((_ extract 557 557) P))
				(PE558 ((_ extract 558 558) P))
				(PE559 ((_ extract 559 559) P))
				(PE560 ((_ extract 560 560) P))				
				(PE561 ((_ extract 561 561) P))
				(PE562 ((_ extract 562 562) P))
				(PE563 ((_ extract 563 563) P))
				(PE564 ((_ extract 564 564) P))
				(PE565 ((_ extract 565 565) P))
				(PE566 ((_ extract 566 566) P))
				(PE567 ((_ extract 567 567) P))
				(PE568 ((_ extract 568 568) P))
				(PE569 ((_ extract 569 569) P))
				(PE570 ((_ extract 570 570) P))				
				(PE571 ((_ extract 571 571) P))
				(PE572 ((_ extract 572 572) P))
				(PE573 ((_ extract 573 573) P))
				(PE574 ((_ extract 574 574) P))
				(PE575 ((_ extract 575 575) P))
				(PE576 ((_ extract 576 576) P))
				(PE577 ((_ extract 577 577) P))
				(PE578 ((_ extract 578 578) P))
				(PE579 ((_ extract 579 579) P))
				(PE580 ((_ extract 580 580) P))				
				(PE581 ((_ extract 581 581) P))
				(PE582 ((_ extract 582 582) P))
				(PE583 ((_ extract 583 583) P))
				(PE584 ((_ extract 584 584) P))
				(PE585 ((_ extract 585 585) P))
				(PE586 ((_ extract 586 586) P))
				(PE587 ((_ extract 587 587) P))
				(PE588 ((_ extract 588 588) P))
				(PE589 ((_ extract 589 589) P))
				(PE590 ((_ extract 590 590) P))				
				(PE591 ((_ extract 591 591) P))
				(PE592 ((_ extract 592 592) P))
				(PE593 ((_ extract 593 593) P))
				(PE594 ((_ extract 594 594) P))
				(PE595 ((_ extract 595 595) P))
				(PE596 ((_ extract 596 596) P))
				(PE597 ((_ extract 597 597) P))
				(PE598 ((_ extract 598 598) P))
				(PE599 ((_ extract 599 599) P))
				(PE600 ((_ extract 600 600) P))				
				(PE601 ((_ extract 601 601) P))
				(PE602 ((_ extract 602 602) P))
				(PE603 ((_ extract 603 603) P))
				(PE604 ((_ extract 604 604) P))
				(PE605 ((_ extract 605 605) P))
				(PE606 ((_ extract 606 606) P))
				(PE607 ((_ extract 607 607) P))
				(PE608 ((_ extract 608 608) P))
				(PE609 ((_ extract 609 609) P))
				(PE610 ((_ extract 610 610) P))				
				(PE611 ((_ extract 611 611) P))
				(PE612 ((_ extract 612 612) P))
				(PE613 ((_ extract 613 613) P))
				(PE614 ((_ extract 614 614) P))
				(PE615 ((_ extract 615 615) P))
				(PE616 ((_ extract 616 616) P))
				(PE617 ((_ extract 617 617) P))
				(PE618 ((_ extract 618 618) P))
				(PE619 ((_ extract 619 619) P))
				(PE620 ((_ extract 620 620) P))				
				(PE621 ((_ extract 621 621) P))
				(PE622 ((_ extract 622 622) P))
				(PE623 ((_ extract 623 623) P))
				(PE624 ((_ extract 624 624) P))
				(PE625 ((_ extract 625 625) P))
				(PE626 ((_ extract 626 626) P))
				(PE627 ((_ extract 627 627) P))
				(PE628 ((_ extract 628 628) P))
				(PE629 ((_ extract 629 629) P))
				(PE630 ((_ extract 630 630) P))				
				(PE631 ((_ extract 631 631) P))
				(PE632 ((_ extract 632 632) P))
				(PE633 ((_ extract 633 633) P))
				(PE634 ((_ extract 634 634) P))			
				(PE635 ((_ extract 635 635) P))
				(PE636 ((_ extract 636 636) P))			
				(PE637 ((_ extract 637 637) P))
				(PE638 ((_ extract 638 638) P))
				(PE639 ((_ extract 639 639) P))
				(PE640 ((_ extract 640 640) P))
				(PE641 ((_ extract 641 641) P))
				(PE642 ((_ extract 642 642) P))
				(PE643 ((_ extract 643 643) P))
				(PE644 ((_ extract 644 644) P))
				(PE645 ((_ extract 645 645) P))
				(PE646 ((_ extract 646 646) P))
				(PE647 ((_ extract 647 647) P))
				(PE648 ((_ extract 648 648) P))
				(PE649 ((_ extract 649 649) P))
				(PE650 ((_ extract 650 650) P))
				(PE651 ((_ extract 651 651) P))
				(PE652 ((_ extract 652 652) P))
				(PE653 ((_ extract 653 653) P))
				(PE654 ((_ extract 654 654) P))
				(PE655 ((_ extract 655 655) P))
				(PE656 ((_ extract 656 656) P))
				(PE657 ((_ extract 657 657) P))
				(PE658 ((_ extract 658 658) P))
				(PE659 ((_ extract 659 659) P))
				(PE660 ((_ extract 660 660) P))
				(PE661 ((_ extract 661 661) P))
				(PE662 ((_ extract 662 662) P))
				(PE663 ((_ extract 663 663) P))
				(PE664 ((_ extract 664 664) P))
				(PE665 ((_ extract 665 665) P))
				(PE666 ((_ extract 666 666) P))
				(PE667 ((_ extract 667 667) P))
				(PE668 ((_ extract 668 668) P))
				(PE669 ((_ extract 669 669) P))
				(PE670 ((_ extract 670 670) P))
				(PE671 ((_ extract 671 671) P))
				(PE672 ((_ extract 672 672) P))
				(PE673 ((_ extract 673 673) P))
				(PE674 ((_ extract 674 674) P))
				(PE675 ((_ extract 675 675) P))
				(PE676 ((_ extract 676 676) P))
				(PE677 ((_ extract 677 677) P))
				(PE678 ((_ extract 678 678) P))
				(PE679 ((_ extract 679 679) P))
				(PE680 ((_ extract 680 680) P))
				(PE681 ((_ extract 681 681) P))
				(PE682 ((_ extract 682 682) P))
				(PE683 ((_ extract 683 683) P))
				(PE684 ((_ extract 684 684) P))
				(PE685 ((_ extract 685 685) P))
				(PE686 ((_ extract 686 686) P))
				(PE687 ((_ extract 687 687) P))
				(PE688 ((_ extract 688 688) P))
				(PE689 ((_ extract 689 689) P))
				(PE690 ((_ extract 690 690) P))
				(PE691 ((_ extract 691 691) P))
				(PE692 ((_ extract 692 692) P))
				(PE693 ((_ extract 693 693) P))
				(PE694 ((_ extract 694 694) P))
				(PE695 ((_ extract 695 695) P))
				(PE696 ((_ extract 696 696) P))
				(PE697 ((_ extract 697 697) P))
				(PE698 ((_ extract 698 698) P))
				(PE699 ((_ extract 699 699) P))
				(PE700 ((_ extract 700 700) P))
				(PE701 ((_ extract 701 701) P))
				(PE702 ((_ extract 702 702) P))
				(PE703 ((_ extract 703 703) P))
				(PE704 ((_ extract 704 704) P))
				(PE705 ((_ extract 705 705) P))
				(PE706 ((_ extract 706 706) P))
				(PE707 ((_ extract 707 707) P))
				(PE708 ((_ extract 708 708) P))
				(PE709 ((_ extract 709 709) P))
				(PE710 ((_ extract 710 710) P))
				(PE711 ((_ extract 711 711) P))
				(PE712 ((_ extract 712 712) P))
				(PE713 ((_ extract 713 713) P))
				(PE714 ((_ extract 714 714) P))
				(PE715 ((_ extract 715 715) P))
				(PE716 ((_ extract 716 716) P))
				(PE717 ((_ extract 717 717) P))
				(PE718 ((_ extract 718 718) P))
				(PE719 ((_ extract 719 719) P))
				(PE720 ((_ extract 720 720) P))
				(PE721 ((_ extract 721 721) P))
				(PE722 ((_ extract 722 722) P))
				(PE723 ((_ extract 723 723) P))
				(PE724 ((_ extract 724 724) P))
				(PE725 ((_ extract 725 725) P))
				(PE726 ((_ extract 726 726) P))
				(PE727 ((_ extract 727 727) P))
				(PE728 ((_ extract 728 728) P))
				(PE729 ((_ extract 729 729) P))
				(PE730 ((_ extract 730 730) P))
				(PE731 ((_ extract 731 731) P))
				(PE732 ((_ extract 732 732) P))
				(PE733 ((_ extract 733 733) P))
				(PE734 ((_ extract 734 734) P))
				(PE735 ((_ extract 735 735) P))
				(PE736 ((_ extract 736 736) P))
				(PE737 ((_ extract 737 737) P))
				(PE738 ((_ extract 738 738) P))
				(PE739 ((_ extract 739 739) P))
				(PE740 ((_ extract 740 740) P))
				(PE741 ((_ extract 741 741) P))
				(PE742 ((_ extract 742 742) P))
				(PE743 ((_ extract 743 743) P))
				(PE744 ((_ extract 744 744) P))
				(PE745 ((_ extract 745 745) P))
				(PE746 ((_ extract 746 746) P))
				(PE747 ((_ extract 747 747) P))
				(PE748 ((_ extract 748 748) P))
				(PE749 ((_ extract 749 749) P))
				(PE750 ((_ extract 750 750) P))
				(PE751 ((_ extract 751 751) P))
				(PE752 ((_ extract 752 752) P))
				(PE753 ((_ extract 753 753) P))
				(PE754 ((_ extract 754 754) P))
				(PE755 ((_ extract 755 755) P))
				(PE756 ((_ extract 756 756) P))
				(PE757 ((_ extract 757 757) P))
				(PE758 ((_ extract 758 758) P))
				(PE759 ((_ extract 759 759) P))
				(PE760 ((_ extract 760 760) P))
				(PE761 ((_ extract 761 761) P))
				(PE762 ((_ extract 762 762) P))
				(PE763 ((_ extract 763 763) P))
				(PE764 ((_ extract 764 764) P))
				(PE765 ((_ extract 765 765) P))
				(PE766 ((_ extract 766 766) P))
				(PE767 ((_ extract 767 767) P))
				(PE768 ((_ extract 768 768) P))
				(PE769 ((_ extract 769 769) P))
				(PE770 ((_ extract 770 770) P))
				(PE771 ((_ extract 771 771) P))
				(PE772 ((_ extract 772 772) P))
				(PE773 ((_ extract 773 773) P))
				(PE774 ((_ extract 774 774) P))
				(PE775 ((_ extract 775 775) P))
				(PE776 ((_ extract 776 776) P))
				(PE777 ((_ extract 777 777) P))
				(PE778 ((_ extract 778 778) P))
				(PE779 ((_ extract 779 779) P))
				(PE780 ((_ extract 780 780) P))
				(PE781 ((_ extract 781 781) P))
				(PE782 ((_ extract 782 782) P))
				(PE783 ((_ extract 783 783) P))
				(PE784 ((_ extract 784 784) P))
				(PE785 ((_ extract 785 785) P))
				(PE786 ((_ extract 786 786) P))
				(PE787 ((_ extract 787 787) P))
				(PE788 ((_ extract 788 788) P))
				(PE789 ((_ extract 789 789) P))
				(PE790 ((_ extract 790 790) P))
				(PE791 ((_ extract 791 791) P))
				(PE792 ((_ extract 792 792) P))
				(PE793 ((_ extract 793 793) P))
				(PE794 ((_ extract 794 794) P))
				(PE795 ((_ extract 795 795) P))
				(PE796 ((_ extract 796 796) P))
				(PE797 ((_ extract 797 797) P))				
				(PE797 ((_ extract 797 797) P))
				(PE798 ((_ extract 798 798) P))
				(PE799 ((_ extract 799 799) P))
				(PE800 ((_ extract 800 800) P))
				(PE801 ((_ extract 801 801) P))
				(PE802 ((_ extract 802 802) P))
				(PE803 ((_ extract 803 803) P))
				(PE804 ((_ extract 804 804) P))
				(PE805 ((_ extract 805 805) P))
				(PE806 ((_ extract 806 806) P))
				(PE807 ((_ extract 807 807) P))
				(PE808 ((_ extract 808 808) P))
				(PE809 ((_ extract 809 809) P))
				(PE810 ((_ extract 810 810) P))
				(PE811 ((_ extract 811 811) P))
				(PE812 ((_ extract 812 812) P))
				(PE813 ((_ extract 813 813) P))
				(PE814 ((_ extract 814 814) P))
				(PE815 ((_ extract 815 815) P))
				(PE816 ((_ extract 816 816) P))
				(PE817 ((_ extract 817 817) P))
				(PE818 ((_ extract 818 818) P))
				(PE819 ((_ extract 819 819) P))
				(PE820 ((_ extract 820 820) P))
				(PE821 ((_ extract 821 821) P))
				(PE822 ((_ extract 822 822) P))
				(PE823 ((_ extract 823 823) P))
				(PE824 ((_ extract 824 824) P))
				(PE825 ((_ extract 825 825) P))
				(PE826 ((_ extract 826 826) P))
				(PE827 ((_ extract 827 827) P))
				(PE828 ((_ extract 828 828) P))
				(PE829 ((_ extract 829 829) P))
				(PE830 ((_ extract 830 830) P))
				(PE831 ((_ extract 831 831) P))
				(PE832 ((_ extract 832 832) P))
				(PE833 ((_ extract 833 833) P))
				(PE834 ((_ extract 834 834) P))
				(PE835 ((_ extract 835 835) P))
				(PE836 ((_ extract 836 836) P))
				(PE837 ((_ extract 837 837) P))
				(PE838 ((_ extract 838 838) P))
				(PE839 ((_ extract 839 839) P))
				(PE840 ((_ extract 840 840) P))
				(PE841 ((_ extract 841 841) P))
				(PE842 ((_ extract 842 842) P))
				(PE843 ((_ extract 843 843) P))
				(PE844 ((_ extract 844 844) P))
				(PE845 ((_ extract 845 845) P))
				(PE846 ((_ extract 846 846) P))
				(PE847 ((_ extract 847 847) P))
				(PE848 ((_ extract 848 848) P))
				(PE849 ((_ extract 849 849) P))
				(PE850 ((_ extract 850 850) P))
				(PE851 ((_ extract 851 851) P))
				(PE852 ((_ extract 852 852) P))
				(PE853 ((_ extract 853 853) P))
				(PE854 ((_ extract 854 854) P))
				(PE855 ((_ extract 855 855) P))
				(PE856 ((_ extract 856 856) P))
				(PE857 ((_ extract 857 857) P))
				(PE858 ((_ extract 858 858) P))
				(PE859 ((_ extract 859 859) P))
				(PE860 ((_ extract 860 860) P))
				(PE861 ((_ extract 861 861) P))
				(PE862 ((_ extract 862 862) P))
				(PE863 ((_ extract 863 863) P))
				(PE864 ((_ extract 864 864) P))
				(PE865 ((_ extract 865 865) P))
				(PE866 ((_ extract 866 866) P))
				(PE867 ((_ extract 867 867) P))
				(PE868 ((_ extract 868 868) P))
				(PE869 ((_ extract 869 869) P))
				(PE870 ((_ extract 870 870) P))
				(PE871 ((_ extract 871 871) P))
				(PE872 ((_ extract 872 872) P))
				(PE873 ((_ extract 873 873) P))
				(PE874 ((_ extract 874 874) P))
				(PE875 ((_ extract 875 875) P))
				(PE876 ((_ extract 876 876) P))
				(PE877 ((_ extract 877 877) P))
				(PE878 ((_ extract 878 878) P))
				(PE879 ((_ extract 879 879) P))
				(PE880 ((_ extract 880 880) P))
				(PE881 ((_ extract 881 881) P))
				(PE882 ((_ extract 882 882) P))
				(PE883 ((_ extract 883 883) P))
				(PE884 ((_ extract 884 884) P))
				(PE885 ((_ extract 885 885) P))
				(PE886 ((_ extract 886 886) P))
				(PE887 ((_ extract 887 887) P))
				(PE888 ((_ extract 888 888) P))
				(PE889 ((_ extract 889 889) P))
				(PE890 ((_ extract 890 890) P))
				(PE891 ((_ extract 891 891) P))
				(PE892 ((_ extract 892 892) P))
				(PE893 ((_ extract 893 893) P))
				(PE894 ((_ extract 894 894) P))
				(PE895 ((_ extract 895 895) P))
				(PE896 ((_ extract 896 896) P))
				(PE897 ((_ extract 897 897) P))
				(PE898 ((_ extract 898 898) P))
				(PE899 ((_ extract 899 899) P))
				(PE900 ((_ extract 900 900) P))
				(PE901 ((_ extract 901 901) P))
				(PE902 ((_ extract 902 902) P))
				(PE903 ((_ extract 903 903) P))
				(PE904 ((_ extract 904 904) P))
				(PE905 ((_ extract 905 905) P))
				(PE906 ((_ extract 906 906) P))
				(PE907 ((_ extract 907 907) P))
				(PE908 ((_ extract 908 908) P))
				(PE909 ((_ extract 909 909) P))
				(PE910 ((_ extract 910 910) P))
				(PE911 ((_ extract 911 911) P))
				(PE912 ((_ extract 912 912) P))
				(PE913 ((_ extract 913 913) P))
				(PE914 ((_ extract 914 914) P))
				(PE915 ((_ extract 915 915) P))
				(PE916 ((_ extract 916 916) P))
				(PE917 ((_ extract 917 917) P))
				(PE918 ((_ extract 918 918) P))
				(PE919 ((_ extract 919 919) P))
				(PE920 ((_ extract 920 920) P))
				(PE921 ((_ extract 921 921) P))
				(PE922 ((_ extract 922 922) P))
				(PE923 ((_ extract 923 923) P))
				(PE924 ((_ extract 924 924) P))
				(PE925 ((_ extract 925 925) P))
				(PE926 ((_ extract 926 926) P))
				(PE927 ((_ extract 927 927) P))
				(PE928 ((_ extract 928 928) P))
				(PE929 ((_ extract 929 929) P))
				(PE930 ((_ extract 930 930) P))
				(PE931 ((_ extract 931 931) P))
				(PE932 ((_ extract 932 932) P))
				(PE933 ((_ extract 933 933) P))
				(PE934 ((_ extract 934 934) P))
				(PE935 ((_ extract 935 935) P))
				(PE936 ((_ extract 936 936) P))
				(PE937 ((_ extract 937 937) P))
				(PE938 ((_ extract 938 938) P))
				(PE939 ((_ extract 939 939) P))
				(PE940 ((_ extract 940 940) P))
				(PE941 ((_ extract 941 941) P))
				(PE942 ((_ extract 942 942) P))
				(PE943 ((_ extract 943 943) P))
				(PE944 ((_ extract 944 944) P))
				(PE945 ((_ extract 945 945) P))
				(PE946 ((_ extract 946 946) P))
				(PE947 ((_ extract 947 947) P))
				(PE948 ((_ extract 948 948) P))
				(PE949 ((_ extract 949 949) P))
				(PE950 ((_ extract 950 950) P))
				(PE951 ((_ extract 951 951) P))
				(PE952 ((_ extract 952 952) P))
				(PE953 ((_ extract 953 953) P))
				(PE954 ((_ extract 954 954) P))
				(PE955 ((_ extract 955 955) P))
				(PE956 ((_ extract 956 956) P))
				(PE957 ((_ extract 957 957) P))
				(PE958 ((_ extract 958 958) P))
				(PE959 ((_ extract 959 959) P))
				(PE960 ((_ extract 960 960) P))
				(PE961 ((_ extract 961 961) P))
				(PE962 ((_ extract 962 962) P))
				(PE963 ((_ extract 963 963) P))
				(PE964 ((_ extract 964 964) P))
				(PE965 ((_ extract 965 965) P))
				(PE966 ((_ extract 966 966) P))
				(PE967 ((_ extract 967 967) P))
				(PE968 ((_ extract 968 968) P))
				(PE969 ((_ extract 969 969) P))
				(PE970 ((_ extract 970 970) P))
				(PE971 ((_ extract 971 971) P))
				(PE972 ((_ extract 972 972) P))
				(PE973 ((_ extract 973 973) P))
				(PE974 ((_ extract 974 974) P))
				(PE975 ((_ extract 975 975) P))
				(PE976 ((_ extract 976 976) P))
				(PE977 ((_ extract 977 977) P))
				(PE978 ((_ extract 978 978) P))
				(PE979 ((_ extract 979 979) P))
				(PE980 ((_ extract 980 980) P))
				(PE981 ((_ extract 981 981) P))
				(PE982 ((_ extract 982 982) P))
				(PE983 ((_ extract 983 983) P))
				(PE984 ((_ extract 984 984) P))
				(PE985 ((_ extract 985 985) P))
				(PE986 ((_ extract 986 986) P))
				(PE987 ((_ extract 987 987) P))
				(PE988 ((_ extract 988 988) P))
				(PE989 ((_ extract 989 989) P))
				(PE990 ((_ extract 990 990) P))
				(PE991 ((_ extract 991 991) P))
				(PE992 ((_ extract 992 992) P))
				(PE993 ((_ extract 993 993) P))
				(PE994 ((_ extract 994 994) P))
				(PE995 ((_ extract 995 995) P))
				(PE996 ((_ extract 996 996) P))
				(PE997 ((_ extract 997 997) P))
				(PE998 ((_ extract 998 998) P))
				(PE999 ((_ extract 999 999) P))
				(PE1000 ((_ extract 1000 1000) P))
				(PE1001 ((_ extract 1001 1001) P))
				(PE1002 ((_ extract 1002 1002) P))
				(PE1003 ((_ extract 1003 1003) P))
				(PE1004 ((_ extract 1004 1004) P))
				(PE1005 ((_ extract 1005 1005) P))
				(PE1006 ((_ extract 1006 1006) P))
				(PE1007 ((_ extract 1007 1007) P))
				(PE1008 ((_ extract 1008 1008) P))
				(PE1009 ((_ extract 1009 1009) P))
				(PE1010 ((_ extract 1010 1010) P))
				(PE1011 ((_ extract 1011 1011) P))
				(PE1012 ((_ extract 1012 1012) P))
				(PE1013 ((_ extract 1013 1013) P))
				(PE1014 ((_ extract 1014 1014) P))
				(PE1015 ((_ extract 1015 1015) P))
				(PE1016 ((_ extract 1016 1016) P))
				(PE1017 ((_ extract 1017 1017) P))
				(PE1018 ((_ extract 1018 1018) P))
				(PE1019 ((_ extract 1019 1019) P))
				(PE1020 ((_ extract 1020 1020) P))
				(PE1021 ((_ extract 1021 1021) P))
				(PE1022 ((_ extract 1022 1022) P))
				(PE1023 ((_ extract 1023 1023) P))
				(PE1024 ((_ extract 1024 1024) P))
				(PE1025 ((_ extract 1025 1025) P))
				(PE1026 ((_ extract 1026 1026) P))
				(PE1027 ((_ extract 1027 1027) P))
				(PE1028 ((_ extract 1028 1028) P))
				(PE1029 ((_ extract 1029 1029) P))
				(PE1030 ((_ extract 1030 1030) P))
				(PE1031 ((_ extract 1031 1031) P))
				(PE1032 ((_ extract 1032 1032) P))
				(PE1033 ((_ extract 1033 1033) P))
				(PE1034 ((_ extract 1034 1034) P))
				(PE1035 ((_ extract 1035 1035) P))
				(PE1036 ((_ extract 1036 1036) P))
				(PE1037 ((_ extract 1037 1037) P))
				(PE1038 ((_ extract 1038 1038) P))
				(PE1039 ((_ extract 1039 1039) P))
				(PE1040 ((_ extract 1040 1040) P))
				(PE1041 ((_ extract 1041 1041) P))
				(PE1042 ((_ extract 1042 1042) P))
				(PE1043 ((_ extract 1043 1043) P))
				(PE1044 ((_ extract 1044 1044) P))
				(PE1045 ((_ extract 1045 1045) P))
				(PE1046 ((_ extract 1046 1046) P))
				(PE1047 ((_ extract 1047 1047) P))
				(PE1048 ((_ extract 1048 1048) P))
				(PE1049 ((_ extract 1049 1049) P))
				(PE1050 ((_ extract 1050 1050) P))
				(PE1051 ((_ extract 1051 1051) P))
				(PE1052 ((_ extract 1052 1052) P))
				(PE1053 ((_ extract 1053 1053) P))
				(PE1054 ((_ extract 1054 1054) P))
				(PE1055 ((_ extract 1055 1055) P))
				(PE1056 ((_ extract 1056 1056) P))
				(PE1057 ((_ extract 1057 1057) P))
				(PE1058 ((_ extract 1058 1058) P))
				(PE1059 ((_ extract 1059 1059) P))
				(PE1060 ((_ extract 1060 1060) P))
				(PE1061 ((_ extract 1061 1061) P))
				(PE1062 ((_ extract 1062 1062) P))
				(PE1063 ((_ extract 1063 1063) P))
				(PE1064 ((_ extract 1064 1064) P))
				(PE1065 ((_ extract 1065 1065) P))
				(PE1066 ((_ extract 1066 1066) P))
				(PE1067 ((_ extract 1067 1067) P))
				(PE1068 ((_ extract 1068 1068) P))
				(PE1069 ((_ extract 1069 1069) P))
				(PE1070 ((_ extract 1070 1070) P))
				(PE1071 ((_ extract 1071 1071) P))
				(PE1072 ((_ extract 1072 1072) P))
				(PE1073 ((_ extract 1073 1073) P))
				(PE1074 ((_ extract 1074 1074) P))
				(PE1075 ((_ extract 1075 1075) P))
				(PE1076 ((_ extract 1076 1076) P))
				(PE1077 ((_ extract 1077 1077) P))
				(PE1078 ((_ extract 1078 1078) P))
				(PE1079 ((_ extract 1079 1079) P))
				(PE1080 ((_ extract 1080 1080) P))
				(PE1081 ((_ extract 1081 1081) P))
				(PE1082 ((_ extract 1082 1082) P))
				(PE1083 ((_ extract 1083 1083) P))
				(PE1084 ((_ extract 1084 1084) P))
				(PE1085 ((_ extract 1085 1085) P))
				(PE1086 ((_ extract 1086 1086) P))
				(PE1087 ((_ extract 1087 1087) P))
				(PE1088 ((_ extract 1088 1088) P))
				(PE1089 ((_ extract 1089 1089) P))
				(PE1090 ((_ extract 1090 1090) P))
				(PE1091 ((_ extract 1091 1091) P))
				(PE1092 ((_ extract 1092 1092) P))
				(PE1093 ((_ extract 1093 1093) P))
				(PE1094 ((_ extract 1094 1094) P))
				(PE1095 ((_ extract 1095 1095) P))
				(PE1096 ((_ extract 1096 1096) P))
				(PE1097 ((_ extract 1097 1097) P))
				(PE1098 ((_ extract 1098 1098) P))
				(PE1099 ((_ extract 1099 1099) P))
				(PE1100 ((_ extract 1100 1100) P))
				(PE1101 ((_ extract 1101 1101) P))
				(PE1102 ((_ extract 1102 1102) P))
				(PE1103 ((_ extract 1103 1103) P))
				(PE1104 ((_ extract 1104 1104) P))
				(PE1105 ((_ extract 1105 1105) P))
				(PE1106 ((_ extract 1106 1106) P))
				(PE1107 ((_ extract 1107 1107) P))
				(PE1108 ((_ extract 1108 1108) P))
				(PE1109 ((_ extract 1109 1109) P))
				(PE1110 ((_ extract 1110 1110) P))
				(PE1111 ((_ extract 1111 1111) P))
				(PE1112 ((_ extract 1112 1112) P))
				(PE1113 ((_ extract 1113 1113) P))
				(PE1114 ((_ extract 1114 1114) P))
				(PE1115 ((_ extract 1115 1115) P))

			)
		(let
			(
				(P0 (concat PE1115 PE1114 PE1113 PE1112	PE1111 PE1110 PE1109 PE1108	PE1107 PE1106 PE1105 PE1104	PE1103 PE1102 PE1101 PE1100 PE1099 PE1098 PE1097 PE1096	PE1095 PE1094 PE1093 PE1092	PE1091 PE1090 PE1089 PE1088	PE1087 PE1086 PE1085 PE1084	PE1083 PE1082 PE1081 PE1080	PE1079 PE1078 PE1077 PE1076	PE1075 PE1074 PE1073 PE1072	PE1071 PE1070 PE1069 PE1068	PE1067 PE1066 PE1065 PE1064	PE1063 PE1062 PE1061 PE1060	PE1059 PE1058 PE1057 PE1056	PE1055 PE1054 PE1053 PE1052	PE1051 PE1050 PE1049 PE1048	PE1047 PE1046 PE1045 PE1044	PE1043 PE1042 PE1041 PE1040	PE1039 PE1038 PE1037 PE1036	PE1035 PE1034 PE1033 PE1032	PE1031 PE1030 PE1029 PE1028	PE1027 PE1026 PE1025 PE1024	PE1023 PE1022 PE1021 PE1020	PE1019 PE1018 PE1017 PE1016	PE1015 PE1014 PE1013 PE1012	PE1011 PE1010 PE1009 PE1008	PE1007 PE1006 PE1005 PE1004	PE1003 PE1002 PE1001 PE1000	PE999 PE998	PE997 PE996	PE995 PE994	PE993 PE992	PE991 PE990	PE989 PE988	PE987 PE986	PE985 PE984	PE983 PE982	PE981 PE980	PE979 PE978	PE977 PE976	PE975 PE974	PE973 PE972	PE971 PE970	PE969 PE968	PE967 PE966	PE965 PE964	PE963 PE962	PE961 PE960	PE959 PE958	PE957 PE956	PE955 PE954	PE953 PE952	PE951 PE950	PE949 PE948	PE947 PE946	PE945 PE944	PE943 PE942	PE941 PE940	PE939 PE938	PE937 PE936	PE935 PE934	PE933 PE932	PE931 PE930	PE929 PE928	PE927 PE926	PE925 PE924	PE923 PE922	PE921 PE920	PE919 PE918	PE917 PE916	PE915 PE914	PE913 PE912	PE911 PE910	PE909 PE908	PE907 PE906	PE905 PE904	PE903 PE902	PE901 PE900	PE899 PE898	PE897 PE896	PE895 PE894	PE893 PE892	PE891 PE890	PE889 PE888	PE887 PE886	PE885 PE884	PE883 PE882	PE881 PE880	PE879 PE878	PE877 PE876	PE875 PE874	PE873 PE872	PE871 PE870	PE869 PE868	PE867 PE866	PE865 PE864	PE863 PE862	PE861 PE860	PE859 PE858	PE857 PE856	PE855 PE854	PE853 PE852	PE851 PE850	PE849 PE848	PE847 PE846	PE845 PE844	PE843 PE842	PE841 PE840	PE839 PE838	PE837 PE836	PE835 PE834	PE833 PE832	PE831 PE830	PE829 PE828	PE827 PE826	PE825 PE824	PE823 PE822	PE821 PE820	PE819 PE818	PE817 PE816	PE815 PE814	PE813 PE812	PE811 PE810	PE809 PE808	PE807 PE806	PE805 PE804	PE803 PE802	PE801 PE800 PE799 PE798 PE797 PE796	PE795 PE794	PE793 PE792	PE791 PE790	PE789 PE788	PE787 PE786	PE785 PE784	PE783 PE782	PE781 PE780	PE779 PE778	PE777 PE776	PE775 PE774	PE773 PE772	PE771 PE770	PE769 PE768	PE767 PE766	PE765 PE764	PE763 PE762	PE761 PE760	PE759 PE758	PE757 PE756	PE755 PE754	PE753 PE752	PE751 PE750	PE749 PE748	PE747 PE746	PE745 PE744	PE743 PE742	PE741 PE740	PE739 PE738	PE737 PE736	PE735 PE734	PE733 PE732	PE731 PE730	PE729 PE728	PE727 PE726	PE725 PE724	PE723 PE722	PE721 PE720	PE719 PE718	PE717 PE716	PE715 PE714	PE713 PE712	PE711 PE710	PE709 PE708	PE707 PE706	PE705 PE704	PE703 PE702	PE701 PE700	PE699 PE698	PE697 PE696	PE695 PE694	PE693 PE692	PE691 PE690	PE689 PE688 PE687 PE686	PE685 PE684 PE683 PE682	PE681 PE680	PE679 PE678	PE677 PE676	PE675 PE674	PE673 PE672	PE671 PE670	PE669 PE668	PE667 PE666	PE665 PE664	PE663 PE662	PE661 PE660	PE659 PE658	PE657 PE656	PE655 PE654	PE653 PE652	PE651 PE650	PE649 PE648	PE647 PE646	PE645 PE644	PE643 PE642	PE641 PE640	PE639 PE638 PE637 PE636 PE635 PE634 PE633 PE632 PE631 PE630 PE629 PE628 PE627 PE626 PE625 PE624 PE623 PE622 PE621 PE620 PE619 PE618 PE617 PE616 PE615 PE614 PE613 PE612 PE611 PE610 PE609 PE608 PE607 PE606 PE605 PE604 PE603 PE602 PE601 PE600 PE599 PE598 PE597 PE596 PE595 PE594 PE593 PE592 PE591 PE590 PE589 PE588 PE587 PE586 PE585 PE584 PE583 PE582 PE581 PE580 PE579 PE578 PE577 PE576 PE575 PE574 PE573 PE572 PE571 PE570 PE569 PE568 PE567 PE566 PE565 PE564 PE563 PE562 PE561 PE560 PE559 PE558 PE557 PE556 PE555 PE554 PE553 PE552 PE551 PE550 PE549 PE548 PE547 PE546 PE545 PE544 PE543 PE542 PE541 PE540 PE539 PE538 PE537 PE536 PE535 PE534 PE533 PE532 PE531 PE530 PE529 PE528 PE527 PE526 PE525 PE524 PE523 PE522 PE521 PE520 PE519 PE518 PE517 PE516 PE515 PE514 PE513 PE512 PE511 PE510 PE509 PE508 PE507 PE506 PE505 PE504 PE503 PE502 PE501 PE500 PE499 PE498 PE497 PE496 PE495 PE494 PE493 PE492 PE491 PE490 PE489 PE488 PE487 PE486 PE485 PE484 PE483 PE482 PE481 PE480 PE479 PE478 PE477 PE476 PE475 PE474 PE473 PE472 PE471 PE470 PE469 PE468 PE467 PE466 PE465 PE464 PE463 PE462 PE461 PE460 PE459 PE458 PE457 PE456 PE455 PE454 PE453 PE452 PE451 PE450 PE449 PE448 PE447 PE446 PE445 PE444 PE443 PE442 PE441 PE440 PE439 PE438 PE437 PE436 PE435 PE434 PE433 PE432 PE431 PE430 PE429 PE428 PE427 PE426 PE425 PE424 PE423 PE422 PE421 PE420 PE419 PE418 PE417 PE416 PE415 PE414 PE413 PE412 PE411 PE410 PE409 PE408 PE407 PE406 PE405 PE404 PE403 PE402 PE401 PE400 PE399 PE398 PE397 PE396 PE395 PE394 PE393 PE392 PE391 PE390 PE389 PE388 PE387 PE386 PE385 PE384 PE383 PE382 PE381 PE380 PE379 PE378 PE377 PE376 PE375 PE374 PE373 PE372 PE371 PE370 PE369 PE368 PE367 PE366 PE365 PE364 PE363 PE362 PE361 PE360 PE359 PE358 PE357 PE356 PE355 PE354 PE353 PE352 PE351 PE350 PE349 PE348 PE347 PE346 PE345 PE344 PE343 PE342 PE341 PE340 PE339 PE338 PE337 PE336 PE335 PE334 PE333 PE332 PE331 PE330 PE329 PE328 PE327 PE326 PE325 PE324 PE323 PE322 PE321 PE320 PE319 PE318 PE317 PE316 PE315 PE314 PE313 PE312 PE311 PE310 PE309 PE308 PE307 PE306 PE305 PE304 PE303 PE302 PE301 PE300 PE299 PE298 PE297 PE296 PE295 PE294 PE293 PE292 PE291 PE290 PE289 PE288 PE287 PE286 PE285 PE284 PE283 PE282 PE281 PE280 PE279 PE278 PE277 PE276 PE275 PE274 PE273 PE272 PE271 PE270 PE269 PE268 PE267 PE266 PE265 PE264 PE263 PE262 PE261 PE260 PE259 PE258 PE257 PE256 PE255 PE254 PE253 PE252 PE251 PE250 PE249 PE248 PE247 PE246 PE245 PE244 PE243 PE242 PE241 PE240 PE239 PE238 PE237 PE236 PE235 PE234 PE233 PE232 PE231 PE230 PE229 PE228 PE227 PE226 PE225 PE224 PE223 PE222 PE221 PE220 PE219 PE218 PE217 PE216 PE215 PE214 PE213 PE212 PE211 PE210 PE209 PE208 PE207 PE206 PE205 PE204 PE203 PE202 PE201 PE200 PE199 PE198 PE197 PE196 PE195 PE194 PE193 PE192 PE191 PE190 PE189 PE188 PE187 PE186 PE185 PE184 PE183 PE182 PE181 PE180 PE179 PE178 PE177 PE176 PE175 PE174 PE173 PE172 PE171 PE170 PE169 PE168 PE167 PE166 PE165 PE164 PE163 PE162 PE161 PE160 PE159 PE158 PE157 PE156 PE155 PE154 PE153 PE152 PE151 PE150 PE149 PE148 PE147 PE146 PE145 PE144 PE143 PE142 PE141 PE140 PE139 PE138 PE137 PE136 PE135 PE134 PE133 PE132 PE131 PE130 PE129 PE128 PE127 PE126 PE125 PE124 PE123 PE122 PE121 PE120 PE119 PE118 PE117 PE116 PE115 PE114 PE113 PE112 PE111 PE110 PE109 PE108 PE107 PE106 PE105 PE104 PE103 PE102 PE101 PE100 PE99 PE98 PE97 PE96 PE95 PE94 PE93 PE92 PE91 PE90 PE89 PE88 PE87 PE86 PE85 PE84 PE83 PE82 PE81 PE80 PE79 PE78 PE77 PE76 PE75 PE74 PE73 PE72 PE71 PE70 PE69 PE68 PE67 PE66 PE65 PE64 PE63 PE62 PE61 PE60 PE59 PE58 PE57 PE56 PE55 PE54 PE53 PE52 PE51 PE50 PE49 PE48 PE47 PE46 PE45 PE44 PE43 PE42 PE41 PE40 PE39 PE38 PE37 PE36 PE35 PE34 PE33 PE32 PE31 PE30 PE29 PE28 PE27 PE26 PE25 PE24 PE23 PE22 PE21 PE20 PE19 PE18 PE17 PE16 PE15 PE14 PE13 PE12 PE11 PE10 PE9 PE8 PE7 PE6 PE5 PE4 PE3 PE2 PE1 PE0 bit_0))
				(P1_1 (concat PE1115 PE1114 PE1113 PE1112 PE1111 PE1110 PE1109 PE1108 PE1107 PE1106 PE1105 PE1104 PE1103 PE1102 PE1101 PE1100 PE1099 PE1098 PE1097 PE1096 PE1095 PE1094 PE1093 PE1092 PE1091 PE1090 PE1089 PE1088 PE1087 PE1086 PE1085 PE1084 PE1083 PE1082 PE1081 PE1080 PE1079 PE1078 PE1077 PE1076 PE1075 PE1074 PE1073 PE1072 PE1071 PE1070 PE1069 PE1068 PE1067 PE1066 PE1065 PE1064 PE1063 PE1062 PE1061 PE1060 PE1059 PE1058 PE1057 PE1056 PE1055 PE1054 PE1053 PE1052 PE1051 PE1050 PE1049 PE1048 PE1047 PE1046 PE1045 PE1044 PE1043 PE1042 PE1041 PE1040 PE1039 PE1038 PE1037 PE1036 PE1035 PE1034 PE1033 PE1032 PE1031 PE1030 PE1029 PE1028 PE1027 PE1026 PE1025 PE1024 PE1023 PE1022 PE1021 PE1020 PE1019 PE1018 PE1017 PE1016 PE1015 PE1014 PE1013 PE1012 PE1011 PE1010 PE1009 PE1008 PE1007 PE1006 PE1005 PE1004 PE1003 PE1002 PE1001 PE1000 PE999 PE998 PE997 PE996 PE995 PE994 PE993 PE992 PE991 PE990 PE989 PE988 PE987 PE986 PE985 PE984 PE983 PE982 PE981 PE980 PE979 PE978 PE977 PE976 PE975 PE974 PE973 PE972 PE971 PE970 PE969 PE968 PE967 PE966 PE965 PE964 PE963 PE962 PE961 PE960 PE959 PE958 PE957 PE956 PE955 PE954 PE953 PE952 PE951 PE950 PE949 PE948 PE947 PE946 PE945 PE944 PE943 PE942 PE941 PE940 PE939 PE938 PE937 PE936 PE935 PE934 PE933 PE932 PE931 PE930 PE929 PE928 PE927 PE926 PE925 PE924 PE923 PE922 PE921 PE920 PE919 PE918 PE917 PE916 PE915 PE914 PE913 PE912 PE911 PE910 PE909 PE908 PE907 PE906 PE905 PE904 PE903 PE902 PE901 PE900 PE899 PE898 PE897 PE896 PE895 PE894 PE893 PE892 PE891 PE890 PE889 PE888 PE887 PE886 PE885 PE884 PE883 PE882 PE881 PE880 PE879 PE878 PE877 PE876 PE875 PE874 PE873 PE872 PE871 PE870 PE869 PE868 PE867 PE866 PE865 PE864 PE863 PE862 PE861 PE860 PE859 PE858 PE857 PE856 PE855 PE854 PE853 PE852 PE851 PE850 PE849 PE848 PE847 PE846 PE845 PE844 PE843 PE842 PE841 PE840 PE839 PE838 PE837 PE836 PE835 PE834 PE833 PE832 PE831 PE830 PE829 PE828 PE827 PE826 PE825 PE824 PE823 PE822 PE821 PE820 PE819 PE818 PE817 PE816 PE815 PE814 PE813 PE812 PE811 PE810 PE809 PE808 PE807 PE806 PE805 PE804 PE803 PE802 PE801 PE800 PE799 PE798 PE797 PE796 PE795 PE794 PE793 PE792 PE791 PE790 PE789 PE788 PE787 PE786 PE785 PE784 PE783 PE782 PE781 PE780 PE779 PE778 PE777 PE776 PE775 PE774 PE773 PE772 PE771 PE770 PE769 PE768 PE767 PE766 PE765 PE764 PE763 PE762 PE761 PE760 PE759 PE758 PE757 PE756 PE755 PE754 PE753 PE752 PE751 PE750 PE749 PE748 PE747 PE746 PE745 PE744 PE743 PE742 PE741 PE740 PE739 PE738 PE737 PE736 PE735 PE734 PE733 PE732 PE731 PE730 PE729 PE728 PE727 PE726 PE725 PE724 PE723 PE722 PE721 PE720 PE719 PE718 PE717 PE716 PE715 PE714 PE713 PE712 PE711 PE710 PE709 PE708 PE707 PE706 PE705 PE704 PE703 PE702 PE701 PE700 PE699 PE698 PE697 PE696 PE695 PE694 PE693 PE692 PE691 PE690 PE689 PE688 PE687 PE686 PE685 PE684 PE683 PE682 PE681 PE680 PE679 PE678 PE677 PE676 PE675 PE674 PE673 PE672 PE671 PE670 PE669 PE668 PE667 PE666 PE665 PE664 PE663 PE662 PE661 PE660 PE659 PE658 PE657 PE656 PE655 PE654 PE653 PE652 PE651 PE650 PE649 PE648 PE647 PE646 PE645 PE644 PE643 PE642 PE641 PE640 PE639 PE638 PE637 PE636 PE635 PE634 PE633 PE632 PE631 PE630 PE629 PE628 PE627 PE626 PE625 PE624 PE623 PE622 PE621 PE620 PE619 PE618 PE617 PE616 PE615 PE614 PE613 PE612 PE611 PE610 PE609 PE608 PE607 PE606 PE605 PE604 PE603 PE602 PE601 PE600 PE599 PE598 PE597 PE596 PE595 PE594 PE593 PE592 PE591 PE590 PE589 PE588 PE587 PE586 PE585 PE584 PE583 PE582 PE581 PE580 PE579 PE578 PE577 PE576 PE575 PE574 PE573 PE572 PE571 PE570 PE569 PE568 PE567 PE566 PE565 PE564 PE563 PE562 PE561 PE560 PE559 PE558 PE557 PE556 PE555 PE554 PE553 PE552 PE551 PE550 PE549 PE548 PE547 PE546 PE545 PE544 PE543 PE542 PE541 PE540 PE539 PE538 PE537 PE536 PE535 PE534 PE533 PE532 PE531 PE530 PE529 PE528 PE527 PE526 PE525 PE524 PE523 PE522 PE521 PE520 PE519 PE518 PE517 PE516 PE515 PE514 PE513 PE512 PE511 PE510 PE509 PE508 PE507 PE506 PE505 PE504 PE503 PE502 PE501 PE500 PE499 PE498 PE497 PE496 PE495 PE494 PE493 PE492 PE491 PE490 PE489 PE488 PE487 PE486 PE485 PE484 PE483 PE482 PE481 PE480 PE479 PE478 PE477 PE476 PE475 PE474 PE473 PE472 PE471 PE470 PE469 PE468 PE467 PE466 PE465 PE464 PE463 PE462 PE461 PE460 PE459 PE458 PE457 PE456 PE455 PE454 PE453 PE452 PE451 PE450 PE449 PE448 PE447 PE446 PE445 PE444 PE443 PE442 PE441 PE440 PE439 PE438 PE437 PE436 PE435 PE434 PE433 PE432 PE431 PE430 PE429 PE428 PE427 PE426 PE425 PE424 PE423 PE422 PE421 PE420 PE419 PE418 PE417 PE416 PE415 PE414 PE413 PE412 PE411 PE410 PE409 PE408 PE407 PE406 PE405 PE404 PE403 PE402 PE401 PE400 PE399 PE398 PE397 PE396 PE395 PE394 PE393 PE392 PE391 PE390 PE389 PE388 PE387 PE386 PE385 PE384 PE383 PE382 PE381 PE380 PE379 PE378 PE377 PE376 PE375 PE374 PE373 PE372 PE371 PE370 PE369 PE368 PE367 PE366 PE365 PE364 PE363 PE362 PE361 PE360 PE359 PE358 PE357 PE356 PE355 PE354 PE353 PE352 PE351 PE350 PE349 PE348 PE347 PE346 PE345 PE344 PE343 PE342 PE341 PE340 PE339 PE338 PE337 PE336 PE335 PE334 PE333 PE332 PE331 PE330 PE329 PE328 PE327 PE326 PE325 PE324 PE323 PE322 PE321 PE320 PE319 PE318 PE317 PE316 PE315 PE314 PE313 PE312 PE311 PE310 PE309 PE308 PE307 PE306 PE305 PE304 PE303 PE302 PE301 PE300 PE299 PE298 PE297 PE296 PE295 PE294 PE293 PE292 PE291 PE290 PE289 PE288 PE287 PE286 PE285 PE284 PE283 PE282 PE281 PE280 PE279 PE278 PE277 PE276 PE275 PE274 PE273 PE272 PE271 PE270 PE269 PE268 PE267 PE266 PE265 PE264 PE263 PE262 PE261 PE260 PE259 PE258 PE257 PE256 PE255 PE254 PE253 PE252 PE251 PE250 PE249 PE248 PE247 PE246 PE245 PE244 PE243 PE242 PE241 PE240 PE239 PE238 PE237 PE236 PE235 PE234 PE233 PE232 PE231 PE230 PE229 PE228 PE227 PE226 PE225 PE224 PE223 PE222 PE221 PE220 PE219 PE218 PE217 PE216 PE215 PE214 PE213 PE212 PE211 PE210 PE209 PE208 PE207 PE206 PE205 PE204 PE203 PE202 PE201 PE200 PE199 PE198 PE197 PE196 PE195 PE194 PE193 PE192 PE191 PE190 PE189 PE188 PE187 PE186 PE185 PE184 PE183 PE182 PE181 PE180 PE179 PE178 PE177 PE176 PE175 PE174 PE173 PE172 PE171 PE170 PE169 PE168 PE167 PE166 PE165 PE164 PE163 PE162 PE161 PE160 PE159 PE158 PE157 PE156 PE155 PE154 PE153 PE152 PE151 PE150 PE149 PE148 PE147 PE146 PE145 PE144 PE143 PE142 PE141 PE140 PE139 PE138 PE137 PE136 PE135 PE134 PE133 PE132 PE131 PE130 PE129 PE128 PE127 PE126 PE125 PE124 PE123 PE122 PE121 PE120 PE119 PE118 PE117 PE116 PE115 PE114 PE113 PE112 PE111 PE110 PE109 PE108 PE107 PE106 PE105 PE104 PE103 PE102 PE101 PE100 PE99 PE98 PE97 PE96 PE95 PE94 PE93 PE92 PE91 PE90 PE89 PE88 PE87 PE86 PE85 PE84 PE83 PE82 PE81 PE80 PE79 PE78 PE77 PE76 PE75 PE74 PE73 PE72 PE71 PE70 PE69 PE68 PE67 PE66 PE65 PE64 PE63 PE62 PE61 PE60 PE59 PE58 PE57 PE56 PE55 PE54 PE53 PE52 PE51 PE50 PE49 PE48 PE47 PE46 PE45 PE44 PE43 PE42 PE41 PE40 PE39 PE38 PE37 PE36 PE35 PE34 PE33 PE32 PE31 PE30 PE29 PE28 PE27 PE26 PE25 PE24 PE23 PE22 PE21 PE20 PE19 PE18 PE17 PE16 PE15 PE14 PE13 PE12 PE11 PE10 PE9 PE8 PE7 PE6 PE5 PE4 PE3 PE2 PE1 PE0 bit_1))
			)
;----------------P to Q pipeline step------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q (pipeline P0 datamemory regfile noflush))
			)
		(let
			(
				(Q_r (regfilewriting_pipeline Q regfile))
				(Q_d (datamemorywriting_pipeline Q datamemory))
			)
	
;----------------Q to Qf flush 1-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q1 (pipeline Q Q_d Q_r flush))
			)
		(let
			(
				(Q1_r (regfilewriting_pipeline Q1 Q_r))
				(Q1_d (datamemorywriting_pipeline Q1 Q_d))
			)
		
;----------------Q to Qf flush 2-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q2 (pipeline Q1 Q1_d Q1_r flush))
			)
		(let
			(
				(Q2_r (regfilewriting_pipeline Q2 Q1_r))
				(Q2_d (datamemorywriting_pipeline Q2 Q1_d))
			)

;----------------Q to Qf flush 3-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q3 (pipeline Q2 Q2_d Q2_r flush))
			)
		(let
			(
				(Q3_r (regfilewriting_pipeline Q3 Q2_r))
				(Q3_d (datamemorywriting_pipeline Q3 Q2_d))
			)
			
;----------------Q to Qf flush 4-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q4 (pipeline Q3 Q3_d Q3_r flush))
			)
		(let
			(
				(Q4_r (regfilewriting_pipeline Q4 Q3_r))
				(Q4_d (datamemorywriting_pipeline Q4 Q3_d))
			)
			
;----------------Q to Qf flush 5-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q5 (pipeline Q4 Q4_d Q4_r flush))
			)
		(let
			(
				(Q5_r (regfilewriting_pipeline Q5 Q4_r))
				(Q5_d (datamemorywriting_pipeline Q5 Q4_d))
			)
			
;----------------Q to Qf flush 6-----------------------------------------------------------------------------------------------------------------
		(let
			(
				(Q6 (pipeline Q5 Q5_d Q5_r flush))
			)
		(let
			(
				(Q6_r (regfilewriting_pipeline Q6 Q5_r))
				(Q6_d (datamemorywriting_pipeline Q6 Q5_d))
			)
			
;--------------Q to Qf flush 7----------------
         (let
			(
				(Qf (pipeline Q6 Q6_d Q6_r flush))
			)
		(let
			(
				(Qf_r (regfilewriting_pipeline Qf Q6_r))
				(Qf_d (datamemorywriting_pipeline Qf Q6_d))
			)
			
;-----------------------------------------------------
						

;----------------P to I flush 1------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P1 (pipeline P1_1 datamemory regfile noflush))
			)
		(let
			(
				(P1_r (regfilewriting_pipeline P1 regfile))
				(P1_d (datamemorywriting_pipeline P1 datamemory))
			)
;----------------P to I flush 2------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P2 (pipeline P1 P1_d P1_r flush))
			)
		(let
			(
				(P2_r (regfilewriting_pipeline P2 P1_r))
				(P2_d (datamemorywriting_pipeline P2 P1_d))
			)

;----------------P to I flush 3------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P3 (pipeline P2 P2_d P2_r flush))
			)
		(let
			(
				(P3_r (regfilewriting_pipeline P3 P2_r))
				(P3_d (datamemorywriting_pipeline P3 P2_d))
			)
;----------------P to I flush 4------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P4 (pipeline P3 P3_d P3_r flush))
			)
		(let
			(
				(P4_r (regfilewriting_pipeline P4 P3_r))
				(P4_d (datamemorywriting_pipeline P4 P3_d))
			)
;----------------P to I flush 5------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P5 (pipeline P4 P4_d P4_r flush))
			)
		(let
			(
				(P5_r (regfilewriting_pipeline P5 P4_r))
				(P5_d (datamemorywriting_pipeline P5 P4_d))
			)
;----------------P to I flush 6------------------------------------------------------------------------------------------------------------------
		(let
			(
				(P6 (pipeline P5 P5_d P5_r flush))
			)
		(let
			(
				(P6_r (regfilewriting_pipeline P6 P5_r))
				(P6_d (datamemorywriting_pipeline P6 P5_d))
			)

;----------------P to I flush 7------------------------------------------------------------------------------------------------------------------
		(let
			(
				(I (pipeline P6 P6_d P6_r flush))
			)
		(let
			(
				(I_r (regfilewriting_pipeline I P6_r))
				(I_d (datamemorywriting_pipeline I P6_d))
			)
			
;----------------------------------------------------------------------------------------------------------------------------
;------------------property------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------			
		(let
			(	
				(Qf_pccounter ((_ extract 1100 1037) Qf))
				(I_pccounter ((_ extract 1100 1037) I))
			)	
		(=>
			(and (= flush (_ bv1 1)) (= noflush (_ bv0 1)) (= bit_0 (_ bv0 1)) (= bit_1 (_ bv1 1)))
			
			(and (= Qf_d I_d) 
				 (= I_pccounter Qf_pccounter)
				 (= Qf_r I_r)) 		
			)
		)))))))))))))))))))))))))))))))))
	)
)

(check-sat)
(get-model)
